import os

from launch import LaunchDescription
from launch.actions import DeclareLaunchArgument
from launch.substitutions import LaunchConfiguration, TextSubstitution

from launch_ros.actions import Node

from launch.actions import GroupAction
from launch_ros.actions import PushRosNamespace
from launch_ros.actions import SetParameter
from launch.actions import IncludeLaunchDescription
from launch.launch_description_sources import PythonLaunchDescriptionSource
from launch.conditions import IfCondition


def generate_launch_description():
    ns_arg = DeclareLaunchArgument("ns", default_value=TextSubstitution(text="/dji0"))
    port_arg = DeclareLaunchArgument("port", default_value=TextSubstitution(text="8883"))
    host_arg = DeclareLaunchArgument("host", default_value=TextSubstitution(text="broker.waraps.org"))
    user_arg = DeclareLaunchArgument("user", default_value=TextSubstitution(text="mqtt"))
    passwd_arg = DeclareLaunchArgument("password", default_value=TextSubstitution(text="Check"))
    expand_arg = DeclareLaunchArgument("expand", default_value='false')
    
    return LaunchDescription([
        ns_arg,        
        port_arg,
        host_arg,
        user_arg,
        passwd_arg,
        expand_arg,
#        Node(
#            package='lrs_exec',
#            output='screen',
#            emulate_tty=True,            
#            namespace=LaunchConfiguration('ns'),
#            executable='conc'
#        ),
        Node(
            package='lrs_exec',
            output='screen',
            emulate_tty=True,            
            namespace=LaunchConfiguration('ns'),
            executable='noop'
        ),
        Node(
            package='lrs_exec',
            output='screen',
            emulate_tty=True,            
            namespace=LaunchConfiguration('ns'),
            executable='yes_exec'
        ),
        Node(
            package='lrs_exec',
            output='screen',
            emulate_tty=True,            
            namespace=LaunchConfiguration('ns'),
            executable='say'
        ),
        Node(
            package='lrs_exec',
            output='screen',
            emulate_tty=True,            
            namespace=LaunchConfiguration('ns'),
            executable='monitor'
        ),
        Node(
            package='lrs_exec',
            output='screen',
            emulate_tty=True,            
            namespace=LaunchConfiguration('ns'),
            executable='string_pub'
        ),
        Node(
            package='lrs_exec',
            output='screen',
            emulate_tty=True,            
            namespace=LaunchConfiguration('ns'),
            executable='add_task_to_team_queue'
        ),
        Node(
            package='lrs_exec',
            output='screen',
            emulate_tty=True,            
            namespace=LaunchConfiguration('ns'),
            executable='input_handler'
        ),
        Node(
            package='lrs_exec',
            output='screen',
            emulate_tty=True,            
            namespace=LaunchConfiguration('ns'),
            executable='fail'
        ),
        Node(
            package='lrs_exec',
            output='screen',
            emulate_tty=True,            
            namespace=LaunchConfiguration('ns'),
            executable='collect_release',
            condition=IfCondition(LaunchConfiguration('expand')),            
        ),
        Node(
            package='lrs_exec',
            output='screen',
            emulate_tty=True,            
            namespace=LaunchConfiguration('ns'),
            executable='look_at_position',
            condition=IfCondition(LaunchConfiguration('expand')),
        ),
        Node(
            package='lrs_exec',
            output='screen',
            emulate_tty=True,            
            namespace=LaunchConfiguration('ns'),
            executable='send_signal'
        ),
#        Node(
#            package='lrs_exec',
#            output='screen',
#            emulate_tty=True,            
#            namespace=LaunchConfiguration('ns'),
#            executable='seq'
#        ),
#        Node(
#            package='lrs_exec',
#            output='screen',
#            emulate_tty=True,            
#            namespace=LaunchConfiguration('ns'),
#            executable='start_data_recording'
#        ),
#        Node(
#            package='lrs_exec',
#            output='screen',
#            emulate_tty=True,            
#            namespace=LaunchConfiguration('ns'),
#            executable='stop_data_recording'
#        ),
        Node(
            package='lrs_exec',
            output='screen',
            emulate_tty=True,            
            namespace=LaunchConfiguration('ns'),
            executable='supp'
        ),
#        Node(
#            package='lrs_exec',
#            output='screen',
#            emulate_tty=True,            
#            namespace=LaunchConfiguration('ns'),
#            executable='test_if'
#        ),
        Node(
            package='lrs_exec',
            output='screen',
            emulate_tty=True,            
            namespace=LaunchConfiguration('ns'),
            executable='wait'
        ),
        Node(
            package='lrs_exec',
            output='screen',
            emulate_tty=True,            
            namespace=LaunchConfiguration('ns'),
            executable='wait_joy'
        ),
        Node(
            package='lrs_exec',
            output='screen',
            emulate_tty=True,
            namespace=LaunchConfiguration('ns'),
            executable='query'
        ),
        Node(
            package='lrs_exec',
            output='screen',
            emulate_tty=True,
            namespace=LaunchConfiguration('ns'),
            executable='computation'
        ),
        Node(
            package='lrs_exec',
            output='screen',
            emulate_tty=True,            
            namespace=LaunchConfiguration('ns'),
            executable='remote',
            parameters=[{
                "waraps_mqtt_password": LaunchConfiguration('password'),
                "waraps_mqtt_user": LaunchConfiguration('user'),
                "mqtt_port": LaunchConfiguration('port'),
                "mqtt_host": LaunchConfiguration('host'),
            }
                        ],
        ),
    ])

# Alt: prefix=['stdbuf -o L']
