import os
import uuid

from ament_index_python.packages import get_package_share_directory

from launch import LaunchDescription
from launch.actions import DeclareLaunchArgument
from launch.substitutions import LaunchConfiguration, TextSubstitution
from launch.conditions import IfCondition
from launch.actions import IncludeLaunchDescription
from launch.launch_description_sources import PythonLaunchDescriptionSource

from launch_ros.actions import Node

def generate_launch_description():
    ns_arg = DeclareLaunchArgument("ns", default_value=TextSubstitution(text="/dji0"))
    location_arg = DeclareLaunchArgument("location", default_value=TextSubstitution(text="granso"))
    interaction_arg = DeclareLaunchArgument("interaction", default_value='false')
    basic_executors_arg = DeclareLaunchArgument("basic_executors", default_value='true')
    tree_executor_arg = DeclareLaunchArgument("tree_executor", default_value='true')
    external_arg = DeclareLaunchArgument("external", default_value='false')
    expand_arg = DeclareLaunchArgument("expand", default_value='false')    
    remote_arg = DeclareLaunchArgument("remote", default_value='false')
    ros2_bridge_arg = DeclareLaunchArgument("ros2_bridge", default_value='false')
    
    basic_executors_include = IncludeLaunchDescription(
        PythonLaunchDescriptionSource(
            os.path.join(
                get_package_share_directory('lrs_exec'),
                'basic_executors.launch.py')),
        launch_arguments={"ns": LaunchConfiguration('ns'),
                          "remote": LaunchConfiguration('remote'),
                          "expand": LaunchConfiguration('expand')}.items(),
        condition=IfCondition(LaunchConfiguration('basic_executors')),        
    )

    return LaunchDescription([
        ns_arg,
        location_arg,
        interaction_arg,
        external_arg,
        expand_arg,
        basic_executors_arg,
        tree_executor_arg,
        remote_arg,
        ros2_bridge_arg,
        basic_executors_include,
        Node(
            package='lrs_exec',
            output='screen',
            emulate_tty=True,            
            namespace=LaunchConfiguration('ns'),
            executable='resource',
            name='resource_agent',
            parameters=[{
                "external": LaunchConfiguration('external'),
            }]
        ),
        Node(
            package='lrs_exec',
            output='screen',
            emulate_tty=True,            
            namespace=LaunchConfiguration('ns'),
            executable='tst_exec_factory',
            name='tst_exec_factory',
            parameters=[{
                "external": LaunchConfiguration('external'),
            }]
        ),
        Node(
            package='lrs_exec',
            output='screen',
            emulate_tty=True,            
            namespace=LaunchConfiguration('ns'),
            executable='tst_exec_manager',
            name='tst_exec_manager'),
        Node(
            package='lrs_exec',
            output='screen',
            emulate_tty=True,            
            namespace=LaunchConfiguration('ns'),
            executable='tree_executor',
            name='tree_executor',
            condition=IfCondition(LaunchConfiguration('tree_executor')),        
        ),
        Node(
            package='lrs_wdb',
            output='screen',
            emulate_tty=True,            
            namespace=LaunchConfiguration('ns'),
            executable='wdb',
            name='wdb',
            arguments=["--ros-args", "--log-level", "error"],            
            parameters=[{"external": LaunchConfiguration('external'),
                         "location": LaunchConfiguration('location')
            }]),
#        Node(
#            package='lrs_exec',
#            output='screen',
#            emulate_tty=True,            
#            namespace=LaunchConfiguration('ns'),
#            executable='tst_expansion',
#            name='tst_expansion',
#            parameters=[{
#                "external": LaunchConfiguration('external'),
#            }]
#        ),
        Node(
            package='lrs_interaction',
            output='screen',
            emulate_tty=True,            
            namespace=LaunchConfiguration('ns'),
            executable='gateway',
            condition=IfCondition(LaunchConfiguration('interaction')),
            name='interaction_gateway'),
        Node(
            package='lrs_exec',
            output='screen',
            emulate_tty=True,            
            namespace=LaunchConfiguration('ns'),
            executable='exec_server',
            name='exec_server',
            parameters=[{

            }
            ]
        ),
        Node(
            package='lrs_exec',
            output='screen',
            emulate_tty=True,            
            namespace=LaunchConfiguration('ns'),
            executable='ros2_bridge',
            name='ros2_bridge',
            parameters=[{

            }
                        ],
            condition=IfCondition(LaunchConfiguration('ros2_bridge')),                    
        )
        
    ])

# Alt: prefix=['stdbuf -o L']
