import json
import uuid

from lrs_msgs_tst.msg import TSTUpdateNodeInfo, TSTExecStatus, TSTNodeInfo, TSTTimeParam, TSTNodeStatus
from lrs_exec.exec_status import ExecStatus
from lrs_exec.exec_info import ExecInfo
from lrs_msgs_tst.msg import TSTBoolParam, TSTFloat64Param, TSTFloat64sParam, TSTGeoPointArraysParam
from lrs_msgs_tst.msg import TSTGeoPointParam, TSTGeoPointsParam, TSTInt32Param, TSTPointParam, TSTPointsParam
from lrs_msgs_tst.msg import TSTPoseParam, TSTStringParam, TSTStringsParam, TSTParamSpec, TSTTimeInfo
from geographic_msgs.msg import GeoPoint
import builtin_interfaces

class ExecTstNode():
    def __init__(self):
        self.name = ""
        self.factory_local_id = 0
        self.root_id = 0
        self.node_uuid = ""
        self.root_uuid = ""
        self.execution_ns = ""
        self.delegation_ns = ""
        self.resources_to_lock = []
        self.delexec_resources_to_lock = []
        self.wait_for_finished = []
        self.csp_json = ""
        self.csp_constraints = []
        self.csp_variables = []
        self.execution_ns_alias = ""
        self.abort_when_starting = False
        self.enough_when_starting = False
        self.use_delegation_lock = False
        self.use_execution_lock = False
        self.approved = False
        self.wait_for_stime = True
        self.wait_for_etime = True
        self.params = {}
        self.children = []
        self.exec_status = ExecStatus()
        self.exec_info = ExecInfo()
        self.unique_node_id = -1
        self.tree_distributed_to = []
        self.start_executor = False
        self.stime_lb = -1
        self.stime_ub = -1
        self.etime_lb = -1
        self.etime_ub = -1
        self.duration_lb = -1
        self.duration_ub = -1
        self.tree_start_time = None

    def set_tree_start_time(self, time):
        self.tree_start_time = time

    def is_executing(self):
        return self.exec_status.executing and not self.exec_status.finished

    def get_unexecuted_terminals(self):
        res = []
        if len(self.children) == 0 and  not self.exec_status.executing:
            res.append(self.node_uuid)
        for c in self.children:
            res = [*res, *c.get_unexecuted_terminals()]
        return res
        
    def get_executing_abortable_nodes(self):
        ## print("get_executing_abortable_nodes")
        res = []
        if self.exec_info.can_be_aborted and self.is_executing():
            res.append(self.node_uuid)
        for c in self.children:
            res = [*res, *c.get_executing_abortable_nodes()]
        return res
        
    def get_executing_abortable_nodes_on_unit(self, unit):
        res = []
        unit = unit.lstrip("/")
        if self.exec_info.can_be_aborted and self.is_executing():
            if self.get_exec_ns() == f'/{unit}':
                res.append(self.node_uuid)
        for c in self.children:
            res = [*res, *c.get_executing_abortable_nodes_on_unit(unit)]
        return res
        
    def get_executing_enoughable_nodes(self):
        res = []
        if self.exec_info.can_be_enoughed and self.is_executing():
            res.append(self.node_uuid)
        for c in self.children:
            res = [*res, *c.get_executing_enoughable_nodes()]
        return res
        
    def get_executing_enoughable_nodes_on_unit(self, unit):
        res = []
        unit = unit.lstrip("/")
        if self.exec_info.can_be_enoughed and self.is_executing():
            if self.get_exec_ns() == f'/{unit}':
                res.append(self.node_uuid)
        for c in self.children:
            res = [*res, *c.get_executing_enoughable_nodes_on_unit(unit)]
        return res
        
    def get_executing_pauseable_nodes(self):
        res = []
        if self.exec_info.can_be_paused and self.is_executing():
            res.append(self.node_uuid)
        for c in self.children:
            res = [*res, *c.get_executing_pauseable_nodes()]
        return res
        
    def get_executing_pauseable_nodes_on_unit(self, unit):
        res = []
        unit = unit.lstrip("/")
        if self.exec_info.can_be_paused and self.is_executing():
            if self.get_exec_ns() == f'/{unit}':
                res.append(self.node_uuid)
        for c in self.children:
            res = [*res, *c.get_executing_pauseable_nodes_on_unit(unit)]
        return res
        
    def update(self, msg):
        try:
            # print("ExecTstNode update:", msg)
            self.start_executor = msg.start_executor
            #self.tree_start_time = msg.tree_start_time
            self.approved = msg.approved
            self.abort_when_starting = msg.abort_when_starting
            self.enough_when_starting = msg.enough_when_starting
            self.use_delegation_lock = msg.use_delegation_lock
            self.use_execution_lock = msg.use_execution_lock
            self.exec_status.update(msg.exec_status)
            self.exec_info.update(msg.exec_info)
        except Exception as exc:
            print("EXCEPTION update", type(exc))
            print(exc)
        
    def get_status_msg(self):
        msg = TSTNodeStatus()
        try:
            msg.ns = ""
            msg.id = self.get_factory_local_id()
            msg.root_id = self.root_id
            msg.node_uuid = self.get_node_uuid()
            msg.unique_node_id = self.unique_node_id
            msg.type = self.get_type()
            msg.exec_status = self.exec_status.get_exec_status_msg()
            msg.exec_info = self.exec_info.get_exec_info_msg()
        except Exception as exc:
            print("EXCEPTION get_status_msg", type(exc))
            print(exc)
        return msg

    def get_update_node_info_msg(self):
        msg = TSTUpdateNodeInfo()
        try:
            msg.group = ""
            msg.node_uuid = self.node_uuid
            # msg.tree_start_time
            msg.approved = self.approved
            msg.abort_when_starting = self.abort_when_starting
            msg.enough_when_starting = self.enough_when_starting
            msg.use_delegation_lock = self.use_delegation_lock
            msg.use_execution_lock = self.use_execution_lock
            msg.start_executor = self.start_executor
            msg.exec_status = self.exec_status.get_exec_status_msg()
            msg.exec_info = self.exec_info.get_exec_info_msg()
            msg.sender = ""
        except Exception as exc:
            print("EXCEPTION get_update_node_info_msg", type(exc))
            print(exc)
        return msg

    def get_exec_ns(self):
        return self.execution_ns

    def get_factory_local_id(self):
        return self.factory_local_id

    def get_type(self):
        return self.name

    def set_node_parameter(self, name, val):
        self.params[name] = val

    def set_start_executor(self, val):
        # print("set_start_executor:", self.get_type(), val)
        self.start_executor = val

    def set_distribute_to(self, units):
        self.tree_distributed_to = units

    def get_distribute_to(self):
        return self.tree_distributed_to

    def get_node_uuid(self):
        return self.node_uuid

    def parse_node_uuid(self, jobj, approve=True):
        if "node-uuid" in jobj:
            return jobj["node-uuid"]
        if "node_uuid" in jobj:
            return jobj["node_uuid"]
        if "common-params" in jobj:
            return self.parse_node_uuid(jobj["common-params"])
        if "common_params" in jobj:
            return self.parse_node_uuid(jobj["common_params"])
        # print("node uuid not set, creating it")
        return str(uuid.uuid4())

    def parse_params(self, jobj):
        for k,v in jobj.items():
            k = k.replace("-", "_")
            # print("parse_params:", k, v)
            if k == "tree_start_time":
                self.tree_start_time = builtin_interfaces.msg.Time()
                self.tree_start_time.sec = v["sec"]
                self.tree_start_time.nanosec = v["nanosec"]
                continue
            if k == "stime_lb":
                self.stime_lb = v
                continue
            if k == "stime_ub":
                self.stime_ub = v
                continue
            if k == "etime_lb":
                self.etime_lb = v
                continue
            if k == "etime_ub":
                self.etime_ub = v
                continue
            if k == "duration_lb":
                self.duration_lb = v
                continue
            if k == "duration_ub":
                self.duration_ub = v
                continue
            if k == "wait_for_stime":
                self.wait_for_stime = v
                continue
            if k == "wait_for_etime":
                self.wait_for_etime = v
                continue
            if k == "enough_when_starting":
                self.enough_when_starting = v
                continue
            if k == "abort_when_starting":
                self.abort_when_starting = v
                continue
            if k == "unique_node_id":
                self.unique_node_id = v
                continue
            if k == "execution_ns_alias":
                self.execution_ns_alias = v
                continue
            if k == "execunitalias":
                self.execution_ns_alias = v
                continue
            if k == "execution_ns":
                self.execution_ns = v
                continue
            if k == "execution_unit":
                self.execution_ns = v
                continue
            if k == "execunit":
                self.execution_ns = v
                continue
            if k == "delegation_ns":
                self.delegation_ns = v
                continue
            if k == "delegation_unit":
                self.delegation_ns = v
                continue
            if k == "delunit":
                self.delegation_ns = v
                continue
            if k == "approved":
                self.approved = v
                continue
            if k == "abort_when_starting":
                self.abort_when_starting = v
                continue
            if k == "enough_when_starting":
                self.enough_when_starting = v
                continue
            if k == "use_delegation_lock":
                self.use_delegation_lock = v
                continue
            if k == "use_execution_lock":
                self.use_execution_lock = v
                continue
            if k == "node_uuid":
                if v != "":
                    self.node_uuid = v
                continue
            if k == "root_uuid":
                if v != "":
                    self.root_uuid = v
                continue
            # util.cc : all_same, all_different, delexec_resources_to_lock, allowed_units,
            # wait_for_finished
            self.params[k] = v
    
    def create_tree(self, jobj, free_id, factory_local_id, root_id, approve=True):
        try:
            # print("create_tree:", json.dumps(jobj, sort_keys=True, indent=4, separators=(',', ': ')))
            if not "name" in jobj:
                print("name is not in node")
                return (False, free_id, factory_local_id)
            self.name = jobj["name"]
            self.node_uuid = self.parse_node_uuid(jobj, approve=approve)
            self.factory_local_id = factory_local_id
            self.root_id = root_id
            factory_local_id += 1
            self.approved = approve

            #
            # Top level things
            #
        
            if "unique_node_id" in jobj:
                self.unique_node_id = jobj["unique_node_id"]

            if "execution_ns" in jobj:
                self.execution_ns = jobj["execution_ns"]

            if "root_uuid" in jobj:
                self.root_uuid = jobj["root_uuid"]

            if "approved" in jobj:
                if jobj["approved"]:
                    self.approved = True
                else:
                    self.approved = False
            #
            # Top level things can be overridden by parameters...
            #

            if "tree_start_time" in jobj:
                v = jobj["tree_start_time"]
                self.tree_start_time = builtin_interfaces.msg.Time()
                self.tree_start_time.sec = v["sec"]
                self.tree_start_time.nanosec = v["nanosec"]
        
            if "common-params" in jobj:
                self.parse_params(jobj["common-params"])

            if "common_params" in jobj:
                self.parse_params(jobj["common_params"])

            if "params" in jobj:
                self.parse_params(jobj["params"])

            if self.unique_node_id < 0:
                self.unique_node_id = free_id
                free_id += 1
            

            # get paremeters default values from config...
            # resources from config
            # util.cc is manually adding parameters, strange...

            if "children" in jobj:
                for cobj in jobj["children"]:
                    child = ExecTstNode()
                    (res, free_id, factory_local_id) = child.create_tree(cobj, free_id, factory_local_id, root_id, approve=approve)
                    self.children.append(child)
                    if not res:
                        return (False, free_id, factory_local_id)
            return (True, free_id, factory_local_id)
        except Exception as exc:
            print(f'EXCEPTION create_tree: {type(exc)} - {exc}')
        
    
    def get_json_obj(self):
        jobj = {
            "name": self.name,
            "id": self.factory_local_id,
            "node_uuid": self.node_uuid,
            "root_uuid": self.root_uuid,
            "approved": self.approved,
            "execution_ns": self.execution_ns,
            "delegation_ns": self.delegation_ns,
            "resources_to_lock": self.resources_to_lock,
            "delexec_resources_to_lock": self.delexec_resources_to_lock,
            "wait_for_finished": self.wait_for_finished,
            "wait_for_stime": self.wait_for_stime,
            "wait_for_etime": self.wait_for_etime,
            "enough_when_starting": self.enough_when_starting,
            "abort_when_starting": self.abort_when_starting,
            "csp_json": self.csp_json,
            "csp_constraints": self.csp_constraints,
            "csp_variables": self.csp_variables,
            "execution_ns_alias": self.execution_ns_alias,
            "unique_node_id": self.unique_node_id,
            "children": [],
            "params": self.params
        }
        if self.stime_lb >= 0:
            jobj["stime_lb"] = self.stime_lb
        if self.stime_ub >= 0:
            jobj["stime_ub"] = self.stime_ub
        if self.tree_start_time:
            jobj["tree_start_time"] = {
                "sec": self.tree_start_time.sec,
                "nanosec": self.tree_start_time.nanosec
            }
            
        for child in self.children:
            jobj["children"].append(child.get_json_obj())
        return jobj

    def get_time_info(self):
        msg = TSTTimeInfo()
        msg.not_valid = True
        return msg

    def get_time_param(self, name, val):
        res = TSTTimeParam()
        if val >= 0:
            res.name = name
            res.value.sec = val
            res.value.nanosec = 0
            res.have_value = True
        return res
        
    def get_node_info(self):
        msg = TSTNodeInfo()
        # print("exec_tst_node get_node_info")
        try:
            msg.not_valid = False
            msg.ns = ""
            msg.factory_local_id = self.get_factory_local_id()
            msg.root_ns = ""
            msg.parent_id = -1
            msg.name = self.name
            msg.type = self.name
            msg.unique_node_id = self.unique_node_id
            msg.root_uuid = ""
            msg.node_uuid = self.node_uuid
            msg.delegation_ns = ""
            msg.prefer_children_on_different_units = False
            msg.stime_lb = self.get_time_param("stime_lb", self.stime_lb)
            msg.stime_ub = self.get_time_param("stime_ub", self.stime_ub)
            msg.etime_lb = self.get_time_param("etime_lb", self.etime_lb)
            msg.etime_ub = self.get_time_param("etime_ub", self.etime_ub)
            msg.duration_lb = self.get_time_param("duration_lb", self.duration_lb)
            msg.duration_ub = self.get_time_param("duration_ub", self.stime_ub)
            if self.tree_start_time:
                msg.tree_start_time = self.tree_start_time
            # msg.node_start_time
            # msg.have_node_start_time
            msg.resources = []
            msg.resources_to_lock = self.resources_to_lock
            msg.delexec_resources_to_lock = self.delexec_resources_to_lock
            msg.use_delegation_lock = self.use_delegation_lock
            msg.use_execution_lock = self.use_execution_lock
            msg.wait_for_finished = self.wait_for_finished
            msg.csp_json = ""
            msg.csp_variables = []
            msg.csp_constraints = []
            msg.allowed_units = []
            msg.execution_ns_alias = self.execution_ns_alias
            msg.all_same = []
            msg.all_different = []
            msg.string_params = self.get_string_params()
            msg.strings_params = self.get_strings_params()
            msg.bool_params = self.get_bool_params()
            msg.int32_params = self.get_int32_params()
            msg.float64_params = self.get_float64_params()
            msg.float64s_params = self.get_float64s_params()
            msg.geo_point_params = self.get_geo_point_params()
            msg.geo_points_params = self.get_geo_points_params()
            msg.geo_point_arrays_params = self.get_geo_point_arrays_params()
            msg.point_params = self.get_point_params()
            msg.points_params = self.get_points_params()
            msg.pose_params = self.get_pose_params()
            msg.children = self.get_children_ids()

            msg.start_executor = self.start_executor
            msg.approved = self.approved
            msg.execution_ns = self.execution_ns
            msg.wait_for_stime = self.wait_for_stime
            msg.wait_for_etime = self.wait_for_etime
            msg.abort_when_starting = self.abort_when_starting
            msg.enough_when_starting = self.enough_when_starting
            msg.exec_status = self.exec_status.get_exec_status_msg()
            msg.exec_info = self.exec_info.get_exec_info_msg()
        except Exception as exc:
            print("EXCEPTION get_node_info", type(exc))
            print(exc)
        return msg

    def get_exec_status_msg(self):
        return self.exec_status.get_exec_status_msg()

    def get_string_params(self):
        msg = []
        try:
            for k,v in self.params.items():
                # print("GET_STRING_PARAMS:", k, type(v), v)
                if isinstance(v, str):
                    p = TSTStringParam()
                    p.name = k
                    p.value = v
                    p.have_value = True
                    p.have_default_value = False
                    msg.append(p)
        except Exception as exc:
            print("EXCEPTION get_string_params", type(exc))
            print(exc)
        return msg
    
    def get_strings_params(self):
        msg = []
        try:
            for k,v in self.params.items():
                if isinstance(v, list):
                    if len(v)>0:
                        if isinstance(v[0], str):
                            p = TSTStringsParam()
                            p.name = k
                            for el in v:
                                p.value.append(el)
                            p.have_value = True
                            p.have_default_value = False
                            msg.append(p)
        except Exception as exc:
            print("EXCEPTION get_strings_params", type(exc))
            print(exc)
        return msg
    
    def get_bool_params(self):
        msg = []
        try:
            for k,v in self.params.items():
                if isinstance(v, bool):
                    p = TSTBoolParam()
                    p.name = k
                    p.value = v
                    p.have_value = True
                    p.have_default_value = False
                    msg.append(p)
        except Exception as exc:
            print("EXCPETION get_bool_params", type(exc))
            print(exc)
        return msg
    
    def get_int32_params(self):
        msg = []
        try:
            for k,v in self.params.items():
                if type(v) is int:
                    p = TSTInt32Param()
                    p.name = k
                    p.value = v
                    p.have_value = True
                    p.have_default_value = False
                    msg.append(p)
        except Exception as exc:
            print("EXCEPTION get_int32_params", type(exc))
            print(exc)
        return msg
    
    def get_float64_params(self):
        msg = []
        try:
            for k,v in self.params.items():
                if isinstance(v, float):
                    p = TSTFloat64Param()
                    p.name = k
                    p.value = float(v)
                    p.have_value = True
                    p.have_default_value = False
                    msg.append(p)
        except Exception as exc:
            print("EXCEPTION get_float64_params", type(exc))
            print(exc)
        return msg
    
    def get_float64s_params(self):
        msg = []
        try:
            for k,v in self.params.items():
                if isinstance(v, list):
                    if len(v)>0:
                        if isinstance(v[0], float):
                            p = TSTFloat64sParam()
                            p.name = k
                            p.value = v
                            p.have_value = True
                            p.have_default_value = False
                            p.default_value = []
                            msg.append(p)
        except Exception as exc:
            print("EXCEPTION get_float64s_params", type(exc))
            print(exc)
        return msg
    
    def get_geo_point_params(self):
        msg = []
        try:
            for k,v in self.params.items():
                if isinstance(v, dict):
                    # print("DICT:", k)
                    if v["rostype"] == "GeoPoint":
                        # print("DICT VAL:", v)
                        p = TSTGeoPointParam()
                        p.name = k
                        p.value.latitude = float(v["latitude"])
                        p.value.longitude = float(v["longitude"])
                        p.value.altitude = float(v["altitude"])
                        p.have_value = True
                        p.have_default_value = False
                        msg.append(p)
        except Exception as exc:
            print(f'EXCEPTION get_geo_point_params: {type(exc)}')
            print(exc)
        return msg
    
    def get_geo_points_params(self):
        msg = []
        try:
            for k,v in self.params.items():
                ## print(f"get_geo_points_params: {k} - {v}")
                if isinstance(v, list):
                    if len(v)>0:
                        if not isinstance(v[0], str) and not isinstance(v[0], float) and v[0]["rostype"] == "GeoPoint":
                            p = TSTGeoPointsParam()
                            p.name = k
                            for el in v:
                                geo = GeoPoint()
                                geo.latitude = float(el["latitude"])
                                geo.longitude = float(el["longitude"])
                                geo.altitude = float(el["altitude"])
                                p.value.append(geo)
                            p.have_value = True
                            p.have_default_value = False
                            p.default_value = []
                            msg.append(p)
        except Exception as exc:
            print("EXCEPTION exec_tst_node.py get_geo_points_params", type(exc))
            print(exc)
        return msg
    
    def get_geo_point_arrays_params(self):
        msg = []
        p = TSTGeoPointArraysParam()
        return msg
    
    def get_point_params(self):
        msg = []
        p = TSTPointParam()
        return msg
    
    def get_points_params(self):
        msg = []
        p = TSTPointsParam()
        return msg
    
    def get_pose_params(self):
        msg = []
        try:
            for k,v in self.params.items():
                if isinstance(v, dict):
                    # print("DICT:", k)
                    if v["rostype"] == "PoseStamped":
                        # print("DICT VAL:", v)
                        p = TSTPoseParam()
                        p.name = k
                        p.value.header.frame_id = v["header"]["frame_id"]
                        # p.value.header.stamp
                        p.value.pose.orientation.x = float(v["pose"]["orientation"]["x"])
                        p.value.pose.orientation.y = float(v["pose"]["orientation"]["y"])
                        p.value.pose.orientation.z = float(v["pose"]["orientation"]["z"])
                        p.value.pose.orientation.w = float(v["pose"]["orientation"]["w"])
                        p.value.pose.position.x = float(v["pose"]["position"]["x"])
                        p.value.pose.position.y = float(v["pose"]["position"]["y"])
                        p.value.pose.position.z = float(v["pose"]["position"]["z"])
                        p.have_value = True
                        p.have_default_value = False
                        msg.append(p)
        except Exception as exc:
            print(f'EXCEPTION get_pose_params: {type(exc)}')
            print(exc)
        return msg
    
    def get_children_ids(self):
        children = []
        for c in self.children:
            children.append(c.get_factory_local_id())
        return children

    def set_exec_info(self, val):
        self.exec_info.set_exec_info(val)

    def set_active_flag(self, val):
        self.exec_status.active = val

    def set_waiting_flag(self, val):
        self.exec_status.waiting = val
        
    def set_executing_flag(self, val):
        self.exec_status.executing = val

    def set_finished_flag(self, val):
        self.exec_status.finished = val

    def set_was_finished_flag(self, val):
        self.exec_status.was_finished = val

    def set_succeeded_flag(self, val):
        self.exec_status.succeeded = val

    def set_aborted_flag(self, val):
        self.exec_status.aborted = val

    def set_paused_flag(self, val):
        self.exec_status.paused = val

    def set_revoked_flag(self, val):
        self.exec_status.revoked = val

    def dot_node_name(self):
        return f'node_{self.root_id}_{self.factory_local_id}'
        #return f'node_{self.root_id}_{self.unique_node_id}'

    def get_label_name(self):
        res = self.get_type()
        if res == "seq":
            res = "S"
        if res == "conc":
            res = "C"
        return res

    def dot_node_label(self):
        try:
            label = self.execution_ns_alias
            if label and self.execution_ns:
                label = f'{label}='
            label = f'{label}{self.execution_ns}'
            res = f'{self.get_label_name()} ({label})'
            if self.get_type() == "wait":
                if "duration" in self.params:
                    duration = self.params["duration"]
                else:
                    duration = "NO DURATION"
                res += f"\n{duration}"
            return res
        except Exception as exc:
            self.get_logger().error(f'EXCEPTION dot_node_label: {type(exc)}')
            self.get_logger().error(f'{exc}')

    def get_dot_node(self, type):
        try:
            label = self.dot_node_label()
            if type in ["conc", "seq", "loop", "C", "S", "test_if", "logging"]:
                res = f'{self.dot_node_name()} [shape=circle,peripheris=1,label="{label}",style=filled,fillcolor="cyan"];'
            else:
                res = f'{self.dot_node_name()} [shape=box,peripheris=1,label="{label}",style=filled,fillcolor="cyan"];'
#            res += f' {self.dot_node_name()};\n'
            return res
        except Exception as exc:
            self.get_logger().error(f'EXCPETION get_dot_node: {type(exc)}')
            self.get_logger().error(f'{exc}')

    def get_dot_string(self, level):
        try:
            dot1 = self.get_dot_node(self.get_type())
            for c in self.children:
                dot1 += c.get_dot_string(level+1)
                dot1 += f'{self.dot_node_name()} -> {c.dot_node_name()};\n'
            return dot1
        except Exception as exc:
            self.get_logger().error(f'EXCEPTION get_dot_string: {type(exc)}')
            self.get_logger().error(f'{exc}')

    def get_dot(self):
        try:
            dot1 = self.get_dot_string(1)
            res = ""
            res += "digraph TST {\n"
            res += "  rankdir=LR;\n"
            res +=  self.get_dot_string(1)
            res += "  ordering=out;\n"
            res += "}\n"
            return res
        except Exception as exc:
            self.get_logger().error(f'EXCEPTION get_dot: {type(exc)}')
            self.get_logger().error(f'{exc}')


#    def get_exec_json(self, jsonobj):
        #node["name"] = tni.type;
        #node["execution_ns"] = tni.execution_ns;
        #node["delegation_ns"] = tni.delegation_ns;
        #node["children"] = Json::arrayValue;
        #node["resources_to_lock"] = Json::arrayValue;
        #for (unsigned int i=0; i< tni.resources_to_lock.size(); i++) {
        #node["resources_to_lock"][i] = tni.resources_to_lock[i];
        #}
        #node["delexec_resources_to_lock"] = Json::arrayValue;
        #for (unsigned int i=0; i< tni.delexec_resources_to_lock.size(); i++) {
        #node["delexec_resources_to_lock"][i] = tni.delexec_resources_to_lock[i];
        #}
        # node["wait_for_finished"] = Json::arrayValue;
        # for (unsigned int i=0; i< tni.wait_for_finished.size(); i++) {
        # node["wait_for_finished"][i] = tni.wait_for_finished[i];
        # }
        # node["csp_json"] = tni.csp_json;
        # node["csp_constraints"] = Json::arrayValue;
        # for (unsigned int i=0; i< tni.csp_constraints.size(); i++) {
        # node["csp_constraints"][i] = tni.csp_constraints[i];
        #}
        #node["execution_ns_alias"] = tni.execution_ns_alias;
        #node["csp_variables"] = Json::arrayValue;
        #for (unsigned int i=0; i< tni.csp_variables.size(); i++) {
        #        node["csp_variables"][i] = tni.csp_variables[i];
        #}

#  for (unsigned int i=0; i<tni.string_params.size(); i++) {
        #    if (tni.string_params[i].have_value) {
        #      node["params"][tni.string_params[i].name] = tni.string_params[i].value;
        #    }
        #  }
        #
        #  for (unsigned int i=0; i<tni.strings_params.size(); i++) {
        #    if (tni.strings_params[i].have_value) {
        #      for (unsigned int j=0; j<tni.strings_params[i].value.size(); j++) {
        #        node["params"][tni.strings_params[i].name][j] = tni.strings_params[i].value[j];
        #      }
        #    }
        #  }
        #
        #  for (unsigned int i=0; i<tni.bool_params.size(); i++) {
        #    if (tni.bool_params[i].have_value) {
        #      node["params"][tni.bool_params[i].name] = (bool) tni.bool_params[i].value;
        #    }
        #  }
        #
        #  for (unsigned int i=0; i<tni.int32_params.size(); i++) {
        #    if (tni.int32_params[i].have_value) {
        #      node["params"][tni.int32_params[i].name] = tni.int32_params[i].value;
        #    }
        #  }
        #
        #  for (unsigned int i=0; i<tni.float64_params.size(); i++) {
        #    if (tni.float64_params[i].have_value) {
        #      node["params"][tni.float64_params[i].name] = tni.float64_params[i].value;
        #    }
        #  }
        #
        #  for (unsigned int i=0; i<tni.float64s_params.size(); i++) {
        #    if (tni.float64s_params[i].have_value) {
        #      for (unsigned int j=0; j<tni.float64s_params[i].value.size(); j++) {      
        #	node["params"][tni.float64s_params[i].name][j] = tni.float64s_params[i].value[j];
        #      }
        #    }
        #  }
        #
        #  for (unsigned int i=0; i<tni.geo_point_params.size(); i++) {
        #    if (tni.geo_point_params[i].have_value) {
        #      node["params"][tni.geo_point_params[i].name] = ::get_json_value(tni.geo_point_params[i].value);
        #    }
        #  }
        #
        #  for (unsigned int i=0; i<tni.geo_points_params.size(); i++) {
        #    if (tni.geo_points_params[i].have_value) {
        #      for (unsigned int j=0; j<tni.geo_points_params[i].value.size(); j++) {
        #        node["params"][tni.geo_points_params[i].name][j] = ::get_json_value(tni.geo_points_params[i].value[j]);
        #      }
        #    }
        #  }
        #
        #  for (unsigned int i=0; i<tni.geo_point_arrays_params.size(); i++) {
        #    if (tni.geo_point_arrays_params[i].have_value) {
        #      std::vector<lrs_msgs_common::GeoPointArray> gpas = tni.geo_point_arrays_params[i].value;
        #      for (unsigned int j=0; j<gpas.size(); j++) {
        #        lrs_msgs_common::GeoPointArray gpa = gpas[j];        
        #        for (unsigned int k=0; k<gpa.geopoints.size(); k++) {
        #          node["params"][tni.geo_point_arrays_params[i].name][j][k] = ::get_json_value(gpa.geopoints[k]);
        #        }
        #      }
        #    }
        #  }
        #  
        #  for (unsigned int i=0; i<tni.point_params.size(); i++) {
        #    if (tni.point_params[i].have_value) {
        #      node["params"][tni.point_params[i].name] = ::get_json_value(tni.point_params[i].value);
        #    }
        #  }
        #
        #  for (unsigned int i=0; i<tni.points_params.size(); i++) {
        #    if (tni.points_params[i].have_value) {
        #      for (unsigned int j=0; j<tni.points_params[i].value.size(); j++) {
        #        node["params"][tni.points_params[i].name][j] = ::get_json_value(tni.points_params[i].value[j]);
        #      }
        #    }
        #  }
        #
        #  for (unsigned int i=0; i<tni.pose_params.size(); i++) {
        #    if (tni.pose_params[i].have_value) {
        #      node["params"][tni.pose_params[i].name] = ::get_json_value(tni.pose_params[i].value);
        #    }
        #  }
        #
        #  node["id"] = tni.factory_local_id;
        #  node["unique_node_id"] = tni.unique_node_id;
        #  node["root_uuid"] = tni.root_uuid;
        #  node["node_uuid"] = tni.node_uuid;
        #  //
        #  // Time scheduling values
        #  //
        #
        #  if (tni.stime_lb.have_value) {
        #    node["stime_lb"] = tni.stime_lb.value.toSec();
        #  }
        #  if (tni.stime_ub.have_value) {
        #    node["stime_ub"] = tni.stime_ub.value.toSec();
        #  }
        #  if (tni.etime_lb.have_value) {
        #    node["etime_lb"] = tni.etime_lb.value.toSec();
        #  }
        #  if (tni.etime_ub.have_value) {
        #    node["etime_ub"] = tni.etime_ub.value.toSec();
        #  }
        #  if (tni.duration_lb.have_value) {
        #    node["duration_lb"] = tni.duration_lb.value.toSec();
        #  }
        #  if (tni.duration_lb.have_value) {
        #    node["duration_ub"] = tni.duration_ub.value.toSec();
        #  }
        #
        #  //
        #  // Execution part
        #  //
        #
        #  node["abort_when_starting"] = tni.abort_when_starting;
        #  node["enough_when_starting"] = tni.enough_when_starting;
        #  node["use_delegation_lock"] = tni.use_delegation_lock;
        #  node["use_execution_lock"] = tni.use_execution_lock;
        #  node["approved"] = tni.approved;
        #  node["wait_for_stime"] = tni.wait_for_stime;
        #  node["wait_for_etime"] = tni.wait_for_etime;
        #
        #  node["exec_status"] = get_json_value(tni.exec_status);
        #  node["exec_info"] = get_json_value(tni.exec_info);
        #  //
        #  // Children
        #  //
        #       
        #  for (unsigned int i=0; i<tni.children.size(); i++) {
        #    Json::Value child;
        #    get_exec_json (tni.children[i], child);
        #    node["children"][i] = child;
        #  }
        #  
#        pass
        
