import time

from lrs_exec.parameters import Parameters

from lrs_srvs_tst.srv import TSTSetFlag, TSTGetNode, TSTGetNodeFromUuid, TSTSetFailReason, TSTGetExecInfo
from lrs_srvs_tst.srv import TSTSetStartExecutorFlag, TSTGetUuidFromId, TSTSetExecInfo, TSTSendSignal
from lrs_srvs_tst.srv import TSTSetNodeParameterString, TSTGetTreeStringFromUuid
from lrs_srvs_ra.srv import RALockResource, RAUnlockResource
from lrs_srvs_wdb.srv import ExecutionLockRelease, WDBWorldToGeo, WDBGeoToWorld
from lrs_msgs_tst.msg import TSTExecInfo

from rclpy.duration import Duration

class Executor():
    def __init__(self, node, id):
        try:
            self.executor_wait_finished = False
            self.executor_wait_failed = False
            self.use_ticking = False
            self.ticked = False
            self.basic_start_exited = False
            self.verbose = False
            self.id = id
            self.node = node
            self.prepare_flag = False
            ## self.node.get_logger().error(f'executor init')
            self.world_to_geo_client = self.node.create_client(WDBWorldToGeo, "wdb/world_to_geo", callback_group=self.node.group)
            self.geo_to_world_client = self.node.create_client(WDBGeoToWorld, "wdb/geo_to_world", callback_group=self.node.group)            
            self.set_active_flag_client = self.node.create_client(TSTSetFlag, "exec/tstfactory/set_active_flag", callback_group=node.group)
            self.set_waiting_flag_client = self.node.create_client(TSTSetFlag, "exec/tstfactory/set_waiting_flag", callback_group=node.group)
            self.set_executing_flag_client = self.node.create_client(TSTSetFlag, "exec/tstfactory/set_executing_flag", callback_group=node.group)
            self.set_finished_flag_client = self.node.create_client(TSTSetFlag, "exec/tstfactory/set_finished_flag", callback_group=node.group)
            self.set_was_finished_flag_client = self.node.create_client(TSTSetFlag, "exec/tstfactory/set_was_finished_flag", callback_group=node.group)                            
            self.set_aborted_flag_client = self.node.create_client(TSTSetFlag, "exec/tstfactory/set_aborted_flag", callback_group=node.group)
            self.set_paused_flag_client = self.node.create_client(TSTSetFlag, "exec/tstfactory/set_paused_flag", callback_group=node.group)            
            self.set_succeeded_flag_client = self.node.create_client(TSTSetFlag, "exec/tstfactory/set_succeeded_flag", callback_group=node.group)
            self.set_fail_reason_client = self.node.create_client(TSTSetFailReason, "exec/tstfactory/set_fail_reason", callback_group=node.group)
            self.get_node_client = self.node.create_client(TSTGetNode, "exec/tstfactory/get_node", callback_group=node.group)
            self.get_node_from_uuid_client = self.node.create_client(TSTGetNodeFromUuid, "exec/tstfactory/get_node_from_uuid", callback_group=node.group)
            self.get_exec_info_client = self.node.create_client(TSTGetExecInfo, "exec/tstfactory/get_exec_info", callback_group=node.group)
            self.set_exec_info_client = self.node.create_client(TSTSetExecInfo, "exec/tstfactory/set_exec_info", callback_group=node.group)            
            self.get_uuid_from_id_client = self.node.create_client(TSTGetUuidFromId, "exec/tstfactory/get_uuid_from_id", callback_group=node.group)                        
            self.lock_resource_client = self.node.create_client(RALockResource, "ra/lock_resource", callback_group=node.group)
            self.unlock_resource_client = self.node.create_client(RAUnlockResource, "ra/unlock_resource", callback_group=node.group)
            self.execution_lock_release_client = self.node.create_client(ExecutionLockRelease, "wdb/execution_lock_release", callback_group=node.group)
            self.set_string_client = self.node.create_client(TSTSetNodeParameterString, "exec/tstfactory/set_node_parameter_string", callback_group=self.node.group)
            self.send_signal_client = self.node.create_client(TSTSendSignal, "exec/tstfactory/send_signal", callback_group=self.node.group)
            self.get_tree_string_client = self.node.create_client(TSTGetTreeStringFromUuid, f'exec/tstfactory/get_tree_string_from_uuid', callback_group=self.node.group)            
            
            self.failed = False
            self.finished = False
            self.normal_exit_flag = True
            self.tni = None
            self.resources_to_lock = []
            self.lock_fail_is_fail = False
            self.handled_signals = []
            self.signals = []
            self.can_be_paused = False
            self.can_be_enoughed = False
            self.can_be_aborted = False
            ## To clear all locks, only for debuggin....
            ## self.unlock_resource("fly")

            self.use_ticking = False
            
            
        except Exception as exc:
            self.node.get_logger().error(f'EXCEPTION executor init: {type(exc)} - {exc}')

    def lprint(self, *val):
        if self.verbose:
            print(val)

    def get_tree_string(self, uuid, format):
        # self.get_logger().info(f'get_tree_string: {uuid} {format}')
        try:
            req = TSTGetTreeStringFromUuid.Request()
            req.uuid = uuid
            req.type = format
            future = self.get_tree_string_client.call_async(req)
            resp = self.wait_future(future, print_waiting=False)
            # self.get_logger().info(f'get_tree_string resp: {resp}')
            return resp
        except Exception as exc:
            self.get_logger().error(f'EXCEPTION get_tree_string: {type(exc)} - {exc}')
        return None

    def get_exec_json(self):
        ##self.node.get_logger().error(f'get_exec_json: {self.tni.node_uuid}')
        resp = self.get_tree_string(self.tni.node_uuid, 4)
        ##self.node.get_logger().info(f'get_exec_json: {resp}')
        return resp.data

    def set_signals(self, signals):
        try:
            self.handled_signals = signals
            einfo = TSTExecInfo()
            einfo.signals = signals
            einfo.can_be_aborted = False
            einfo.can_be_enoughed = False
            einfo.can_be_paused = False
            einfo.can_be_signaled = False
            for signal in signals:
                if signal not in ["$enough", "$pause", "$continue", "$abort"]:
                    ###self.node.get_logger().info(f'Set can_be_signaled: {self.id}')
                    einfo.can_be_signaled = True
                if signal == "$enough":
                    ###self.node.get_logger().info(f'Set can_be_enoughed: {self.id}')
                    self.can_be_enoughed = True
                    einfo.can_be_enoughed = True
                if signal == "$abort":
                    ###self.node.get_logger().info(f'Set can_be_aborted: {self.id}')
                    self.can_be_aborted = True
                    einfo.can_be_aborted = True
                if signal == "$pause":
                    ###self.node.get_logger().info(f'Set can_be_paused: {self.id}')
                    self.can_be_paused = True
                    einfo.can_be_paused = True
                self.set_exec_info(self.id, einfo)
        except Exception as exc:
            self.node.get_logger().error(f'Exception executor set_signals {type(exc)} - {exc}')

    def push_signal(self, signal):
        self.node.get_logger().info(f'push_signal {self.id}: {signal}')
        if signal in self.handled_signals:
            self.node.get_logger().info(f'REALLY push_signal {self.id}: {signal}')
            self.signals.append(signal)
            self.node.get_logger().info(f'SIGNALS {self.id}: {self.signals}')

    def have_signal(self):
        # self.node.get_logger().info(f'HAVE_SIGNALS: {self.id}')
        return (len(self.signals) > 0)

    def pop_signal(self):
        res = ""
        if len(self.signals) > 0:
            res = self.signals.pop(0)
        return res

    def add_resource_to_lock(self, name):
        self.resources_to_lock.append(name)

    def get_set_flag_request(self, flag):
        req = TSTSetFlag.Request()
        req.id = self.id
        req.val = flag
        return req

    def wait_future(self, future, print_waiting=False):
        while not future.done():
            if print_waiting:
                print(f"wait_future executor")
        resp = future.result()
        return resp

    def send_signal(self, node_uuid, signal):
        try:
            self.node.get_logger().info(f'send_signal: {node_uuid}')
            req = TSTSendSignal.Request()
            req.node_uuid = node_uuid
            req.signal = signal
            future = self.send_signal_client.call_async(req)
            resp = self.wait_future(future)        
            self.node.get_logger().info(f'send_signal_client resp: {resp}')
            return resp
        except Exception as exc:
            self.node.get_logger().error(f'EXCEPTION send_signal: {type(exc)} - {exc}')
            
    def world_to_geo(self, x, y, z):
        self.lprint("world_to_geo:", x, y, z)
        try:
            req = WDBWorldToGeo.Request()
            req.x = float(x)
            req.y = float(y)
            req.z = float(z)
            future = self.world_to_geo_client.call_async(req)
            resp = self.wait_future(future, print_waiting=False)
            return resp
        except Exception as exc:
            print("world_to_geo",  type(exc))
            print(exc)
        return None
    
    def geo_to_world(self, lat, lon, alt):
        self.lprint("geo_to_world:", lat, lon, alt)
        try:
            req = WDBGeoToWorld.Request()
            req.lat = float(lat)
            req.lon = float(lon)
            req.alt = float(alt)
            future = self.geo_to_world_client.call_async(req)
            resp = self.wait_future(future, print_waiting=False)
            return resp
        except Exception as exc:
            print("world_to_geo",  type(exc))
            print(exc)
        return None

    def execution_lock_release(self, uuidstr):
        self.lprint("execution_lock_release", uuidstr)
        req = ExecutionLockRelease.Request()
        req.uuid = uuidstr
        future = self.set_succeeded_flag_client.call_async(req)
        resp = self.wait_future(future)
        self.lprint("execution_lock_release resp:", resp)
        return resp
    

    def set_node_parameter_string(self, id, name, val):
        self.lprint("set_node_parameter_string", id, name, val)
        req = TSTSetNodeParameterString.Request()
        req.id = id
        req.name = name
        req.value = val
        future = self.set_string_client.call_async(req)
        resp = self.wait_future(future)
        self.lprint("set_node_parameter_string resp:", resp)
        return resp
    

    def set_start_executor_flag(self, id, flag):
        return self.node.set_start_executor_flag(id, flag)
            
    def set_succeeded_flag(self, flag):
        self.lprint("set_succeeded_flag", flag)
        future = self.set_succeeded_flag_client.call_async(self.get_set_flag_request(flag))
        resp = self.wait_future(future)
        self.lprint("set_succeeded_flag resp:", resp)
        return resp

    def set_finished_flag(self, flag):
        self.lprint("set_finished_flag_flag", flag)        
        future = self.set_finished_flag_client.call_async(self.get_set_flag_request(flag))
        resp = self.wait_future(future)
        self.lprint("set_finished_flag resp:", resp)
        self.finished = True
        return resp

    def set_was_finished_flag(self, flag):
        self.lprint("set_was_finished_flag_flag", flag)        
        future = self.set_was_finished_flag_client.call_async(self.get_set_flag_request(flag))
        resp = self.wait_future(future)
        self.lprint("set_was_finished_flag resp:", resp)
        return resp

    def set_active_flag(self, flag):
        self.node.get_logger().error(f'set_active_flag: {flag}')                
        future = self.set_active_flag_client.call_async(self.get_set_flag_request(flag))
        resp = self.wait_future(future)
        self.node.get_logger().error(f'set_active_flag resp: {resp}')                        
        return resp

    def set_waiting_flag(self, flag):
        self.lprint("set_waiting_flag", flag)        
        future = self.set_waiting_flag_client.call_async(self.get_set_flag_request(flag))
        resp = self.wait_future(future)
        self.lprint("set_waiting_flag resp:", resp)
        return resp

    def set_executing_flag(self, flag):
        self.lprint("set_executing_flag", flag)        
        future = self.set_executing_flag_client.call_async(self.get_set_flag_request(flag))
        resp = self.wait_future(future)
        self.lprint("set_executing_flag resp:", resp)
        return resp

    def set_aborted_flag(self, flag):
        self.lprint("set_aborted_flag", flag)        
        future = self.set_aborted_flag_client.call_async(self.get_set_flag_request(flag))
        resp = self.wait_future(future)
        self.lprint("set_aborted_flag resp:", resp)
        return resp

    def set_paused_flag(self, flag):
        self.lprint("set_paused_flag", flag)        
        future = self.set_paused_flag_client.call_async(self.get_set_flag_request(flag))
        resp = self.wait_future(future)
        self.lprint("set_paused_flag resp:", resp)
        return resp

    def get_node(self, id):
        # self.lprint("get_node:", id)
        req = TSTGetNode.Request()
        req.id = id
        future = self.get_node_client.call_async(req)
        resp = self.wait_future(future)        
        # self.lprint("get_node resp:", resp.info.type)
        return resp

    def get_node_from_uuid(self, nuuid):
        # self.lprint("get_node_from_uuid:", nuuid)
        req = TSTGetNodeFromUuid.Request()
        req.node_uuid = nuuid
        future = self.get_node_from_uuid_client.call_async(req)
        resp = self.wait_future(future)        
        # self.lprint("get_node_from_uuid resp:", resp)
        return resp

    def get_exec_info(self, nuuid):
        ## self.node.get_logger().error(f'get_exec_info: {nuuid}')        
        req = TSTGetExecInfo.Request()
        req.node_uuid = nuuid
        future = self.get_exec_info_client.call_async(req)
        resp = self.wait_future(future)
        ## self.node.get_logger().error(f'get_exec_info resp: {resp}')                
        return resp

    def set_exec_info(self, id, exec_info):
        try:
            # print("get_exec_info:", nuuid)
            ###self.node.get_logger().info(f'set_exec_info: {id} - {exec_info}')            
            req = TSTSetExecInfo.Request()
            req.id = id
            req.val = exec_info
            future = self.set_exec_info_client.call_async(req)
            resp = self.wait_future(future)
            ###self.node.get_logger().info(f'set_exec_info resp: {resp}')
            return resp
        except Exception as exc:
            self.node.get_logger().error(f'Exception executor set_exec_info {type(exc)} - {exc}')            

    def get_uuid_from_id(self, id):
        res = ""
        try:
            self.node.get_logger().info(f'get_uuid_from_id: {id}')            
            req = TSTGetUuidFromId.Request()
            req.id = id
            future = self.get_uuid_from_id_client.call_async(req)
            resp = self.wait_future(future)        
            if resp.success:
                res = resp.uuid
            else:
                self.node.get_logger().error(f'failed get_uuid_from_id resp: {resp}')
        except Exception as exc:
            self.node.get_logger().error(f'Exception get_uuid_from_id: {type(exc)} - {exc}')
        return res

    def lock_resource(self, resource):
        # print("lock_resource:", resource)
        req = RALockResource.Request()
        req.resource = resource
        req.node_uuid = self.tni.node_uuid
        future = self.lock_resource_client.call_async(req)
        resp = self.wait_future(future, print_waiting=False)        
        # print("lock_resource resp:", resp)
        return resp

    def unlock_resource(self, resource):
        self.lprint("unlock_resource:", resource)
        req = RAUnlockResource.Request()
        req.resource = resource
        future = self.unlock_resource_client.call_async(req)
        resp = self.wait_future(future)        
        self.lprint("unlock_resource resp:", resp)
        return resp

    def prepare(self):
        self.prepare_flag = True
        self.node.get_logger().error(f'prepare')                
        # set_active_flag (node_ns, node_id, true);
        self.set_active_flag(True)
        return True

    def abort_fail(self, str):
        self.node.get_logger().error(f'ABORT FAIL executor: {str}')        
        self.normal_exit_flag = False
        # send info to mqtt
        # unlock_resources
        unlock_res = self.unlock_resources()
        self.node.get_logger().error(f'abort_fail cleanup: unlock_resources: {unlock_res}')
        exec_unlock_res = True
        if self.tni.use_execution_lock:
            exec_unlock_res = self.execution_lock_release(self, tni.root_uuid)
            self.node.get_logger().error(f'abort_fail cleanup: exec_unlock_res: {exec_unlock_res}')
        # release possible execution lock
        # set_fail_reason (node_ns, node_id, msg);
        self.set_succeeded_flag(False)
        self.set_aborted_flag(True)
        self.set_finished_flag(True)
        self.finished = True

    def signal_finish(self):
        self.node.get_logger().error(f'SIGNAL_FINISH executor')                
        self.normal_exit_flag = False
        # send info to mqtt
        # unlock_resources
        # release possible execution lock
        # set_fail_reason (node_ns, node_id, msg);
        self.set_was_finished_flag(True)
        self.set_succeeded_flag(True)
        self.set_aborted_flag(False)
        self.set_finished_flag(True)
        self.finished = True

    def fail(self, str):
        self.node.get_logger().error(f'FAIL executor: {str}')                
        self.normal_exit_flag = False
        # send info to mqtt
        # unlock_resources
        # release possible execution lock
        # set_fail_reason (node_ns, node_id, msg);
        unlock_res = self.unlock_resources()
        exec_unlock_res = True        
        if self.tni.use_execution_lock:
            exec_unlock_res = self.execution_lock_release(self, tni.root_uuid)
        self.set_succeeded_flag(False)
        self.set_aborted_flag(False)
        self.set_finished_flag(True)
        self.finished = True
        self.params = None
    
    def is_active(self):
        res = True
        if self.tni:
            res = self.tni.exec_status.active
        else:
            res = False
        return res

    def fetch_node_info(self):
        req = TSTGetNode.Request()
        req.id = self.id
        future = self.get_node_client.call_async(req)
        i = 0
        while not future.done():
            time.sleep(0.1)
            i += 1
            if i>10:
                self.node.get_logger().error(f'waiting fetch_node_info: calling get_node_client again')
                self.get_node_client = self.node.create_client(TSTGetNode, "exec/tstfactory/get_node", callback_group=self.node.group)
                future = self.get_node_client.call_async(req)
                i = 0
        resp = future.result()
        # print("fetch_node_info:", resp.info.name, resp.info.type)
        self.tni = resp.info

    def init_params(self):
        self.params = Parameters(self.tni)
        return True
    
    def do_before_work(self):
        self.node.get_logger().info(f'do_before_work: {self.id}')                        
        #   normal_exit_flag = true;
        self.fetch_node_info()
        # print("NodeInfo:", self.tni)
        self.node.get_logger().info(f'do_before_work - got node-info: {self.id}')

        # Whould be done when ticking and after executor wait
        #if self.tni.abort_when_starting:
        #    self.abort_fail("abort_when_starting flag set")
        #    return False

        #if not self.is_active():
        #    self.node.get_logger().error(f'do_before_work - NOT ACTIVE: {self.id}')            
        #    self.set_succeeded_flag(False)
        #    self.set_finished_flag(True)
        #    return False;

        if not self.init_params():
            self.node.get_logger().error(f'do_before_work - init_params failes: {self.id}')                        
            print("init_params failed")
            return False

        if not self.use_ticking:
            if not self.wait_for_prework_conditions():
                self.node.get_logger().error(f'do_before_work - pre-work conditions failed: {self.id}')
                return False

        self.node.get_logger().info(f'do_before_work - RETURN true: {self.id}')            
        return True

    def wait_for_approved(self):
        # ROS_INFO ("executor_wait check for approved: %s %d", ns.c_str(), id);
        # tni = exec_get_node_info (ns, id);
        # while (!tni.approved) {
        # usleep (100000);
        # tni = exec_get_node_info (ns, id);
        # }
        pass
    
    def executor_wait(self):
        self.node.get_logger().error(f'****executor_wait')
        debug=False
        self.set_waiting_flag(True)
        if debug:
            print(1)
        for wait_uuid in self.tni.wait_for_finished:
            resp = self.get_node_from_uuid(wait_uuid)
            while resp.success and not resp.info.exec_status.finished:
                time.sleep(0.5)
                resp = self.get_node_from_uuid(wait_uuid)
            if not resp.success:
                return False

        if debug:
            print(2)
        while not self.tni.approved:
            time.sleep(0.5)
            print("Wait for approved")
            self.fetch_node_info()

        if debug:
            print(3)
            
        # // Check for execution time

        if self.tni.stime_lb.have_value:
            if self.tni.wait_for_stime:
                while True:
                    now = self.node.get_clock().now().to_msg()
                    cdiff = now.sec - self.tni.tree_start_time.sec
                    self.node.get_logger().info(f'wait until stime_lb: {cdiff} - {now} - {self.tni.tree_start_time}')                
                    if cdiff > self.tni.stime_lb.value.sec:
                        break
                    time.sleep(1)
            else:
                now = self.node.get_clock().now().to_msg()
                cdiff = now.sec - self.tni.tree_start_time.sec
                self.node.get_logger().info(f'wait until stime_lb: {cdiff} - {now} - {self.tni.tree_start_time}')                
                if cdiff < self.tni.stime_lb.value.sec:
                    return False

        for resname in self.resources_to_lock:
            self.node.get_logger().info(f'****LOCK_RESOURCE_ {resname}')
            resp = self.lock_resource(resname)
            if self.lock_fail_is_fail and not resp.got_lock:
                self.node.get_logger().error(f'****FAILED TO LOCK_RESOURCE_ {resname}')
                return False
            while not resp.got_lock:
                time.sleep(0.5)
                resp = self.lock_resource(resname)

        if debug:
            print(4)
            
        for resname in self.tni.resources_to_lock:
            print("LOCK EXTRA TNI RESOURCE:", resname)
            resp = self.lock_resource(resname)
            if self.lock_fail_is_fail and not resp.got_lock:
                return False
            while not resp.got_lock:
                time.sleep(0.5)
                resp = self.lock_resource(resname)

        return True

    def ticked_executor_wait(self):
        try:
            ## self.node.get_logger().info(f'====================ticked_executor_wait entering')
            executor_wait_finished = True
            debug=False

            if debug:
                print(1)
            
            for wait_uuid in self.tni.wait_for_finished:
                resp = self.get_node_from_uuid(wait_uuid)
                while resp.success and not resp.info.exec_status.finished:
                    time.sleep(0.5)
                    resp = self.get_node_from_uuid(wait_uuid)
                if not resp.success:
                    return False

            if debug:
                print(2)
            
            if not self.tni.approved:
                executor_wait_finished = False
                self.fetch_node_info()
                return False

            if debug:
                print(3)
            
            # // Check for execution time

            if self.tni.stime_lb.have_value:
                now = self.node.get_clock().now().to_msg()
                cdiff = now.sec - self.tni.tree_start_time.sec
                # self.node.get_logger().info(f'wait until stime_lb: {cdiff} - {now} - {self.tni.tree_start_time}')
                if cdiff < self.tni.stime_lb.value.sec:
                    if not self.tni.wait_for_stime:
                        self.executor_wait_failed = True
                    return False

            for resname in self.resources_to_lock:
                self.lprint("LOCK RESOURCE:", resname)
                resp = self.lock_resource(resname)
                if not resp.got_lock:
                    return False
            #if self.lock_fail_is_fail and not resp.got_lock:
            #    return False
            #while not resp.got_lock:
            #    time.sleep(0.5)
            #    resp = self.lock_resource(resname)

            if debug:
                print(4)
            
            for resname in self.tni.resources_to_lock:
                print("LOCK EXTRA TNI RESOURCE:", resname)
                resp = self.lock_resource(resname)
                if not resp.got_lock:
                    return False
                #if self.lock_fail_is_fail and not resp.got_lock:
                #    return False
                #while not resp.got_lock:
                #    time.sleep(0.5)
                #    resp = self.lock_resource(resname)
                
        except Exception as exc:
            self.node.get_logger().error(f'Exception get_uuid_from_id: {type(exc)} - {exc}')
            return False
        self.node.get_logger().info(f'ticked_executor_wait returning True')
        return True

    def wait_for_prework_conditions(self):
        self.lprint("wait_for_prework_conditions", type(self))
        if self.executor_wait():
            self.set_executing_flag(True)
            return True
        else:
            self.set_succeeded_flag(False)
            self.set_finished_flag(True)
            return False

    def ticked_wait_for_prework_conditions(self):
        ## self.node.get_logger().error(f'ticked_wait_for_prework_conditions')        
        if self.ticked_executor_wait():
            ## self.node.get_logger().error(f'ticked_wait_for_prework_conditions.SUCCESS')        
            self.executor_wait_finished = True
            self.set_executing_flag(True)
        else:
            self.node.get_logger().error(f'ticked_wait_for_prework_conditions.FAIL')                    

    def unlock_resources(self):
        res = True
        for resname in self.resources_to_lock:
            self.lprint("UNLOCK RESOURCE:", resname)
            resp = self.unlock_resource(resname)
            if not resp.success:
                res = False
        for resname in self.tni.resources_to_lock:
            self.lprint("UNLOCK RESOURCE:", resname)
            resp = self.unlock_resource(resname)
            if not resp.success:
                res = False
        return res

    def wait_for_postwork_conditions(self):
        self.node.get_logger().info(f'executor wait_for_postwork_conditions: {type(self)} - {self.id}')                

        # TODO: wait for etime

        unlock_res = self.unlock_resources()

        # self.node.get_logger().info(f'executor wait_for_postwork_conditions - after unlock resources: {type(self)}')                        

        exec_unlock_res = True
        if self.tni.use_execution_lock:
            exec_unlock_res = self.execution_lock_release(self, tni.root_uuid)

            # self.node.get_logger().info(f'executor wait_for_postwork_conditions - after exec lock release: {type(self)}')                            
        
        success = unlock_res and exec_unlock_res
        if success:
            self.set_succeeded_flag(True)
            self.set_finished_flag(True)
        else:
            self.set_succeeded_flag(False)
            self.set_finished_flag(True)

        self.node.get_logger().info(f'executor wait_for_postwork_conditions - before return {success}: {type(self)} - {self.id}')                
        return success

    def normal_exit(self):
        return self.normal_exit_flag

    def basic_do_before_work(self):
        self.node.get_logger().info(f'basic_do_before_work - ENTER true: {self.id}')            
        if not self.do_before_work():
            self.failed = True
            self.finished = True
            ## self.node.get_logger().error(f'Executor basic_start: do_before_work failed')
            self.basic_start_exited = True
            self.node.get_logger().info(f'basic_do_before_work - RETURN false: {self.id}')
            return False
        self.node.get_logger().info(f'basic_do_before_work - RETURN true: {self.id}')                    
        return True

    def basic_start(self):
        ## self.node.get_logger().error(f'Executor basic_start: {self.id}')        

        if not self.basic_do_before_work():
            self.node.get_logger().error(f'Executor basic_do_before_work failed')
            return

        ## self.node.get_logger().error(f'Executor basic_start before callin start: {self.id}')                
        self.start()

        if self.normal_exit():
            # self.node.get_logger().error(f'Executor basic_start: normal exit wait for postwork conditions')
            self.wait_for_postwork_conditions()
            # self.node.get_logger().error(f'Executor basic_start: normal exit FINISHED waiting for postwork conditions')
        else:
            # cleanup anyway
            self.node.get_logger().error(f'Fail cleanup')
                        
            unlock_res = self.unlock_resources()
            self.node.get_logger().error(f'Fail cleanup: unlock_resources: {unlock_res}')

            exec_unlock_res = True
            if self.tni.use_execution_lock:
                exec_unlock_res = self.execution_lock_release(self, tni.root_uuid)
                self.node.get_logger().error(f'Fail cleanup: exec_unlock_res: {exec_unlock_res}')

        self.basic_start_exited = True                


