import time
import uuid
import rclpy
from rclpy.executors import MultiThreadedExecutor

from lrs_exec.ticked_executor import TickedExecutor
from lrs_exec.ticked_executor_factory import TickedExecutorFactory

from lrs_team_msgs.srv import AddTask

class AddTaskToTeamQueueExecutor(TickedExecutor):
    def __init__(self, node, id):
        super().__init__(node, id)
        self.add_task_client = node.create_client(AddTask, "team/add_task", callback_group=node.group)
        
    def do_work(self):
        try:
            self.node.get_logger().info(f'do_work: yes')
            answer = "true"
            response = self.set_node_parameter_string(self.id, "answer", answer)
            self.params.print()
            team_name = self.params.get_value("team_name")
            task_name = self.params.get_value("task_name")
            task_type = self.params.get_value("task_type")
            task_string = self.params.get_value("task_string")
            task_priority = self.params.get_value("task_priority") # int


            print("AddTaskToTeamQueueExecutor team name:", team_name)
            print("AddTaskToTeamQueueExecutor task name:", task_name)
            print("AddTaskToTeamQueueExecutor task type:", task_type)
            print("AddTaskToTeamQueueExecutor task string:", task_string)
            print("AddTaskToTeamQueueExecutor task priority:", task_priority)

            self.add_task_client.wait_for_service()

            req = AddTask.Request()
            req.team_name = team_name
            req.task.uuid = str(uuid.uuid4())
            req.task.priority = task_priority
            req.task.type = task_type
            req.task.task = task_string
            req.task.name = task_name
            self.add_task_client.call_async(req)

            
        except Exception as exc:
            self.node.get_logger().error(f'EXCEPTION start: {type(exc)} - {exc}')

class AddTaskToTeamQueueExecutorFactory(TickedExecutorFactory):
    def __init__(self):
        super().__init__("add-task-to-team-queue")

    def get_executor(self, node, id):
        return self.add_executor(AddTaskToTeamQueueExecutor(node, id))

def main(args=None):
    rclpy.init(args=args)
    
    executor = MultiThreadedExecutor(num_threads=8)    
    
    node = AddTaskToTeamQueueExecutorFactory()
    
    executor.add_node(node)    
    print(f"Spinning {__name__}")
    try:
        executor.spin()
    except KeyboardInterrupt:
        pass

    executor.shutdown()
    rclpy.shutdown()

if __name__ == '__main__':
    main()
    
