import time
import rclpy
from rclpy.executors import MultiThreadedExecutor

from lrs_exec.executor_factory import ExecutorFactory
from lrs_msgs_tst.msg import TSTNodeStatus
from lrs_exec.ticked_executor import TickedExecutor
from lrs_exec.ticked_executor_factory import TickedExecutorFactory

from lrs_util.jsonutil import *
from lrs_util.json_exec_node import JsonExecNode

import std_msgs

class CollectReleaseExecutorFactory(ExecutorFactory):
    def __init__(self):
        super().__init__("collect-release", expandable=True)
        
    def json_expand(self, node_json):
        self.get_logger().error(f'json_expand node_json: {node_json}')        
        res = ""
        if not self.expandable:
            return res
        try:
            # self.get_logger().error(f'json_expand node_json: {node_json}')
            jobj = json.loads(node_json)

            unit = "/sup1"

            
            
            gp0 = GeoPoint()
            gp0.altitude = jobj["params"]["gp0"]["altitude"]
            gp0.latitude = jobj["params"]["gp0"]["latitude"]
            gp0.longitude = jobj["params"]["gp0"]["longitude"]
            
            gp1 = GeoPoint()
            gp1.altitude = jobj["params"]["gp1"]["altitude"]
            gp1.latitude = jobj["params"]["gp1"]["latitude"]
            gp1.longitude = jobj["params"]["gp1"]["longitude"]
            

            collect_move = JsonExecNode(unit, name="move-to",
                                        geopoint=gp0, speed="standard")
            collect = JsonExecNode(unit, name="collect", cargo="ball")
            
            release_move = JsonExecNode(unit, name="move-to",
                                        geopoint=gp1, speed="standard")
            release = JsonExecNode(unit, name="release")
            tree = JsonExecNode(self.get_namespace(), name="seq", node_uuid=str(uuid.uuid4()))
            
            tree.add_child(collect_move)
            tree.add_child(collect)
            tree.add_child(release_move)
            tree.add_child(release)
            treeobj = tree.get_json()
            res = json.dumps(treeobj, sort_keys=True, indent=4, separators=(',', ': '))
            return res
        except Exception as exc:
            self.get_logger().error(f'EXCEPTION move_to json_expand: {type(exc)} - {exc}')            
        return res
    
def main(args=None):
    rclpy.init(args=args)
    
    executor = MultiThreadedExecutor(num_threads=8)    
    
    node = CollectReleaseExecutorFactory()
    
    executor.add_node(node)    
    print(f"Spinning {__name__}")
    try:
        executor.spin()
    except KeyboardInterrupt:
        pass

    executor.shutdown()
    rclpy.shutdown()

if __name__ == '__main__':
    main()
