import time
import rclpy
from rclpy.executors import MultiThreadedExecutor

from lrs_exec.ticked_executor import TickedExecutor
from lrs_exec.ticked_executor_factory import TickedExecutorFactory

class FailExecutor(TickedExecutor):
    def __init__(self, node, id):
        super().__init__(node, id)
        
    def do_work(self):
        try:
            self.node.get_logger().info(f'do_work: fail')
            time.sleep(1)
            self.fail("Mandatory fail")
            
        except Exception as exc:
            self.node.get_logger().error(f'EXCEPTION start: {type(exc)} - {exc}')

class FailExecutorFactory(TickedExecutorFactory):
    def __init__(self):
        super().__init__("fail")

    def get_executor(self, node, id):
        return self.add_executor(FailExecutor(node, id))

def main(args=None):
    rclpy.init(args=args)
    
    executor = MultiThreadedExecutor(num_threads=8)    
    
    node = FailExecutorFactory()
    
    executor.add_node(node)    
    print(f"Spinning {__name__}")
    try:
        executor.spin()
    except KeyboardInterrupt:
        pass

    executor.shutdown()
    rclpy.shutdown()

if __name__ == '__main__':
    main()
    
