import os
import time
import rclpy
from rclpy.executors import MultiThreadedExecutor

from lrs_exec.executor_factory import ExecutorFactory
from lrs_msgs_tst.msg import TSTNodeStatus
from lrs_exec.ticked_executor import TickedExecutor
from lrs_exec.ticked_executor_factory import TickedExecutorFactory

from lrs_util.jsonutil import *
from lrs_util.json_exec_node import JsonExecNode
from lrs_util.coordtrans import CoordTrans

import std_msgs

class LookAtPositionExecutorFactory(ExecutorFactory):
    def __init__(self):
        super().__init__("look-at-position", expandable=True)
        self.coordtrans = CoordTrans(location=os.environ.get("LRS_LOCATION"))        
        
    def json_expand(self, node_json):
        self.get_logger().error(f'json_expand node_json: {node_json}')        
        res = ""
        if not self.expandable:
            return res
        try:
            # self.get_logger().error(f'json_expand node_json: {node_json}')
            jobj = json.loads(node_json)

            gp0 = GeoPoint()
            gp0.altitude = float(jobj["params"]["geopoint"]["altitude"])
            gp0.latitude = float(jobj["params"]["geopoint"]["latitude"])
            gp0.longitude = float(jobj["params"]["geopoint"]["longitude"])

            (lon, lat, alt) = self.coordtrans.world_to_wgs84(-40.0, -30.0, 0.0)
            gp1 = GeoPoint()
            gp1.altitude = alt
            gp1.latitude = lat
            gp1.longitude = lon
                        
            look = JsonExecNode(self.get_namespace(), name="collect-release",
                                geopoint0=gp0, geopoint1=gp1,
                                speed="standard")
            tree = JsonExecNode(self.get_namespace(), name="seq", node_uuid=str(uuid.uuid4()))
            
            tree.add_child(look)
            treeobj = tree.get_json()
            res = json.dumps(treeobj, sort_keys=True, indent=4, separators=(',', ': '))
            return res
        except Exception as exc:
            self.get_logger().error(f'EXCEPTION look_at_position json_expand: {type(exc)} - {exc}')            
        return res
    
def main(args=None):
    rclpy.init(args=args)
    
    executor = MultiThreadedExecutor(num_threads=8)    
    
    node = LookAtPositionExecutorFactory()
    
    executor.add_node(node)    
    print(f"Spinning {__name__}")
    try:
        executor.spin()
    except KeyboardInterrupt:
        pass

    executor.shutdown()
    rclpy.shutdown()

if __name__ == '__main__':
    main()
