import time
import rclpy
from rclpy.executors import MultiThreadedExecutor

from lrs_exec.ticked_executor import TickedExecutor
from lrs_exec.ticked_executor_factory import TickedExecutorFactory

try:
    from google_speech import Speech
except:
    pass

class SayExecutor(TickedExecutor):
    def __init__(self, node, id):
        super().__init__(node, id)
        
    def do_work(self):
        try:
            self.node.get_logger().info(f'do_work: say')
            self.params.print()            

            text = self.params.get_value("string")
            gain = self.params.get_value("gain", default="3")
            lang = "en"
            speech = Speech(text, lang)
            sox_effects = ("speed", "0.9", "gain", gain)
            speech.play(sox_effects)
            
        except Exception as exc:
            self.node.get_logger().error(f'EXCEPTION do_work: {type(exc)} - {exc}')
            self.node.get_logger().info(f'Tried to say: {text}')

class SayExecutorFactory(TickedExecutorFactory):
    def __init__(self):
        super().__init__("say")

    def get_executor(self, node, id):
        return self.add_executor(SayExecutor(node, id))

def main(args=None):
    rclpy.init(args=args)
    
    executor = MultiThreadedExecutor(num_threads=8)    
    
    node = SayExecutorFactory()
    
    executor.add_node(node)    
    print(f"Spinning {__name__}")
    try:
        executor.spin()
    except KeyboardInterrupt:
        pass

    executor.shutdown()
    rclpy.shutdown()

if __name__ == '__main__':
    main()
    
