import time
import rclpy
from rclpy.executors import MultiThreadedExecutor

from lrs_exec.ticked_executor import TickedExecutor
from lrs_exec.ticked_executor_factory import TickedExecutorFactory

class SendSignalExecutor(TickedExecutor):
    def __init__(self, node, id):
        super().__init__(node, id)

    def do_work(self):
        try:
            # self.params.print()

            uuid = self.params.get_value("uuid")
            if not uuid:
                self.fail("uuid param must be specified")
                return
            
            self.node.get_logger().info(f'uuid: {uuid}')
            
            signal = self.params.get_value("signal")
            if not signal:
                self.fail("signal param must be specified")
                return

            self.node.get_logger().info(f'signal: {signal}')

            self.send_signal(uuid, signal)
            
        except Exception as exc:
            self.node.get_logger().error(f'Exception: {type(exc)} - {exc}')
            self.fail(f'Exception: {exc}')
            
class SendSignalExecutorFactory(TickedExecutorFactory):
    def __init__(self):
        super().__init__("send-signal", ticked=True)
        
    def get_executor(self, node, id):
        return self.add_executor(SendSignalExecutor(node, id))
    

def main(args=None):
    rclpy.init(args=args)
    
    executor = MultiThreadedExecutor(num_threads=8)    
    
    node = SendSignalExecutorFactory()
    
    executor.add_node(node)    
    print(f"Spinning {__name__}")
    try:
        executor.spin()
    except KeyboardInterrupt:
        pass

    executor.shutdown()
    rclpy.shutdown()

if __name__ == '__main__':
    main()
    
