import time
import rclpy
from rclpy.executors import MultiThreadedExecutor

from lrs_exec.executor import Executor
from lrs_exec.executor_factory import ExecutorFactory
from ros_kdb_interfaces.srv import StartTopicRecording

class StartDataRecordingExecutor(Executor):
    def __init__(self, node, id):
        super().__init__(node, id)
        self.start_topic_recording_client = self.node.create_client(StartTopicRecording, "kDBServerNodelet/start_topic_recording", callback_group=self.node.group)                

    def start(self):
        try:
            self.node.get_logger().error(f'{__name__} start start-data-recording')
            self.params.print()

            topics = self.params.get_value("topics")
            opts = self.params.get_value("options")
            dataset_uri = self.params.get_value("dataset_uri")

            self.node.get_logger().info(f'TOPICS: {topics}')
            self.node.get_logger().info(f'OPTIONS: {opts}')
            self.node.get_logger().info(f'DATASET_URI: {dataset_uri}')

            # fail if topics is missing
            #   while(options.size() < topics.size()) { options.push_back(""); }
            # fail if dataset is missing

            for i in range(len(topics)):
                self.node.get_logger().error(f'START TOPIC: {topics[i]} - {opts[i]}')
                try:
                    req = StartTopicRecording.Request()
                    req.sensor_uri = ""
                    req.dataset_uri = dataset_uri
                    req.topic = topics[i]
                    req.options = opts[i]
                    future = self.start_topic_recording_client.call_async(req)
                    resp = self.wait_future(future)
                    self.node.get_logger().info(f'RESULT CALL START TOPIC REC: {resp}')
                except Exception as exc:
                    self.node.get_logger().error(f'EXCEPTION srv call: {type(exc)} - {exc}')
                
                #if (!kdb_start_topic_recording (ns, dataset_uri, topics[i], topics[i], "")) {
                #fail("Failed to start topic recording");
                #return;
                #}
            
        except Exception as exc:
            self.node.get_logger().error(f'EXCEPTION start: {type(exc)} - {exc}')

class StartDataRecordingExecutorFactory(ExecutorFactory):
    def __init__(self):
        super().__init__("start-data-recording")

    def get_executor(self, node, id):
        return StartDataRecordingExecutor(node, id)

def main(args=None):
    rclpy.init(args=args)
    
    executor = MultiThreadedExecutor(num_threads=8)    
    
    node = StartDataRecordingExecutorFactory()
    
    executor.add_node(node)    
    print(f"Spinning {__name__}")
    try:
        executor.spin()
    except KeyboardInterrupt:
        pass

    executor.shutdown()
    rclpy.shutdown()

if __name__ == '__main__':
    main()
    
