import time
import rclpy
from rclpy.executors import MultiThreadedExecutor

from lrs_exec.executor import Executor
from lrs_exec.executor_factory import ExecutorFactory
from ros_kdb_interfaces.srv import StopTopicRecording

class StopDataRecordingExecutor(Executor):
    def __init__(self, node, id):
        super().__init__(node, id)
        self.stop_topic_recording_client = self.node.create_client(StopTopicRecording, "kDBServerNodelet/stop_topic_recording", callback_group=self.node.group)                        

    def start(self):
        try:
            self.node.get_logger().info(f'{__name__} start start-data-recording')
            self.params.print()

            topics = self.params.get_value("topics")
            dataset_uri = self.params.get_value("dataset_uri")

            self.node.get_logger().info(f'TOPICS: {topics}')
            self.node.get_logger().info(f'DATASET_URI: {dataset_uri}')

            # fail if topics is missing
            # fail if dataset is missing

            for topic in topics:
                self.node.get_logger().error(f'STOP TOPIC: {topic}')

                try:
                    req = StopTopicRecording.Request()
                    req.dataset_uri = dataset_uri
                    req.topic = topic
                    req.force = False
                    future = self.stort_topic_recording_client.call_async(req)
                    resp = self.wait_future(future)
                    self.node.get_logger().info(f'RESULT CALL STOP TOPIC REC: {resp}')
                except Exception as exc:
                    self.node.get_logger().error(f'EXCEPTION srv call: {type(exc)} - {exc}')
                
                #if (!kdb_stop_topic_recording (ns, dataset_uri, topics[i])) {
                #fail("Failed to stop topic recording");
                #return;
                #}
                
        except Exception as exc:
            self.node.get_logger().error(f'EXCEPTION start: {type(exc)} - {exc}')

class StopDataRecordingExecutorFactory(ExecutorFactory):
    def __init__(self):
        super().__init__("stop-data-recording")

    def get_executor(self, node, id):
        return StopDataRecordingExecutor(node, id)

def main(args=None):
    rclpy.init(args=args)
    
    executor = MultiThreadedExecutor(num_threads=8)    
    
    node = StopDataRecordingExecutorFactory()
    
    executor.add_node(node)    
    print(f"Spinning {__name__}")
    try:
        executor.spin()
    except KeyboardInterrupt:
        pass

    executor.shutdown()
    rclpy.shutdown()

if __name__ == '__main__':
    main()
    
