import time
import rclpy
from rclpy.executors import MultiThreadedExecutor

from lrs_exec.ticked_executor import TickedExecutor
from lrs_exec.ticked_executor_factory import TickedExecutorFactory

from std_msgs.msg import String

class StringPubExecutor(TickedExecutor):
    def __init__(self, node, id):
        super().__init__(node, id)
        self.node = node
        
    def do_work(self):
        try:
            self.node.get_logger().info(f'do_work: ')
            self.params.print()            

            topic = self.params.get_value("topic", default="stream_deck_config")
            
            string = self.params.get_value("string", default="input")

            print("STRING:", string)
            print("TOPIC:", topic)
            pub = self.node.create_publisher(String, topic, 10)
            
            msg = String()
            msg.data = string
            pub.publish(msg)
            
        except Exception as exc:
            self.node.get_logger().error(f'EXCEPTION do_work: {type(exc)} - {exc}')

class StringPubExecutorFactory(TickedExecutorFactory):
    def __init__(self):
        super().__init__("string-pub")
        ### topic = "stream_deck_config"

    def get_executor(self, node, id):
        return self.add_executor(StringPubExecutor(node, id))

def main(args=None):
    rclpy.init(args=args)
    
    executor = MultiThreadedExecutor(num_threads=8)    
    
    node = StringPubExecutorFactory()
    
    executor.add_node(node)    
    print(f"Spinning {__name__}")
    try:
        executor.spin()
    except KeyboardInterrupt:
        pass

    executor.shutdown()
    rclpy.shutdown()

if __name__ == '__main__':
    main()
    
