import time
import rclpy
from rclpy.executors import MultiThreadedExecutor

from lrs_exec.ticked_executor import TickedExecutor
from lrs_exec.ticked_executor_factory import TickedExecutorFactory

class WaitExecutor(TickedExecutor):
    def __init__(self, node, id):
        super().__init__(node, id, blocking=False)
        self.set_signals(["$abort", "$pause", "$continue", "$enough", "$finished"])

    def do_work_begin(self):
        try:
            self.node.get_logger().info(f'{__name__} do_work_begin')

            # self.params.print()
            self.index = 0
            self.paused = False
            self.duration = self.params.get_value("duration")
            if not self.duration:
                self.fail("duration param must be specified")
                self.do_work_finished()
                return
            self.node.get_logger().info(f'Duration: {self.duration}')            
        except Exception as exc:
            self.node.get_logger().error(f'EXCEPTION start: {type(exc)} - {exc}')            
            self.fail(f'Exception: {exc}')
            
            
    def do_work(self):
        try:
            if 0.1*self.index > self.duration:
                self.do_work_finished()

            ### self.node.get_logger().info(f'WaitExecutor do work')
            
            if self.have_signal():
                signal = self.pop_signal()
                self.node.get_logger().info(f'WaitExecutor got signal: {signal}')
                if signal == "$finished":
                    self.signal_finish()
                    self.do_work_finished()
                    return
                if signal == "$enough":
                    self.do_work_finished()
                if signal == "$abort":
                    self.abort_fail("Got abort signal")
                    self.do_work_finished()
                if signal == "$pause":
                    self.paused = True
                if signal == "$continue":
                    self.paused = False
            if not self.paused:
                self.index += 1
        except Exception as exc:
            self.node.get_logger().error(f'EXCEPTION start: {type(exc)} - {exc}')            
            self.fail(f'Exception: {exc}')
        

#    def do_work(self):
#        try:
#            self.node.get_logger().info(f'{__name__} do_work')
#
#            # self.params.print()
#            index = 0
#            paused = False
#            duration = self.params.get_value("duration")
#            if not duration:
#                self.fail("duration param must be specified")
#                return
#            self.node.get_logger().info(f'Duration: {duration}')            
#            finished_flag = False
#            while not finished_flag and 0.1*index < duration:
#                if self.have_signal():
#                    signal = self.pop_signal()
#                    self.node.get_logger().info(f'WaitExecutor got signal: {signal}')
#                    if signal == "$finished":
#                        self.signal_finish()
#                        finished_flag = True                        
#                    if signal == "$enough":
#                        finished_flag = True
#                    if signal == "$abort":
#                        self.abort_fail("Got abort signal")
#                        finished_flag = True
#                    if signal == "$pause":
#                        paused = True
#                    if signal == "$continue":
#                        self.dji.mission_continue()                        
#                        paused = False
#                if not paused:
#                    index += 1
#                time.sleep(0.1)
#            self.node.get_logger().error(f'Wait finished.')                            
#        except Exception as exc:
#            self.node.get_logger().error(f'EXCEPTION start: {type(exc)} - {exc}')            
#            self.fail(f'Exception: {exc}')
#        ## self.node.get_logger().info("WaitExecutor end")
            
class WaitExecutorFactory(TickedExecutorFactory):
    def __init__(self):
        super().__init__("wait")

    def get_executor(self, node, id):
        return self.add_executor(WaitExecutor(node, id))
        

def main(args=None):
    rclpy.init(args=args)
    
    executor = MultiThreadedExecutor(num_threads=8)    
    
    node = WaitExecutorFactory()
    
    executor.add_node(node)    
    print(f"Spinning {__name__}")
    try:
        executor.spin()
    except KeyboardInterrupt:
        pass

    executor.shutdown()
    rclpy.shutdown()

if __name__ == '__main__':
    main()
    
