import time
import rclpy
from rclpy.executors import MultiThreadedExecutor

from lrs_exec.ticked_executor import TickedExecutor
from lrs_exec.ticked_executor_factory import TickedExecutorFactory
from sensor_msgs.msg import Joy

class WaitJoyExecutor(TickedExecutor):
    def __init__(self, node, id):
        super().__init__(node, id, blocking=False)
        self.set_signals(["$abort", "$pause", "$continue", "$enough", "$finished"])
        self.joy = None

    def joy_callback(self, msg):
        ## print("joy_callback:", msg)
        self.joy_data = msg
        
        
    def do_work_begin(self):
        try:
            self.node.get_logger().info(f'{__name__} do_work')

            self.joy_sub = self.node.create_subscription(Joy, "joy_teleop/joy", self.joy_callback, 10, callback_group=self.node.group)
            
        except Exception as exc:
            self.node.get_logger().error(f'EXCEPTION start: {type(exc)} - {exc}')            
            self.fail(f'Exception: {exc}')
            
            
    def do_work(self):
        try:
            # self.node.get_logger().info(f'do_work: joy_handler')
            # self.params.print()

            ## if self.joy_data.button[]

            if self.joy_data and self.joy_data.buttons[0]:
                self.do_work_finished()
                return
            
            if self.have_signal():
                signal = self.pop_signal()
                self.node.get_logger().info(f'WaitExecutor got signal: {signal}')
                if signal == "$finished":
                    self.signal_finish()
                    self.do_work_finished()
                    return
                if signal == "$enough":
                    self.do_work_finished()
                if signal == "$abort":
                    self.abort_fail("Got abort signal")
                    self.do_work_finished()
                if signal == "$pause":
                    self.paused = True
                if signal == "$continue":
                    self.paused = False
        except Exception as exc:
            self.node.get_logger().error(f'EXCEPTION start: {type(exc)} - {exc}')
            self.fail(f'Exception: {exc}')
        

#    def do_work(self):
#        try:
#            self.node.get_logger().info(f'{__name__} do_work')
#
#            # self.params.print()
#            index = 0
#            paused = False
#            duration = self.params.get_value("duration")
#            if not duration:
#                self.fail("duration param must be specified")
#                return
#            self.node.get_logger().info(f'Duration: {duration}')            
#            finished_flag = False
#            while not finished_flag and 0.1*index < duration:
#                if self.have_signal():
#                    signal = self.pop_signal()
#                    self.node.get_logger().info(f'WaitExecutor got signal: {signal}')
#                    if signal == "$finished":
#                        self.signal_finish()
#                        finished_flag = True                        
#                    if signal == "$enough":
#                        finished_flag = True
#                    if signal == "$abort":
#                        self.abort_fail("Got abort signal")
#                        finished_flag = True
#                    if signal == "$pause":
#                        paused = True
#                    if signal == "$continue":
#                        self.dji.mission_continue()                        
#                        paused = False
#                if not paused:
#                    index += 1
#                time.sleep(0.1)
#            self.node.get_logger().error(f'Wait finished.')                            
#        except Exception as exc:
#            self.node.get_logger().error(f'EXCEPTION start: {type(exc)} - {exc}')            
#            self.fail(f'Exception: {exc}')
#        ## self.node.get_logger().info("WaitExecutor end")
            
class WaitJoyExecutorFactory(TickedExecutorFactory):
    def __init__(self):
        super().__init__("wait-joy")

    def get_executor(self, node, id):
        return self.add_executor(WaitJoyExecutor(node, id))
        

def main(args=None):
    rclpy.init(args=args)
    
    executor = MultiThreadedExecutor(num_threads=8)    
    
    node = WaitJoyExecutorFactory()
    
    executor.add_node(node)    
    print(f"Spinning {__name__}")
    try:
        executor.spin()
    except KeyboardInterrupt:
        pass

    executor.shutdown()
    rclpy.shutdown()

if __name__ == '__main__':
    main()
    
