
from lrs_util.jsonutil import *

def get_name(jobj):
    res = ""
    if "name" in jobj:
        res = jobj["name"].replace("-", "_")
    return res
    
def get_exec_unit(jobj):
    res = ""
    cpobj = jobj["common_params"]
    if "execunit" in cpobj:
        res = cpobj["execunit"]
    if "execution_unit" in cpobj:
        res = cpobj["execution_unit"]
    if "execution_ns" in cpobj:
        res = cpobj["execution_ns"]
    return res

def expand_move_to(jobj):
    print(json.dumps(jobj, sort_keys=True, indent=4, separators=(',', ': ')))
    exec_unit = get_exec_unit(jobj)
    print("EXECUTION UNIT:", exec_unit)

    wp = jobj["params"]["waypoint"]

    flyto = json_basic_flyto(exec_unit, 0.0, 0.0, 0.0, -1.0, speed=jobj["params"]["speed"], geopoint=wp)
    expobj = json_basic_seq(exec_unit)
    expobj["children"].append(json_basic_in_air_goal(exec_unit))
    expobj["children"].append(flyto)
    return expobj
    
def expand_in_air_goal(jobj):
    print(json.dumps(jobj, sort_keys=True, indent=4, separators=(',', ': ')))
    exec_unit = get_exec_unit(jobj)
    print("EXECUTION UNIT:", exec_unit)

    expobj = json_basic_test_if(exec_unit)
    expobj["children"].append(json_basic_in_air_test(exec_unit))
    expobj["children"].append(json_basic_take_off(exec_unit))
    expobj["children"].append(json_basic_noop(exec_unit))
    expobj["children"].append(json_basic_noop(exec_unit))
    
    return expobj

def expand(jobj):
    name = get_name(jobj)
    res = None
    if name:
        try:
            res = eval(f'expand_{name}(jobj)')
        except Exception as exc:
            print("expand:", type(exc))
            print(exc)
    return res


def expand_tree(jobj):

    if not "children" in jobj:
        return

    if len(jobj["children"]) > 0:
        for c in jobj["children"]:
            expand_tree(c)
        return

    # No children, terminal or expansion
    expobj = expand(jobj)
    if expobj:
        jobj["children"].append(expobj)
        expand_tree(expobj)
    
    
    
if __name__ == '__main__':

    lat = 57.7605573519
    lon = 16.6827607783
    alt = 29.8
    
    jobj= json_moveto("/ex0", lat, lon, alt)
    expand_tree(jobj)
    print(json.dumps(jobj, sort_keys=True, indent=4, separators=(',', ': ')))

    #expobj = expand(jobj["children"][0])
    #print(expstr)

    #expstr = expand(obj1["children"][0])
    #print(expstr)
