import time
import rclpy
from rclpy.node import Node
from rclpy.callback_groups import ReentrantCallbackGroup

from lrs_srvs_tst.srv import TSTGetNode
from lrs_srvs_tst.srv import TSTCreateChildNode, TSTSetNodeParameterBool, TSTSetNodeParameterFloat64
from lrs_srvs_tst.srv import TSTSetNodeParameterFloat64s, TSTSetNodeParameterGeoPointArrays
from lrs_srvs_tst.srv import TSTSetNodeParameterGeoPoint, TSTSetNodeParameterGeoPoints, TSTSetNodeParameterInt32
from lrs_srvs_tst.srv import TSTSetNodeParameterPoint, TSTSetNodeParameterPoints, TSTSetNodeParameterPose
from lrs_srvs_tst.srv import TSTSetNodeParameterString, TSTSetNodeParameterStrings

from lrs_exec.executor_factory import ExecutorFactory


class ExpandNode(ExecutorFactory):
    def __init__(self, type):
        super().__init__(type)
        self.set_delegation_expandable(True)
        
        self.create_child_node_client = self.create_client(TSTCreateChildNode, "del/tstfactory/create_child_node", callback_group=self.group)
        self.set_bool_client = self.create_client(TSTSetNodeParameterBool, "del/tstfactory/set_node_parameter_bool", callback_group=self.group)
        self.set_float64_client = self.create_client(TSTSetNodeParameterFloat64, "del/tstfactory/set_node_parameter_float64", callback_group=self.group)
        self.set_float64s_client = self.create_client(TSTSetNodeParameterFloat64s, "del/tstfactory/set_node_parameter_float64s", callback_group=self.group)
        self.set_geopoin_arrays_client = self.create_client(TSTSetNodeParameterGeoPointArrays, "del/tstfactory/set_node_parameter_geo_point_arrays", callback_group=self.group)
        self.set_geopint_client = self.create_client(TSTSetNodeParameterGeoPoint, "del/tstfactory/set_node_parameter_geo_point", callback_group=self.group)
        self.set_geopints_client = self.create_client(TSTSetNodeParameterGeoPoints, "del/tstfactory/set_node_parameter_geo_points", callback_group=self.group)
        self.set_int32_client = self.create_client(TSTSetNodeParameterInt32, "del/tstfactory/set_node_parameter_int32", callback_group=self.group)
        self.set_point_client = self.create_client(TSTSetNodeParameterPoint, "del/tstfactory/set_node_parameter_point", callback_group=self.group)
        self.set_points_client = self.create_client(TSTSetNodeParameterPoints, "del/tstfactory/set_node_parameter_points", callback_group=self.group)
        self.set_pose_client = self.create_client(TSTSetNodeParameterPose, "del/tstfactory/set_node_parameter_pose", callback_group=self.group)
        self.set_string_client = self.create_client(TSTSetNodeParameterString, "del/tstfactory/set_node_parameter_string", callback_group=self.group)
        self.set_strings_client = self.create_client(TSTSetNodeParameterStrings, "del/tstfactory/set_node_parameter_strings", callback_group=self.group)

        self.set_delegation_expandable(True)
        
    def create_child_node(self, name, type, parent_id, approve_flag, node_uuid=""):
        print("create_child_node:", name, type, parent_id, approve_flag, node_uuid)
        req = TSTCreateChildNode.Request()
        req.name = name
        req.type = type
        req.parent = parent_id
        req.node_uuid = node_uuid
        req.approve_flag = approve_flag
        future = self.create_child_node_client.call_async(req)
        resp = self.wait_future(future)        
        print("create_child_node:", resp)
        return resp.id

    def set_parameter_string(self, id, name, val):
        res = True
        print("set_parameter_string:", name, val)
        req = TSTSetNodeParameterString.Request()
        req.id = id
        req.name = name
        req.value = val
        future = self.set_string_client.call_async(req)
        resp = self.wait_future(future)        
        print("set_parameter_string:", resp)
        res = resp.success;
        return res

    def set_parameter_int32(self, id, name, val):
        res = True
        print("set_parameter_int32:", name, val)
        req = TSTSetNodeParameterInt32.Request()
        req.id = id
        req.name = name
        req.value = val
        future = self.set_int32_client.call_async(req)
        resp = self.wait_future(future)        
        print("set_parameter_int32:", resp)
        res = resp.success;
        return res

    def set_parameter_geo_point(self, id, name, val):
        res = True
        print("set_parameter_geo_point:", name, val)
        req = TSTSetNodeParameterGeoPoint.Request()
        req.id = id
        req.name = name
        req.value = val
        future = self.set_geopint_client.call_async(req)
        resp = self.wait_future(future)        
        print("set_parameter_geo_point:", resp)
        res = resp.success;
        return res

    def set_execution_unit(self, id, exec_ns):
        return self.set_parameter_string(id, "execution_unit", exec_ns)
        
    def set_delegation_unit(self, id, exec_ns):
        return self.set_parameter_string(id, "delegation_unit", exec_ns)
    
    def create_child(self, parent_id, exec_ns, free_id, type, node_uuid=""):
        id = self.create_child_node(type, type, parent_id, True, node_uuid)
        if exec_ns != "":
            self.set_execution_unit(id, exec_ns)
        self.set_parameter_int32(id, "unique_node_id", free_id)
        return (id, free_id+1)

    def create_seq(self, parent_id, free_id, exec_ns="", node_uuid=""):
        return self.create_child(parent_id, exec_ns, free_id, "seq", node_uuid)

    def create_take_off(self, parent_id, free_id, exec_ns="", node_uuid=""):
        return self.create_child(parent_id, exec_ns, free_id, "take-off", node_uuid)

    def create_fly_to(self, parent_id, free_id, exec_ns="", node_uuid=""):
        return self.create_child(parent_id, exec_ns, free_id, "fly-to", node_uuid)
    

    def get_executor(self, node, id):
        # return MoveToExecutor(node, id)
        return SeqExecutor(node, id)

    
