from lrs_exec.executor import Executor

class TickedExecutor(Executor):
    def __init__(self, node, id, blocking=True):
        super().__init__(node, id)
        self.use_ticking = True
        self.inside_tick = False
        self.started_flag = False
        self.tick_finished = False
        self.first_tick = True
        self.blocking = blocking
        self.tick_finished = False
        
    def do_work_finished(self):
        self.tick_finished = True

    def do_work_begin(self):
        self.node.get_logger().info(f'Tick DEFAULT do_work_begin: {self.tni.type}')
        return

    def tick(self):
        ## self.node.get_logger().info(f'ticked_executor tick: {self.inside_tick} {self.started_flag}')
        if self.inside_tick:
            return
        #if self.finished:
        #    return
        if not self.started_flag:
            return
        self.inside_tick = True
        try:
            ## self.node.get_logger().error(f'Tick!!! {self.tni.type} - {self.id}')
            
            if self.tick_finished and not self.first_tick:
                self.node.get_logger().info(f'Tick tick_finished: {self.tni.type}')
                if self.normal_exit():
                    self.wait_for_postwork_conditions()
                else:
                    self.node.get_logger().error(f'Tick abnormal exit: {self.tni.type}')
                    unlock_res = self.unlock_resources()
                    if self.tni.use_execution_lock:
                        exec_unlock_res = self.execution_lock_release(self, tni.root_uuid)
                self.started_flag = False
                self.inside_tick = False
                return

            if not self.executor_wait_finished:
                self.ticked_wait_for_prework_conditions()
                if self.executor_wait_failed:
                    self.node.get_logger().error(f'Tick executor_wait_failed: {self.tni.type}')
                    self.fail("Executor wait failed")
                    self.do_work_finished()
                self.inside_tick = False
                return

            ### self.node.get_logger().info(f'Tick executor_wait_finished: {self.executor_wait_finished}')
            
            if self.executor_wait_finished and self.tni.abort_when_starting:
                self.node.get_logger().error(f'Tick ABORT: {self.executor_wait_finished}')
                self.abort_fail("abort_when_starting was true")
                self.do_work_finished()
                self.inside_tick = False
                return

            
            if self.executor_wait_finished and self.tni.enough_when_starting:
                self.node.get_logger().info(f'Tick ENOUGH: {self.executor_wait_finished}')
                self.do_work_finished()
                self.inside_tick = False
                return

            
            if self.first_tick:
                ## self.node.get_logger().info(f'FIRST Tick - {self.id}')
                self.do_work_begin()
                
            if not self.first_tick:
                ## self.node.get_logger().error(f'NOT FIRST Tick - {self.id}')
                if self.blocking:
                    self.tick_finished = True
                self.do_work()

            if self.first_tick:
                self.first_tick = False
                
        except Exception as exc:
            self.node.get_logger().error(f'Exception notify_node_status: {type(exc)} - {exc}')
            
        self.inside_tick = False
        
    def start(self):        
        self.node.get_logger().info(f'{__name__} start')
        self.started_flag = True
        
