import time
import rclpy
import os
import math

from lrs_exec.executor_factory import ExecutorFactory

from lrs_msgs_common.msg import TakeoffInfo
from std_msgs.msg import UInt8, Float32, String
from sensor_msgs.msg import NavSatFix, Joy
from geometry_msgs.msg import QuaternionStamped

class TickedExecutorFactory(ExecutorFactory):
    def __init__(self, name, ticked=True):
        super().__init__(name)
        self.name = name
        self.executors = []
        self.ticked = ticked
        self.tick_timer = self.create_timer(0.1, self.tick, callback_group=self.group)
        self.dji = True

    def add_executor(self, obj):
        self.executors_lock.acquire()                    
        self.executors.append(obj)
        self.executors_lock.release()
        self.get_logger().info(f'add_executor: {self.type}')                
        return obj
    
    def tick(self):
        # self.get_logger().error(f'take-off: removeTICK: {len(self.executors)}')        
        try:
            executors = self.executors.copy()
            if self.ticked:
                for obj in executors:
                    obj.tick()
            remove_list = []
            for obj in executors:                
                if obj.finished:
                    self.get_logger().info(f'{self.name}: queue to remove from executors: {len(self.executors)} - {obj.id}')        
                    remove_list.append(obj)
            self.executors_lock.acquire()                    
            for obj in remove_list:
                if obj in self.executors:
                    self.get_logger().info(f'Really remove Remove from {self.name} executors: {obj.id}')
                    self.executors.remove(obj)
            self.executors_lock.release()                                        
        except Exception as exc:
            self.get_logger().error(f'Seq fac tick Exception: {type(exc)} - {exc}')
