import os
import time
import rclpy
from rclpy.node import Node
from rclpy.executors import MultiThreadedExecutor
from rclpy.callback_groups import ReentrantCallbackGroup

from cv_bridge import CvBridge, CvBridgeError
from sensor_msgs.msg import CompressedImage

import cv2
import numpy as np
## import ffmpeg_streaming

import os

class ToRtmp(Node):

    def __init__(self):
        super().__init__('to_rtmp')

        self.declare_parameter("display_type", "all")
        self.display_type = self.get_parameter("display_type").value
        self.fifo = open('/tmp/fifo', 'wb')
        
        self.fps = 5
        self.graph = None
        self.width = 1280
        self.height = 720
        self.bridge = CvBridge()
        self.nsc_list = []
        self.tree_started = False

        self.can_be_aborted = {}
        self.can_be_paused = {}
        self.can_be_enoughed = {}
        self.can_be_signaled = {}
        
        self.group = ReentrantCallbackGroup()
        self.image_sub = self.create_subscription(CompressedImage, "tststream_exec/image_raw/compressed", self.image_callback, 10, callback_group=self.group)

        # self.timer = self.create_timer(1.0/self.fps, self.timer_callback, callback_group=self.group)         

    def wait_future(self, future, print_waiting=False):
        while not future.done():
            if print_waiting:
                print(f"wait_future executor")
        resp = future.result()
        return resp

    def get_tree_string(self, uuid, format):
        self.get_logger().info(f'get_tree_string: {uuid} {format}')
        try:
            req = TSTGetTreeStringFromUuid.Request()
            req.uuid = uuid
            req.type = format
            future = self.get_tree_string_client.call_async(req)
            resp = self.wait_future(future, print_waiting=False)
            return resp
        except Exception as exc:
            print("get_tree_string", type(exc))
            print(exc)
        return None

    def node_status(self, msg):
        try:
            # print("node_status:", msg)
            if not self.graph:
                return
            new_nsc_list = []
            for el in self.nsc_list:
                if self.update_node(el):
                    pass
                else:
                    new_nsc_list.append(el)
            if self.update_node(msg):
                pass
            else:
                new_nsc_list.append(msg)
            self.nsc_list = new_nsc_list
        except Exception as exc:
            self.get_logger().error(f'Exception node_status: {type(exc)} - {exc}')
            
    def image_callback(self, msg):
        try:
            # self.get_logger().info(f'image_callback:')
            img = self.bridge.compressed_imgmsg_to_cv2(msg)
            # reimg = cv2.resize(img, (640, 360) )
            #cv2.imshow("image", img)
            #cv2.waitKey(1)
            self.fifo.write(img.tostring())
            
        except Exception as exc:
            print("image_callback", type(exc))
            print(exc)
            
    def get_blank_image(self):
        time_text = time.strftime("%H:%M:%S")
        image2 = np.zeros((self.height,self.width,3), np.uint8)
        text_color = (255,255,255)
        cv2.putText(image2, time_text, (50,30), cv2.FONT_HERSHEY_PLAIN, 2.0, text_color, thickness=2)
        cv2.putText(image2, self.get_namespace(), (200,30), cv2.FONT_HERSHEY_PLAIN, 2.0, text_color, thickness=2)        
        return image2

    def timer_callback(self):
        try:
            # print("timer_callback")
            if self.graph:
                jpg = self.graph.create_jpg()
                msg = CompressedImage()
                msg.header.stamp = self.get_clock().now().to_msg()                
                msg.header.frame_id = self.display_type
                msg.format = "jpeg"
                msg.data = jpg

                image = self.bridge.compressed_imgmsg_to_cv2(msg, "bgr8")
                [h, w, c] = image.shape
                wscale = float(w)/self.width
                hscale = float(h)/self.height
                if wscale > hscale:
                    ws = self.width
                    hs = int(h/wscale)
                else:
                    ws = int(w/hscale)
                    hs = self.height

                # print(width, height, w, h, ws, hs)
                img = cv2.resize(image, (ws, hs))
                image2 = cv2.copyMakeBorder(img, 0, self.height-hs, 0, self.width-ws, cv2.BORDER_CONSTANT,value=[255, 255, 255])

                # overlay time of day
                time_text = time.strftime("%H:%M:%S")
                text_color = (0, 0, 0)
                cv2.putText(image2, time_text, (50,30), cv2.FONT_HERSHEY_PLAIN, 2.0, text_color, thickness=2)
                cv2.putText(image2, self.get_namespace(), (200,30), cv2.FONT_HERSHEY_PLAIN, 2.0, text_color, thickness=2)
#                cv2.putText(image2, display_type, (300,30), cv2.FONT_HERSHEY_PLAIN, 2.0, text_color, thickness=2)
#                cv2.putText(image2, display_title, (400,30), cv2.FONT_HERSHEY_PLAIN, 2.0, text_color, thickness=2)
                
                msg2 = self.bridge.cv2_to_compressed_imgmsg(image2)
                msg2.header = msg.header
                self.image_pub.publish(msg2)
                # self.image_pub.publish(msg2)
            else:
                image = self.get_blank_image()
                msg = self.bridge.cv2_to_compressed_imgmsg(image)
                msg.header.stamp = self.get_clock().now().to_msg()
                msg.header.frame_id = self.display_type
                self.image_pub.publish(msg)
        except Exception as exc:
            print("timer_callback", type(exc))
            print(exc)

def main(args=None):
    print('Hi from tst streamer server.')

    rclpy.init(args=args)

    try:
        os.mkfifo("/tmp/fifo")
    except:
        print("/tmp/fifo exist")

    executor = MultiThreadedExecutor(num_threads=8)
    node = ToRtmp()
    executor.add_node(node)
    try:    
        executor.spin()
    except KeyboardInterrupt:
        pass
    executor.shutdown()
    rclpy.shutdown()

if __name__ == '__main__':
    main()

        
