import sys
import time
import json
import threading
import uuid
import math

import rclpy
from rclpy.node import Node
from rclpy.executors import MultiThreadedExecutor
from rclpy.callback_groups import ReentrantCallbackGroup

from lrs_srvs_tst.srv import TSTExpandTree

from lrs_util.json_exec_node import JsonExecNode

from lrs_util.spot_jsonutil import json_basic_walkto, json_basic_graph_walkto
from lrs_util.jsonutil import json_basic_supp
from lrs_util.jsonutil import json_basic_start_data_recording, json_basic_stop_data_recording
from lrs_util.jsonutil import json_flyto, json_basic_wait, json_basic_seq, json_basic_conc, json_basic_flyto
from lrs_util.jsonutil import json_basic_moveto, json_basic_moveto_extra, json_moveto, json_search_position
from lrs_util.jsonutil import json_basic_search_area, json_search_area, get_common_params, json_basic_say
from lrs_srvs_tst.srv import TSTCreateTree, TSTDistributeTreeFromUuid, TSTSetStartExecutorFlagFromUuid
from lrs_msgs_common.srv import GetResponse
from lrs_msgs_common.msg import DisplayRequest
from geographic_msgs.msg import GeoPoint

from lrs_exec.exec_tree import ExecTree
from lrs_srvs_exec.srv import TSTExecute

class TstCommand(ExecTree):

    def __init__(self):
        super().__init__('tst_command')
        self.declare_parameter("command", "")
        self.declare_parameter("object", "medication")
        self.declare_parameter("bucket", "middle")
        self.declare_parameter("x", 30.0)
        self.declare_parameter("y", 20.0)
        self.declare_parameter("z", 10.0)
        self.declare_parameter("lat", 0.0)
        self.declare_parameter("lon", 0.0)
        self.declare_parameter("alt", 0.0)
        self.declare_parameter("yaw", 0.0)
        self.declare_parameter("pitch", -45.0)
        self.declare_parameter("speed", 7.0)
        self.declare_parameter("sspeed", "standard")
        self.declare_parameter("radius", 50.0)
        self.declare_parameter("target_type", "person")
        self.declare_parameter("target_size", 2.0)
        self.declare_parameter("area_type", "forrest")
        self.declare_parameter("vehicle", "dji")                
        self.declare_parameter("abort_tree", False)                
        self.declare_parameter("op0", False)                
        self.declare_parameter("unit", "")                
        self.declare_parameter("expand", True)                
        self.declare_parameter("name", "test")                
        self.declare_parameter("task_name", "say")                
        self.declare_parameter("height_above_takeoff", 17.0)
        self.declare_parameter("topic_string", "input")                        
        self.declare_parameter("string", "i am testing a speaker")                        
        self.declare_parameter("gain", 3)                        
        self.declare_parameter("channel", 0)                        
        self.declare_parameter("bt", False)                        

        self.vehicle = self.get_parameter("vehicle").value
        self.command = self.get_parameter("command").value
        self.object = self.get_parameter("object").value
        self.bucket = self.get_parameter("bucket").value
        self.x = self.get_parameter("x").value
        self.y = self.get_parameter("y").value
        self.z = self.get_parameter("z").value
        self.lat = self.get_parameter("lat").value
        self.lon = self.get_parameter("lon").value
        self.alt = self.get_parameter("alt").value
        self.speed = self.get_parameter("speed").value
        self.pitch = self.get_parameter("pitch").value
        self.yaw = self.get_parameter("yaw").value
        self.sspeed = self.get_parameter("sspeed").value
        self.radius = self.get_parameter("radius").value
        self.target_type = self.get_parameter("target_type").value
        self.target_size = self.get_parameter("target_size").value
        self.area_type = self.get_parameter("area_type").value
        self.abort_tree = self.get_parameter("abort_tree").value
        self.op0 = self.get_parameter("op0").value
        self.unit = self.get_parameter("unit").value
        self.expand = self.get_parameter("expand").value
        self.name = self.get_parameter("name").value
        self.task_name = self.get_parameter("name").value
        self.topic_string = self.get_parameter("topic_string").value
        self.string = self.get_parameter("string").value
        self.gain = self.get_parameter("gain").value
        self.channel = self.get_parameter("channel").value
        # print("STRING================", self.string)
        self.dataset_uri = "dummyuri-2"
        self.height_above_takeoff = self.get_parameter("height_above_takeoff").value
        self.bt = self.get_parameter("bt").value

        self.spot_delivery_json = """
 {
    "children": [
        {
            "children": [],
            "common_params": {
                "execunit": "/spot0",
                "use_lock": false
            },
            "name": "power-motor"
        },
        {
            "children": [],
            "common_params": {
                "execunit": "/spot0",
                "use_lock": false
            },
            "name": "stand",
            "params": {}
        },
        {
            "children": [],
            "common_params": {
                "execunit": "/spot0",
                "use_lock": false
            },
            "name": "localize-init",
            "params": {}
        },
        {
            "children": [
                {
                    "children": [],
                    "common_params": {
                        "execunit": "/spot0",
                        "node-uuid": "7c948c03-ad4f-4360-8687-cf34e4d6ef09",
                        "use_lock": false
                    },
                    "name": "graph-walk-to",
                    "params": {
                        "x": 38.0,
                        "y": 25.0,
                        "z": 1.0
                    }
                },
                {
                    "children": [],
                    "common_params": {
                        "execunit": "/spot0",
                        "node-uuid": "886e075e-c1e2-4140-a041-33932b804623",
                        "use_lock": false
                    },
                    "name": "pick-up"
                },
                {
                    "children": [],
                    "common_params": {
                        "execunit": "/spot0",
                        "node-uuid": "886e475e-c1e2-4140-a041-33932b804623",
                        "use_lock": false
                    },
                    "name": "add-to-bucket"
                }
            ],
            "common_params": {
                "execunit": "/spot0",
                "use_lock": false
            },
            "name": "seq",
            "params": {}
        },
        {
            "children": [
                {
                    "children": [],
                    "common_params": {
                        "execunit": "/spot0",
                        "node-uuid": "6c6ae06b-b9c6-42a1-8968-5631ef0a8ea4",
                        "use_lock": false
                    },
                    "name": "graph-walk-to",
                    "params": {
                        "x": 36.0,
                        "y": 30.0,
                        "z": 1.0
                    }
                },
                {
                    "children": [],
                    "common_params": {
                        "execunit": "/spot0",
                        "node-uuid": "54ff1994-02cd-4994-8d0d-e85dcbf310a6",
                        "use_lock": false
                    },
                    "name": "find-person"
                },
                {
                    "children": [],
                    "common_params": {
                        "execunit": "/spot0",
                        "node-uuid": "54ff1994-01cd-4994-8d0d-e85dcbf310a6",
                        "use_lock": false
                    },
                    "name": "pick-up-body-bucket"
                },
                {
                    "children": [],
                    "common_params": {
                        "execunit": "/spot0",
                        "node-uuid": "ae9f7dd2-7260-48c4-b9b9-0f03b63c16e9",
                        "use_lock": false
                    },
                    "name": "deliver-pkg"
                }
            ],
            "common_params": {
                "execunit": "/spot0",
                "use_lock": false
            },
            "name": "seq",
            "params": {}
        },
        {
            "children": [
                {
                    "children": [],
                    "common_params": {
                        "execunit": "/spot0",
                        "node-uuid": "a0612a2c-adac-4d9d-a19a-d0c1940ccb0f",
                        "use_lock": false
                    },
                    "name": "graph-walk-to",
                    "params": {
                        "x": 35.0,
                        "y": 28.0,
                        "z": 1.0
                    }
                },
                {
                    "children": [],
                    "common_params": {
                        "execunit": "/spot0",
                        "node-uuid": "88bfcb86-0393-4f0e-9db6-470ef9fa2454",
                        "use_lock": false
                    },
                    "name": "pick-up"
                },
                {
                    "children": [],
                    "common_params": {
                        "execunit": "/spot0",
                        "node-uuid": "887fcb86-0393-4f0e-9db6-470ef9fa2454",
                        "use_lock": false
                    },
                    "name": "add-to-bucket"
                }
            ],
            "common_params": {
                "execunit": "/spot0",
                "use_lock": false
            },
            "name": "seq",
            "params": {}
        },
        {
            "children": [
                {
                    "children": [],
                    "common_params": {
                        "execunit": "/spot0",
                        "node-uuid": "743a3248-107a-4598-bc08-24c6e67c1cb0",
                        "use_lock": false
                    },
                    "name": "graph-walk-to",
                    "params": {
                        "x": 36.0,
                        "y": 30.0,
                        "z": 1.0
                    }
                },
                {
                    "children": [],
                    "common_params": {
                        "execunit": "/spot0",
                        "node-uuid": "4c635828-e885-4d35-9401-7facd0c84be3",
                        "use_lock": false
                    },
                    "name": "find-person"
                },
                {
                    "children": [],
                    "common_params": {
                        "execunit": "/spot0",
                        "node-uuid": "4c635828-e885-4d35-a401-7facd0c84be3",
                        "use_lock": false
                    },
                    "name": "pick-up-body-bucket"
                },
                {
                    "children": [],
                    "common_params": {
                        "execunit": "/spot0",
                        "node-uuid": "cab2ebb3-170c-4a78-b917-f1fa30ed71c2",
                        "use_lock": false
                    },
                    "name": "deliver-pkg"
                }
            ],
            "common_params": {
                "execunit": "/spot0",
                "use_lock": false
            },
            "name": "seq",
            "params": {}
        },
        {
            "children": [],
            "common_params": {
                "execunit": "/spot0",
                "node-uuid": "51001a6b-f072-4d4a-ae97-c339eca28397",
                "use_lock": false
            },
            "name": "graph-walk-to",
            "params": {
                "x": 48.3,
                "y": 11.0,
                "z": 1.0
            }
        }
    ],
    "common_params": {
        "execunit": "/spot0",
        "node-uuid": "ef55a87a-4ec2-43c6-b602-ea3550795f05",
        "use_lock": false
    },
    "name": "seq",
    "params": {}
}
"""

        self.spot_delivery_bucket_json = """
{
  "children": [
    {
      "children": [],
      "common_params": {
        "execunit": "/spot0",
        "use_lock": false
      },
      "name": "power-motor"
    },
    {
      "children": [],
      "common_params": {
        "execunit": "/spot0",
        "use_lock": false
      },
      "name": "stand",
      "params": {}
    },
    {
      "children": [],
      "common_params": {
        "execunit": "/spot0",
        "use_lock": false
      },
      "name": "localize-init",
      "params": {}
    },
    {
      "children": [
        {
          "children": [],
          "common_params": {
            "execunit": "/spot0",
            "use_lock": false,
            "node-uuid": "dcca5fdd-2dff-4467-8c33-1e7e8d0fc118"
          },
          "name": "walk-to",
          "params": {
            "x": 109.44928866694681,
            "y": 29.641112674959004,
            "z": 1.0
          }
        },
        {
          "children": [],
          "common_params": {
            "execunit": "/spot0",
            "use_lock": false,
            "node-uuid": "84132605-885e-4bfb-a9e1-2a3bcb66e2ab"
          },
          "name": "pick-up"
        },
        {
          "children": [],
          "common_params": {
            "execunit": "/spot0",
            "use_lock": false,
            "node-uuid": "c25a564b-b2a9-4962-8f67-cb9d7dbfb6b5"
          },
          "name": "query",
          "params": {
            "language": "kdb/krql",
            "query": "SALIENT REGION:\n  action: add to property\n  dataset: 'granso'\n  object: '{ed459194-0480-4f21-bf30-940cf10e5e0e}'\n  property: 'http://askco.re/thing#medication'\n  amount: -1",
            "bindings": "{}"
          }
        },
        {
          "children": [],
          "common_params": {
            "execunit": "/spot0",
            "use_lock": false,
            "node-uuid": "3a7d9b26-b858-4847-8057-475b88c2ceff"
          },
          "name": "query",
          "params": {
            "language": "kdb/kdql",
            "query": "update:\n  from: granso_state\n  matches:\n    name: spot0\n  values:\n    gripper_empty: false",
            "bindings": "{}"
          }
        }
      ],
      "common_params": {
        "execunit": "/spot0",
        "use_lock": false
      },
      "name": "seq",
      "params": {}
    },
    {
      "children": [
        {
          "children": [],
          "common_params": {
            "execunit": "/spot0",
            "use_lock": false,
            "node-uuid": "66642477-991f-4609-9069-7ff3f01454cb"
          },
          "name": "add-to-bucket"
        },
        {
          "children": [],
          "common_params": {
            "execunit": "/spot0",
            "use_lock": false,
            "node-uuid": "903eaa53-958c-4965-8d3a-a87aaa7d7a57"
          },
          "name": "query",
          "params": {
            "language": "kdb/kdql",
            "query": "update:\n  from: granso_state\n  matches:\n    name: spot0\n  values:\n    gripper_empty: true\n    bucket_empty: false",
            "bindings": "{}"
          }
        }
      ],
      "common_params": {
        "execunit": "/spot0",
        "use_lock": false
      },
      "name": "seq",
      "params": {}
    },
    {
      "children": [
        {
          "children": [],
          "common_params": {
            "execunit": "/spot0",
            "use_lock": false,
            "node-uuid": "e80623ea-b4de-4149-a598-08365e0045ab"
          },
          "name": "walk-to",
          "params": {
            "x": 12.086530111730099,
            "y": 27.71860129851848,
            "z": 1.0
          }
        },
        {
          "children": [],
          "common_params": {
            "execunit": "/spot0",
            "use_lock": false,
            "node-uuid": "6ad75c67-1044-42d1-a163-32646379a661"
          },
          "name": "find-person"
        },
        {
          "children": [],
          "common_params": {
            "execunit": "/spot0",
            "use_lock": false,
            "node-uuid": "07e465fd-0c2f-4b06-bb21-d3626f20bfb3"
          },
          "name": "pick-up-body-bucket"
        },
        {
          "children": [],
          "common_params": {
            "execunit": "/spot0",
            "use_lock": false,
            "node-uuid": "1662021b-1341-4af3-993b-d8a0ce91baa5"
          },
          "name": "query",
          "params": {
            "language": "kdb/kdql",
            "query": "update:\n  from: granso_state\n  matches:\n    name: spot0\n  values:\n    gripper_empty: false\n    bucket_empty: true",
            "bindings": "{}"
          }
        },
        {
          "children": [],
          "common_params": {
            "execunit": "/spot0",
            "use_lock": false,
            "node-uuid": "00583d6e-9117-43e3-b4da-1d582d10bbab"
          },
          "name": "deliver-pkg"
        },
        {
          "children": [],
          "common_params": {
            "execunit": "/spot0",
            "use_lock": false,
            "node-uuid": "45b98b0f-7e67-48ef-9225-0cc7b2d8ced0"
          },
          "name": "query",
          "params": {
            "language": "kdb/krql",
            "query": "SALIENT REGION:\n  action: set property\n  dataset: 'granso'\n  object: '{8b9fcd1f-f951-4bd2-bdb6-e91a5639335d}'\n  property: http://askco.re/thing/property#needs\n  value: nothing",
            "bindings": "{}"
          }
        },
        {
          "children": [],
          "common_params": {
            "execunit": "/spot0",
            "use_lock": false,
            "node-uuid": "f1d165f2-4dd7-4e41-9f6b-af2720efb1cd"
          },
          "name": "query",
          "params": {
            "language": "kdb/kdql",
            "query": "update:\n  from: granso_state\n  matches:\n    name: spot0\n  values:\n    gripper_empty: true",
            "bindings": "{}"
          }
        }
      ],
      "common_params": {
        "execunit": "/spot0",
        "use_lock": false
      },
      "name": "seq",
      "params": {}
    },
    {
      "children": [
        {
          "children": [],
          "common_params": {
            "execunit": "/spot0",
            "use_lock": false,
            "node-uuid": "8ea7d1fc-cce5-43db-b4e0-eb3f6ac6610f"
          },
          "name": "walk-to",
          "params": {
            "x": 117.02988214662764,
            "y": 19.956053422763944,
            "z": 1.0
          }
        },
        {
          "children": [],
          "common_params": {
            "execunit": "/spot0",
            "use_lock": false,
            "node-uuid": "9aaa77ee-49d3-4076-9459-c1821ad7213a"
          },
          "name": "pick-up"
        },
        {
          "children": [],
          "common_params": {
            "execunit": "/spot0",
            "use_lock": false,
            "node-uuid": "31dd80f1-b51a-4014-9e7e-ae008744063c"
          },
          "name": "query",
          "params": {
            "language": "kdb/krql",
            "query": "SALIENT REGION:\n  action: add to property\n  dataset: 'granso'\n  object: '{ed459194-0480-4f21-bf30-940cf10e5e0e}'\n  property: 'http://askco.re/thing#medication'\n  amount: -1",
            "bindings": "{}"
          }
        },
        {
          "children": [],
          "common_params": {
            "execunit": "/spot0",
            "use_lock": false,
            "node-uuid": "7b6112c9-af6e-4773-9d96-9a92d600deeb"
          },
          "name": "query",
          "params": {
            "language": "kdb/kdql",
            "query": "update:\n  from: granso_state\n  matches:\n    name: spot0\n  values:\n    gripper_empty: false",
            "bindings": "{}"
          }
        }
      ],
      "common_params": {
        "execunit": "/spot0",
        "use_lock": false
      },
      "name": "seq",
      "params": {}
    },
    {
      "children": [
        {
          "children": [],
          "common_params": {
            "execunit": "/spot0",
            "use_lock": false,
            "node-uuid": "12a62ca9-40eb-40f2-bee9-8b96e8d421f6"
          },
          "name": "add-to-bucket"
        },
        {
          "children": [],
          "common_params": {
            "execunit": "/spot0",
            "use_lock": false,
            "node-uuid": "836a5e14-1da3-47e5-a813-7118e2989dd0"
          },
          "name": "query",
          "params": {
            "language": "kdb/kdql",
            "query": "update:\n  from: granso_state\n  matches:\n    name: spot0\n  values:\n    gripper_empty: true\n    bucket_empty: false",
            "bindings": "{}"
          }
        }
      ],
      "common_params": {
        "execunit": "/spot0",
        "use_lock": false
      },
      "name": "seq",
      "params": {}
    },
    {
      "children": [
        {
          "children": [],
          "common_params": {
            "execunit": "/spot0",
            "use_lock": false,
            "node-uuid": "a34fc3fd-fbab-4261-a529-ea45aea92ad8"
          },
          "name": "walk-to",
          "params": {
            "x": 4.2045988800236955,
            "y": 14.58712319843471,
            "z": 1.0
          }
        },
        {
          "children": [],
          "common_params": {
            "execunit": "/spot0",
            "use_lock": false,
            "node-uuid": "140e9389-d156-4c49-a3a3-d9ae9664c43d"
          },
          "name": "find-person"
        },
        {
          "children": [],
          "common_params": {
            "execunit": "/spot0",
            "use_lock": false,
            "node-uuid": "d83be8f7-bb15-4a44-a346-30096c6dc11a"
          },
          "name": "pick-up-body-bucket"
        },
        {
          "children": [],
          "common_params": {
            "execunit": "/spot0",
            "use_lock": false,
            "node-uuid": "720880d4-93e6-406d-b791-ffb24c81e45c"
          },
          "name": "query",
          "params": {
            "language": "kdb/kdql",
            "query": "update:\n  from: granso_state\n  matches:\n    name: spot0\n  values:\n    gripper_empty: false\n    bucket_empty: true",
            "bindings": "{}"
          }
        },
        {
          "children": [],
          "common_params": {
            "execunit": "/spot0",
            "use_lock": false,
            "node-uuid": "0253ffdb-f3e1-4890-823c-80ef6143a0dc"
          },
          "name": "deliver-pkg"
        },
        {
          "children": [],
          "common_params": {
            "execunit": "/spot0",
            "use_lock": false,
            "node-uuid": "021e5e83-ddc6-47b4-9f0b-10622796e314"
          },
          "name": "query",
          "params": {
            "language": "kdb/krql",
            "query": "SALIENT REGION:\n  action: set property\n  dataset: 'granso'\n  object: '{9a754f6b-be32-4f4d-aebe-c657b43c7e75}'\n  property: http://askco.re/thing/property#needs\n  value: nothing",
            "bindings": "{}"
          }
        },
        {
          "children": [],
          "common_params": {
            "execunit": "/spot0",
            "use_lock": false,
            "node-uuid": "a9780901-225d-4c3f-8c49-aed721f4a1b2"
          },
          "name": "query",
          "params": {
            "language": "kdb/kdql",
            "query": "update:\n  from: granso_state\n  matches:\n    name: spot0\n  values:\n    gripper_empty: true",
            "bindings": "{}"
          }
        }
      ],
      "common_params": {
        "execunit": "/spot0",
        "use_lock": false
      },
      "name": "seq",
      "params": {}
    },
    {
      "children": [],
      "common_params": {
        "execunit": "/spot0",
        "use_lock": false,
      },
      "name": "walk-to",
      "params": {
        "x": 12.086530111730099,
        "y": 27.71860129851848,
        "z": 1.0
      }
    }
  ],
  "common_params": {
    "execunit": "/spot0",
    "use_lock": false,
    "node-uuid": "368e7172-c264-4bfc-94d5-060aa38556e2"
  },
  "name": "seq",
  "params": {}
}
"""

        self.explore1_json = """
{
    "children": [

        {
            "children": [
            ],
            "common_params": {
                "execunit": "/dji0",
                "use_lock": false 
            },
            "name": "search-area",
            "params": {
                "area": [
                    {
                        "altitude": 49.8,
                        "latitude": 57.75981397308571,
                        "longitude": 16.682642698287967,
                        "rostype": "GeoPoint"
                    },
                    {
                        "altitude": 49.8,
                        "latitude": 57.759999986629374,
                        "longitude": 16.68306112289429,
                        "rostype": "GeoPoint"
                    },
                    {
                        "altitude": 49.8,
                        "latitude": 57.76071541441235,
                        "longitude": 16.68261051177979,
                        "rostype": "GeoPoint"
                    },
                    {
                        "altitude": 49.8,
                        "latitude": 57.760489340763655,
                        "longitude": 16.681650280952457,
                        "rostype": "GeoPoint"
                    },
                    {
                        "altitude": 49.8,
                        "latitude": 57.75994275179483,
                        "longitude": 16.682025790214542,
                        "rostype": "GeoPoint"
                    }
                ],
                "area-type": "forrest",
                "camera-data-recording": false,
                "speed": "standard",
                "target-size": 2.0,
                "target-type": "person"
            }
        }
    ],
    "common_params": {
        "execunit": "/dji0",
        "node-uuid": "2f39ef04-5911-4e19-b3c1-18274fe60627",
        "use_lock": false 
    },
    "name": "seq",
    "params": {}
}
"""

        self.explore1_json_debug = """
{
    "children": [
        {
            "children": [
            ],
            "common_params": {
                "execunit": "/dji0",
                "use_lock": false 
            },
            "name": "search-area",
            "params": {
                "area": [
                    {
                        "altitude": 49.8,
                        "latitude": 57.75981397308571,
                        "longitude": 16.682642698287967,
                        "rostype": "GeoPoint"
                    },
                    {
                        "altitude": 49.8,
                        "latitude": 57.759999986629374,
                        "longitude": 16.68306112289429,
                        "rostype": "GeoPoint"
                    },
                    {
                        "altitude": 49.8,
                        "latitude": 57.76071541441235,
                        "longitude": 16.68261051177979,
                        "rostype": "GeoPoint"
                    },
                    {
                        "altitude": 49.8,
                        "latitude": 57.760489340763655,
                        "longitude": 16.681650280952457,
                        "rostype": "GeoPoint"
                    },
                    {
                        "altitude": 49.8,
                        "latitude": 57.75994275179483,
                        "longitude": 16.682025790214542,
                        "rostype": "GeoPoint"
                    }
                ],
                "area-type": "forrest",
                "camera-data-recording": false,
                "speed": "standard",
                "target-size": 2.0,
                "target-type": "person"
            }
        }
    ],
    "common_params": {
        "execunit": "/dji0",
        "node-uuid": "2f39ef04-5911-4e19-b3c1-18274fe60627",
        "use_lock": false 
    },
    "name": "seq",
    "params": {}
}
"""
        
        self.explore2_json = """
{                                                             
    "children": [                                                               
        {                                                                       
            "children": [],                                                     
            "common_params": {                                                  
                "execunit": "/dji0",
                "node-uuid": "6ff943c5-6c4c-4b96-a7f3-b35054b85cbb",
                "use_lock": false
            },
            "name": "search-area",
            "params": {
                "area": [
                    {
                        "altitude": 49.8,
                        "latitude": 57.76054842589023,
                        "longitude": 16.68343272017277,
                        "rostype": "GeoPoint"
                    },
                    {
                        "altitude": 49.8,
                        "latitude": 57.76126670721238,
                        "longitude": 16.683466128235636,
                        "rostype": "GeoPoint"
                    },
                    {
                        "altitude": 49.8,
                        "latitude": 57.760557351900005,
                        "longitude": 16.6827607783,
                        "rostype": "GeoPoint"
                    }
                ],
                "area-type": "forrest",
                "camera-data-recording": false,
                "speed": "standard",
                "target-size": 2.0,
                "target-type": "person"
            }
        }
    ],
    "common_params": {
        "execunit": "/dji0",
        "node-uuid": "2f39ef04-5911-4e19-b3c1-18274fe60627",
        "use_lock": false
    },
    "name": "seq",
    "params": {}
}

"""
        
        self.xyz_paths = {
            "path_1": [[-37.9587778479746, -486.546494806202, 20.0], [-103.12935453959095, -487.9953071380142, 20.0], [-125.64868459036094, -470.19666795275475, 20.0], [-30.742560017675753, -468.08680226450014, 20.0], [-16.95772425460381, -447.5753594571252, 20.0], [-146.1236022803005, -450.44685647975405, 20.0], [-145.15588696906983, -430.2203521079683, 20.0], [8.044296429791705, -426.8145465096159, 20.0], [30.757835890233313, -406.10460897224874, 20.0], [-132.0182824511506, -409.7232979504584, 20.0], [-118.88067793323141, -389.2262437929485, 20.0], [30.99858442281937, -385.89426585738573, 20.0], [31.2393329554054, -365.6839227425227, 20.0], [-106.3985864468276, -368.74376239836215, 20.0], [-112.73983311457158, -348.6797441481584, 20.0], [31.480081487991438, -345.4735796276597, 20.0], [31.720830020577452, -325.2632365127967, 20.0], [-119.08110895206691, -328.61572654642976, 20.0], [-126.91141557334124, -314.7119429084116, 20.0], [-71.86420358056002, -325.7547880939428, 20.0]],
            "path_2": [[29.562162424650268, -165.14692992478072, 20.0], [30.259908756578852, -305.4105785572736, 20.0], [13.205406193887313, -305.8439647642709, 20.0], [12.512055827138918, -166.46400994315195, 20.0], [-7.674096808586087, -169.29790280184503, 20.0], [-6.992610055163849, -306.29298848794053, 20.0], [-27.19062630421502, -306.7420122116102, 20.0], [-27.86024944431109, -172.1317956605381, 20.0], [-48.04640208003609, -174.96568851923115, 20.0], [-47.38864255326618, -307.19103593527984, 20.0], [-67.58824742998436, -307.32070762125585, 20.0], [-68.23636417456117, -177.03378933264347, 20.0], [-88.43368032799151, -177.62354896803015, 20.0], [-87.80714893565124, -303.5712968114909, 20.0], [-108.02757734815695, -299.5149413220667, 20.0], [-108.63099648142183, -178.2133086034168, 20.0], [-128.82831263485218, -178.8030682388034, 20.0], [-128.2480057606627, -295.4585858326425, 20.0], [-139.5639644799449, -174.90367138926624, 20.0]],
            "path_3": [[31.57103781749015, -66.2066591554693, 20.0], [28.394850042055154, -49.00909702129907, 20.0], [6.387961375051699, -23.05295188036238, 20.0], [28.991864229384767, -145.44244875414262, 20.0], [9.041796341932699, -148.64542389380762, 20.0], [-16.11467005336528, -12.434996646617755, 20.0], [-38.61730148178226, -1.817041412873245, 20.0], [-10.98068049612334, -151.45633864059766, 20.0], [-31.003157334179367, -154.26725338738774, 20.0], [-58.91195983470509, -3.154221341496517, 20.0], [-77.61667718624919, -13.100183588454378, 20.0], [-51.025634172235385, -157.0781681341778, 20.0], [-71.21159633162766, -159.00388679985153, 20.0], [-96.11150098828753, -24.182620632999665, 20.0], [-114.60632479032587, -35.26505767754498, 20.0], [-91.64303185834794, -159.60048269779796, 20.0], [-112.07446738506822, -160.19707859574441, 20.0], [-132.66888384582887, -48.68800490679873, 20.0], [-150.43713898892543, -63.70446932640399, 20.0], [-132.50590291178852, -160.79367449369087, 20.0], [-160.18179346483882, -67.73846739642882, 20.0]],
            "path_4": [[-49.81501562250395, 32.775888828238145, 20.0], [-49.8909467612376, 18.74968503630722, 20.0], [109.42845034335453, 149.89685689861452, 20.0], [121.90369746771809, 134.00255239028024, 20.0], [-35.44067041256139, 4.481168108275946, 20.0], [-15.237535874339681, -5.051760464569074, 20.0], [134.37894459208172, 118.10824788194599, 20.0], [146.8541917164453, 102.21394337361168, 20.0], [4.965598663882055, -14.584689037414105, 20.0], [19.413498062770465, -23.399351323570954, 20.0], [142.77131524685913, 90.33195352408524, 20.0], [159.87582535477517, 86.53108395758414, 20.0], [27.25540700395376, -35.739993569446305, 20.0], [39.03655286185071, -52.353299951793964, 20.0], [172.40821914425496, 70.61040013838962, 20.0]],
            "path_5": [[-249.96549713990083, 82.32201325585758, 20.0], [-157.9234891531235, -53.52562530719419, 20.0], [-142.40793043544548, -40.41297954890591, 20.0], [-233.08977684538803, 93.4271545896037, 20.0], [-216.2140565508752, 104.53229592334985, 20.0], [-126.89237171776746, -27.300333790617632, 20.0], [-110.74788140163963, -15.11594757746406, 20.0], [-199.33833625636237, 115.63743725709593, 20.0], [-182.46261596184956, 126.74257859084204, 20.0], [-93.39368911222692, -4.716997819145591, 20.0], [-76.03949682281421, 5.68195193917289, 20.0], [-165.5868956673367, 137.84771992458812, 20.0], [-148.7111753728239, 148.9528612583342, 20.0], [-59.01779365400223, 16.571632645994836, 20.0], [-55.94782131868024, 33.365711273283075, 20.0], [-137.95129337912078, 154.39716503059802, 20.0]],
            "test": [[140, 0, 40], [140, 80, 40], [0, 0, 40]] ,
            "path_6": [[-37.043831376439826, -485.0729716294393, 20.0], [-112.138085335913, -482.8802268356337, 20.0], [-136.01092289389197, -461.28717480018304, 20.0], [-29.790674171180953, -464.3887955647911, 20.0], [-14.529871403947547, -444.6257997264702, 20.0], [-145.96034870613053, -440.7880427789511, 20.0], [-138.38197670043786, -420.8007207216729, 20.0], [8.270122821075109, -425.0829475335937, 20.0], [29.74749550969293, -405.50147492257327, 20.0], [-125.73089303892544, -400.96152134339206, 20.0], [-113.07980937741299, -381.1223219651112, 20.0], [30.31632082435522, -385.30947479469353, 20.0], [30.606970073873118, -365.1093519532046, 20.0], [-109.26997888030527, -361.0249588682397, 20.0], [-115.65882526342503, -340.6297954144343, 20.0], [30.897619323390906, -344.9092291117157, 20.0], [31.188268572908722, -324.7091062702268, 20.0], [-122.04770619324329, -320.2346309518686, 20.0], [-125.43927954855562, -308.5960671341634, 20.0], [-68.94988097829636, -316.52649876058814, 20.0]],
            "path_7": [[30.31560813403258, -64.80716885745242, 20.0], [10.518368371809073, -40.989052642898734, 20.0], [30.001118683082936, -145.8248043178485, 20.0], [10.06418482760715, -149.10140952087158, 20.0], [-15.206642301907717, -13.120292109672638, 20.0], [-37.55453876058273, -3.423622352623701, 20.0], [-9.95926781391165, -151.9124612561394, 20.0], [-29.98272045543044, -154.72351299140726, 20.0], [-57.01725558435075, -9.251966821502908, 20.0], [-76.19898275136862, -16.592303252894673, 20.0], [-50.00617309694923, -157.5345647266751, 20.0], [-70.17434243448986, -159.5669028119207, 20.0], [-95.38070991838653, -23.93263968428632, 20.0], [-114.55311026423875, -31.32316329623611, 20.0], [-90.60931193869395, -160.16360190511995, 20.0], [-111.04428144289805, -160.7603009983192, 20.0], [-132.93414203095097, -42.97200354832042, 20.0], [-151.3151737976631, -54.62084380040469, 20.0], [-131.47925094710214, -161.35700009151842, 20.0], [-161.704461479205, -61.32790326118076, 20.0]],
            "path_8": [[28.70796996042181, -165.78249538523113, 20.0], [28.70796996042181, -304.9965988051475, 20.0], [13.4, -304.4774273348414, 20.0], [13.4, -166.87051575738676, 20.0], [-6.8, -169.70635261674386, 20.0], [-6.8, -303.8875893312736, 20.0], [-27.0, -303.2977513277058, 20.0], [-27.0, -172.54218947610096, 20.0], [-47.2, -175.37802633545806, 20.0], [-47.2, -302.70791332413796, 20.0], [-67.4, -302.00701159443287, 20.0], [-67.4, -177.53886944516415, 20.0], [-87.6, -178.1287074520251, 20.0], [-87.6, -299.1711747310869, 20.0], [-107.8, -296.33533786774103, 20.0], [-107.8, -178.7185454588861, 20.0], [-128.0, -179.30838346574708, 20.0], [-128.0, -293.49950100439514, 20.0], [-138.67636851158642, -174.8542314881846, 20.0]],
            "path_9": [[-32.709976409534626, 38.80567362689767, 20.0], [91.32564758342077, 151.35651762050523, 20.0], [109.14795968038521, 149.05780731211905, 20.0], [-34.866119139979844, 18.378365419565704, 20.0], [-21.269741049477787, 3.439175347798333, 20.0], [121.70688205148885, 133.17722223571238, 20.0], [134.25320905583328, 117.28520803020942, 20.0], [-7.567984584978873, -11.404393606823739, 20.0], [6.133771879520069, -26.24796256144578, 20.0], [146.79953606017773, 101.39319382470646, 20.0], [159.34586306452218, 85.50117961920351, 20.0], [20.26262702190023, -40.7039790120645, 20.0], [35.163953024740394, -54.45904968189545, 20.0], [171.8921900688666, 69.60916541370055, 20.0]],
            "path_10": [[-249.10008823158176, 81.3937665434216, 20.0], [-163.2004397768669, -44.151873505777075, 20.0], [-146.12768140793406, -33.33214165251411, 20.0], [-232.24671202579165, 92.53413386589307, 20.0], [-215.39333582000143, 103.67450118836452, 20.0], [-129.05492303900124, -22.512409799251152, 20.0], [-110.85871782540536, -13.334638718957283, 20.0], [-198.53995961421128, 114.814868510836, 20.0], [-181.68658340842114, 125.95523583330741, 20.0], [-91.46169196024196, -5.91191320633903, 20.0], [-72.06466609507855, 1.510812306279229, 20.0], [-164.833207202631, 137.09560315577886, 20.0], [-147.97983099684086, 148.23597047825032, 20.0], [-52.667640229915136, 8.933537818897497, 20.0], [-41.52416644008572, 16.259548720209473, 20.0], [-136.7932865046134, 155.49903189144203, 20.0]]
        }

        
        #self.cli = self.create_client(AddTwoInts, 'add_two_ints')
        #while not self.cli.wait_for_service(timeout_sec=1.0):
        #    self.get_logger().info('service not available, waiting again...')
        #self.req = AddTwoInts.Request()
        self.ns = self.get_namespace()
        if not self.unit:
            self.unit = self.ns
        print("NAMESPACE:", self.ns)
        print("UNIT:", self.unit)

        self.root_ns = self.ns
        if self.op0:
            self.root_ns = "/op0"
        print("ROOT NAMESPACE:", self.root_ns)

        execserv = f'{self.root_ns}/tst_execute'
        if self.bt:
            execserv = f'{self.root_ns}/bt_execute'
        self.tst_execute_client = self.create_client(TSTExecute, execserv, callback_group=self.group)

        print(f"Wait for service {execserv}...")
        self.tst_execute_client.wait_for_service()
        print("Services are up and running...")
        
        self.timer = self.create_timer(0.1, self.do_command, callback_group=self.group)
        if self.abort_tree:
            self.timer2 = self.create_timer(7.0, self.do_signal, callback_group=self.group)

    def deliver_pkg(self, new=True):
        try:
            deliver_pkg = JsonExecNode(self.unit, name="deliver-pkg")
            tree = JsonExecNode(self.unit, name="seq", node_uuid=str(uuid.uuid4()))            
            tree.add_child(deliver_pkg)
            
            if new:
                resp = self.new_handle_tree(tree.get_json())
                print("RESP:", resp)
            else:
                (res, reason) = self.handle_tree(tree.get_json(), expand=self.expand)
                print(res, reason)
        except Exception as exc:
            self.get_logger().error(f'EXCEPTION pick_up: {type(exc)} - {exc}')            

    def dock(self, new=True):
        try:
            dock = JsonExecNode(self.unit, name="dock")
            tree = JsonExecNode(self.unit, name="seq", node_uuid=str(uuid.uuid4()))            
            tree.add_child(dock)
            
            if new:
                resp = self.new_handle_tree(tree.get_json())
                print("RESP:", resp)
            else:
                (res, reason) = self.handle_tree(tree.get_json(), expand=self.expand)
                print(res, reason)
        except Exception as exc:
            self.get_logger().error(f'EXCEPTION pick_up: {type(exc)} - {exc}')
            
    def undock(self, new=True):
        try:
            dock = JsonExecNode(self.unit, name="undock")
            tree = JsonExecNode(self.unit, name="seq", node_uuid=str(uuid.uuid4()))            
            tree.add_child(dock)
            
            if new:
                resp = self.new_handle_tree(tree.get_json())
                print("RESP:", resp)
            else:
                (res, reason) = self.handle_tree(tree.get_json(), expand=self.expand)
                print(res, reason)
        except Exception as exc:
            self.get_logger().error(f'EXCEPTION pick_up: {type(exc)} - {exc}')            

    def pick_up(self, new=True):
        try:
            pick_up = JsonExecNode(self.unit, name="pick-up")
            tree = JsonExecNode(self.ns, name="seq", node_uuid=str(uuid.uuid4()))            
            tree.add_child(pick_up)
            
            if new:
                resp = self.new_handle_tree(tree.get_json())
                print("RESP:", resp)
            else:
                (res, reason) = self.handle_tree(tree.get_json(), expand=self.expand)
                print(res, reason)
        except Exception as exc:
            self.get_logger().error(f'EXCEPTION pick_up: {type(exc)} - {exc}')            

            
    def pick_up_body_bucket(self, new=True):
        try:
            pick_up = JsonExecNode(self.unit, name="pick-up-body-bucket", bucket=self.bucket)
            tree = JsonExecNode(self.unit, name="seq", node_uuid=str(uuid.uuid4()))            
            tree.add_child(pick_up)
            
            if new:
                resp = self.new_handle_tree(tree.get_json())
                print("RESP:", resp)
            else:
                (res, reason) = self.handle_tree(tree.get_json(), expand=self.expand)
                print(res, reason)
        except Exception as exc:
            self.get_logger().error(f'EXCEPTION pick_up_body_bucket: {type(exc)} - {exc}')            

    def arm_carry(self, new=True):
        try:
            pick_up = JsonExecNode(self.unit, name="arm-carry")
            tree = JsonExecNode(self.unit, name="seq", node_uuid=str(uuid.uuid4()))            
            tree.add_child(pick_up)
            
            if new:
                resp = self.new_handle_tree(tree.get_json())
                print("RESP:", resp)
            else:
                (res, reason) = self.handle_tree(tree.get_json(), expand=self.expand)
                print(res, reason)
        except Exception as exc:
            self.get_logger().error(f'EXCEPTION arm_carry: {type(exc)} - {exc}')            

    def arm_lower(self, new=True):
        try:
            pick_up = JsonExecNode(self.unit, name="arm-lower")
            tree = JsonExecNode(self.unit, name="seq", node_uuid=str(uuid.uuid4()))            
            tree.add_child(pick_up)
            
            if new:
                resp = self.new_handle_tree(tree.get_json())
                print("RESP:", resp)
            else:
                (res, reason) = self.handle_tree(tree.get_json(), expand=self.expand)
                print(res, reason)
        except Exception as exc:
            self.get_logger().error(f'EXCEPTION arm_lower: {type(exc)} - {exc}')            

    def arm_stow(self, new=True):
        try:
            pick_up = JsonExecNode(self.unit, name="arm-stow")
            tree = JsonExecNode(self.unit, name="seq", node_uuid=str(uuid.uuid4()))            
            tree.add_child(pick_up)
            
            if new:
                resp = self.new_handle_tree(tree.get_json())
                print("RESP:", resp)
            else:
                (res, reason) = self.handle_tree(tree.get_json(), expand=self.expand)
                print(res, reason)
        except Exception as exc:
            self.get_logger().error(f'EXCEPTION arm_stow: {type(exc)} - {exc}')            

    def gripper_close(self, new=True):
        try:
            pick_up = JsonExecNode(self.unit, name="gripper-close")
            tree = JsonExecNode(self.unit, name="seq", node_uuid=str(uuid.uuid4()))            
            tree.add_child(pick_up)
            
            if new:
                resp = self.new_handle_tree(tree.get_json())
                print("RESP:", resp)
            else:
                (res, reason) = self.handle_tree(tree.get_json(), expand=self.expand)
                print(res, reason)
        except Exception as exc:
            self.get_logger().error(f'EXCEPTION gripper-close: {type(exc)} - {exc}')            

    def gripper_open(self, new=True):
        try:
            pick_up = JsonExecNode(self.unit, name="gripper-open")
            tree = JsonExecNode(self.unit, name="seq", node_uuid=str(uuid.uuid4()))            
            tree.add_child(pick_up)
            
            if new:
                resp = self.new_handle_tree(tree.get_json())
                print("RESP:", resp)
            else:
                (res, reason) = self.handle_tree(tree.get_json(), expand=self.expand)
                print(res, reason)
        except Exception as exc:
            self.get_logger().error(f'EXCEPTION gripper-open: {type(exc)} - {exc}')            

    def claim(self, new=True):
        try:
            pick_up = JsonExecNode(self.unit, name="claim")
            tree = JsonExecNode(self.unit, name="seq", node_uuid=str(uuid.uuid4()))            
            tree.add_child(pick_up)
            
            if new:
                resp = self.new_handle_tree(tree.get_json())
                print("RESP:", resp)
            else:
                (res, reason) = self.handle_tree(tree.get_json(), expand=self.expand)
                print(res, reason)
        except Exception as exc:
            self.get_logger().error(f'EXCEPTION claim: {type(exc)} - {exc}')            

    def upload_graph(self, new=True):
        try:
            pick_up = JsonExecNode(self.unit, name="upload-graph")
            tree = JsonExecNode(self.unit, name="seq", node_uuid=str(uuid.uuid4()))            
            tree.add_child(pick_up)
            
            if new:
                resp = self.new_handle_tree(tree.get_json())
                print("RESP:", resp)
            else:
                (res, reason) = self.handle_tree(tree.get_json(), expand=self.expand)
                print(res, reason)
        except Exception as exc:
            self.get_logger().error(f'EXCEPTION upload-graph: {type(exc)} - {exc}')            

    def release(self, new=True):
        try:
            pick_up = JsonExecNode(self.unit, name="release")
            tree = JsonExecNode(self.unit, name="seq", node_uuid=str(uuid.uuid4()))            
            tree.add_child(pick_up)
            
            if new:
                resp = self.new_handle_tree(tree.get_json())
                print("RESP:", resp)
            else:
                (res, reason) = self.handle_tree(tree.get_json(), expand=self.expand)
                print(res, reason)
        except Exception as exc:
            self.get_logger().error(f'EXCEPTION release: {type(exc)} - {exc}')            

    def localize_init(self, new=True):
        try:
            pick_up = JsonExecNode(self.unit, name="localize-init")
            tree = JsonExecNode(self.unit, name="seq", node_uuid=str(uuid.uuid4()))            
            tree.add_child(pick_up)
            
            if new:
                resp = self.new_handle_tree(tree.get_json())
                print("RESP:", resp)
            else:
                (res, reason) = self.handle_tree(tree.get_json(), expand=self.expand)
                print(res, reason)
        except Exception as exc:
            self.get_logger().error(f'EXCEPTION localize-init: {type(exc)} - {exc}')            

    def power_motor(self, new=True):
        try:
            pick_up = JsonExecNode(self.unit, name="power-motor")
            tree = JsonExecNode(self.unit, name="seq", node_uuid=str(uuid.uuid4()))            
            tree.add_child(pick_up)
            
            if new:
                resp = self.new_handle_tree(tree.get_json())
                print("RESP:", resp)
            else:
                (res, reason) = self.handle_tree(tree.get_json(), expand=self.expand)
                print(res, reason)
        except Exception as exc:
            self.get_logger().error(f'EXCEPTION power-motor: {type(exc)} - {exc}')            

    def find_person(self, new=True):
        try:
            find_person = JsonExecNode(self.unit, name="find-person")
            tree = JsonExecNode(self.unit, name="seq", node_uuid=str(uuid.uuid4()))            
            tree.add_child(find_person)
            
            if new:
                resp = self.new_handle_tree(tree.get_json())
                print("RESP:", resp)
            else:
                (res, reason) = self.handle_tree(tree.get_json(), expand=self.expand)
                print(res, reason)
        except Exception as exc:
            self.get_logger().error(f'EXCEPTION find_person: {type(exc)} - {exc}')
            
    def add_to_bucket(self, new=True):
        try:
            add_to_bucket = JsonExecNode(self.unit, name="add-to-bucket", bucket=self.bucket)
            tree = JsonExecNode(self.unit, name="seq", node_uuid=str(uuid.uuid4()))            
            tree.add_child(add_to_bucket)
            
            if new:
                resp = self.new_handle_tree(tree.get_json())
                print("RESP:", resp)
            else:
                (res, reason) = self.handle_tree(tree.get_json(), expand=self.expand)
                print(res, reason)
        except Exception as exc:
            self.get_logger().error(f'EXCEPTION find_person: {type(exc)} - {exc}')            

    def fly(self, new=True):
        print("FLY!")
        try:
            if self.lat > 1.0:
                (lat, lon, alt) = (self.lat, self.lon, self.alt)
            else:
                (lat, lon, alt) = self.xyz_to_latlonalt(self.x, self.y, self.z)
            fly = JsonExecNode(self.unit, name="fly-to", lat=lat, lon=lon, alt=alt, speed=self.speed)
            tree = JsonExecNode(self.unit, name="seq", node_uuid=str(uuid.uuid4()))
            tree.add_child(fly)
            if new:
                resp = self.new_handle_tree(tree.get_json())
                print("RESP:", resp)
            else:
                (res, reason) = self.handle_tree(tree.get_json(), expand=self.expand)
                print(res, reason)
        except Exception as exc:
            self.get_logger().error(f'EXCEPTION fly: {type(exc)} - {exc}')            

    def climb(self, new=True):
        try:
            (lat, lon, alt) = self.xyz_to_latlonalt(self.x, self.y, self.z)
            climb = JsonExecNode(self.unit, name="climb", z=alt)
            tree = JsonExecNode(self.unit, name="seq", node_uuid=str(uuid.uuid4()))            
            tree.add_child(climb)
            
            if new:
                resp = self.new_handle_tree(tree.get_json())
                print("RESP:", resp)
            else:
                (res, reason) = self.handle_tree(tree.get_json(), expand=self.expand)
                print(res, reason)
        except Exception as exc:
            self.get_logger().error(f'EXCEPTION fly: {type(exc)} - {exc}')            

    def fly_near(self, new=True):
        try:
            if self.lat > 1.0:
                (lat, lon, alt) = (self,lat, self.lon, self.alt)
            else:
                (lat, lon, alt) = self.xyz_to_latlonalt(self.x, self.y, self.z)
            fly = JsonExecNode(self.unit, name="fly-near", lat=lat, lon=lon, alt=alt, speed=self.speed)
            tree = JsonExecNode(self.unit, name="seq", node_uuid=str(uuid.uuid4()))
            tree.add_child(fly)
            if new:
                resp = self.new_handle_tree(tree.get_json())
                print("RESP:", resp)
            else:
                (res, reason) = self.handle_tree(tree.get_json(), expand=self.expand)
                print(res, reason)
        except Exception as exc:
            self.get_logger().error(f'EXCEPTION fly_near: {type(exc)} - {exc}')            

    def return_home(self, new=True):
        try:
            fly = JsonExecNode(self.unit, name="return-home", fly_home_height=30.0, speed=self.speed)
            tree = JsonExecNode(self.unit, name="seq", node_uuid=str(uuid.uuid4()))
            tree.add_child(fly)
            if new:
                resp = self.new_handle_tree(tree.get_json())
                print("RESP:", resp)
            else:
                (res, reason) = self.handle_tree(tree.get_json(), expand=self.expand)
                print(res, reason)
        except Exception as exc:
            self.get_logger().error(f'EXCEPTION fly_near: {type(exc)} - {exc}')            

    def go_home(self, new=True):
        try:
            fly = JsonExecNode(self.unit, name="go-home", fly_home_height=30.0, speed=self.speed)
            tree = JsonExecNode(self.unit, name="seq", node_uuid=str(uuid.uuid4()))
            tree.add_child(fly)
            if new:
                resp = self.new_handle_tree(tree.get_json())
                print("RESP:", resp)
            else:
                (res, reason) = self.handle_tree(tree.get_json(), expand=self.expand)
                print(res, reason)
        except Exception as exc:
            self.get_logger().error(f'EXCEPTION fly_near: {type(exc)} - {exc}')            

    def fly_path(self, new=True):
        try:
            path = [[140, 0, 40], [140, 80, 40], [0, 0, 40]]            
            geopath = self.world_to_geo_path(path)
            print("GEOPATH:", geopath)
            fly = JsonExecNode(self.unit, name="fly-path", waypoints=geopath, speed=self.sspeed)
            tree = JsonExecNode(self.unit, name="seq", node_uuid=str(uuid.uuid4()))
            tree.add_child(fly)
            # tree = json_fly_path(self.ns, geopath, speed=self.sspeed)
            if new:
                resp = self.new_handle_tree(tree.get_json())
                print("RESP:", resp)
            else:
                (res, reason) = self.handle_tree(tree.get_json(), expand=self.expand)
                print(res, reason)
        except Exception as exc:
            self.get_logger().error(f'EXCEPTION fly_path: {type(exc)} - {exc}')            
           
        
    def fly_path_with_speed(self, new=True):
        try:
            path = [[140, 0, 40], [140, 80, 40], [0, 0, 40]]
            commanded_speed = [2.0, 1.0, 3.0]
            geopath = self.world_to_geo_path(path)
            print("GEOPATH:", geopath)
            fly = JsonExecNode(self.unit, name="fly-path-with-speed", waypoints=geopath, commanded_speed=commanded_speed)
            tree = JsonExecNode(self.unit, name="seq", node_uuid=str(uuid.uuid4()))
            tree.add_child(fly)
            # tree = json_fly_path(self.ns, geopath, speed=self.sspeed)
            if new:
                resp = self.new_handle_tree(tree.get_json())
                print("RESP:", resp)
            else:
                (res, reason) = self.handle_tree(tree.get_json(), expand=self.expand)
                print(res, reason)
        except Exception as exc:
            self.get_logger().error(f'EXCEPTION fly_path_with_speed: {type(exc)} - {exc}')            
           
        
    def fly_pattern(self, name="test", new=True):
        try:
            path = self.xyz_paths[name]
            geopath = self.world_to_geo_path(path)
            print("GEOPATH:", geopath)
            fly_near = JsonExecNode(self.unit, name="fly-near", geopoint=geopath[0], distance=5.0, speed=self.sspeed) 
            fly = JsonExecNode(self.unit, name="fly-pattern", waypoints=geopath, speed=self.sspeed)
            tree = JsonExecNode(self.unit, name="seq", node_uuid=str(uuid.uuid4()))
            tree.add_child(fly_near)
            tree.add_child(fly)
            # tree = json_fly_path(self.ns, geopath, speed=self.sspeed)
            if new:
                resp = self.new_handle_tree(tree.get_json())
                print("RESP:", resp)
            else:
                (res, reason) = self.handle_tree(tree.get_json(), expand=self.expand)
                print(res, reason)
        except Exception as exc:
            self.get_logger().error(f'EXCEPTION fly_patttern: {type(exc)} - {exc}')

    def check_pattern(self, name="test", new=True):
        try:
            path = self.xyz_paths[name]
            geopath = self.world_to_geo_path(path)
            print("PATH:", path)
            x0 = 0.0
            y0 = 0.0
            for p in path:
                dx = p[0]- x0
                dy = p[1]- y0
                dist = math.sqrt(dx*dx+dy*dy)
                print("DIST:", dist)
                x0 = p[0]
                y0 = p[1]
                
        except Exception as exc:
            self.get_logger().error(f'EXCEPTION check_pattern: {type(exc)} - {exc}')
            

    def test_fly_patterns(self, new=True):            
        pass
    
    def take_off(self, new=True):
        try:
            jobj = JsonExecNode(self.unit, name="take-off", height_above_takeoff=self.height_above_takeoff)
            tree = JsonExecNode(self.root_ns, name="seq", node_uuid=str(uuid.uuid4()))
            tree.add_child(jobj)
            
            if new:
                resp = self.new_handle_tree(tree.get_json())
                print("RESP:", resp)
            else:
                (res, reason) = self.handle_tree(tree.get_json(), add_constraints=False, expand=self.expand)            
                print(res, reason)
        except Exception as exc:
            self.get_logger().error(f'EXCEPTION take_off: {type(exc)} - {exc}')            
       
    def basic_take_off(self, new=True):
        try:
            jobj = JsonExecNode(self.unit, name="basic-take-off")
            tree = JsonExecNode(self.root_ns, name="seq", node_uuid=str(uuid.uuid4()))
            tree.add_child(jobj)
            
            if new:
                resp = self.new_handle_tree(tree.get_json())
                print("RESP:", resp)
            else:
                (res, reason) = self.handle_tree(tree.get_json(), add_constraints=False, expand=self.expand)            
                print(res, reason)
        except Exception as exc:
            self.get_logger().error(f'EXCEPTION take_off: {type(exc)} - {exc}')            
       
       

    def in_air_goal(self, new=True):
        try:
            jobj = JsonExecNode(self.unit, name="in-air-goal", height=15.0)
            tree = JsonExecNode(self.root_ns, name="seq", node_uuid=str(uuid.uuid4()))
            tree.add_child(jobj)

            if new:
                resp = self.new_handle_tree(tree.get_json())
                print("RESP:", resp)
            else:
                (res, reason) = self.handle_tree(tree.get_json(), add_constraints=False, expand=self.expand)            
                print(res, reason)
        except Exception as exc:
            self.get_logger().error(f'EXCEPTION in_air_goal: {type(exc)} - {exc}')            
       

    def land(self, new=True):
        try:
            jobj = JsonExecNode(self.unit, name="land")
            tree = JsonExecNode(self.root_ns, name="seq", node_uuid=str(uuid.uuid4()))
            tree.add_child(jobj)
            
            if new:
                resp = self.new_handle_tree(tree.get_json())
                print("RESP:", resp)
            else:
                (res, reason) = self.handle_tree(tree.get_json(), add_constraints=False, expand=self.expand)            
                print(res, reason)
        except Exception as exc:
            self.get_logger().error(f'EXCEPTION land: {type(exc)} - {exc}')            
       

    def basic_land(self, new=True):
        try:
            jobj = JsonExecNode(self.unit, name="basic-land")
            tree = JsonExecNode(self.root_ns, name="seq", node_uuid=str(uuid.uuid4()))
            tree.add_child(jobj)
            
            if new:
                resp = self.new_handle_tree(tree.get_json())
                print("RESP:", resp)
            else:
                (res, reason) = self.handle_tree(tree.get_json(), add_constraints=False, expand=self.expand)            
                print(res, reason)
        except Exception as exc:
            self.get_logger().error(f'EXCEPTION land: {type(exc)} - {exc}')            
       

    def drop(self, new=True):
        try:
            print("tst_command: drop")
            jobj = JsonExecNode(self.unit, name="drop")
            tree = JsonExecNode(self.root_ns, name="seq", node_uuid=str(uuid.uuid4()))
            tree.add_child(jobj)
            
            if new:
                resp = self.new_handle_tree(tree.get_json())
                print("RESP:", resp)
            else:
                (res, reason) = self.handle_tree(tree.get_json(), add_constraints=False, expand=self.expand)            
                print(res, reason)
        except Exception as exc:
            self.get_logger().error(f'EXCEPTION land: {type(exc)} - {exc}')            
       

    def wait1(self, new=True, duration=10):
        try:
            self.tree_uuid = str(uuid.uuid4())
            wait = JsonExecNode(self.unit, name="wait", duration=duration)
            tree = JsonExecNode(self.root_ns, name="seq", node_uuid=self.tree_uuid)
            tree.add_child(wait)
            
            if new:
                resp = self.new_handle_tree(tree.get_json())
                print("RESP:", resp)
            else:
                (res, reason) = self.handle_tree(tree.get_json(), add_constraints=False, expand=self.expand)            
                print(res, reason)
        except Exception as exc:
            self.get_logger().error(f'EXCEPTION wait1: {type(exc)} - {exc}')            
       

    def sit(self, new=True):
        try:
            jobj = JsonExecNode(self.unit, name="sit")
            tree = JsonExecNode(self.root_ns, name="seq", node_uuid=str(uuid.uuid4()))
            tree.add_child(jobj)

            if new:
                resp = self.new_handle_tree(tree.get_json())
                print("RESP:", resp)
            else:
                (res, reason) = self.handle_tree(tree.get_json(), add_constraints=False, expand=self.expand)
                print(res, reason)
        except Exception as exc:
            self.get_logger().error(f'EXCEPTION land: {type(exc)} - {exc}')            
       

    def stand(self, new=True):
        try:
            jobj = JsonExecNode(self.unit, name="stand")
            tree = JsonExecNode(self.root_ns, name="seq", node_uuid=str(uuid.uuid4()))
            tree.add_child(jobj)
            if new:
                resp = self.new_handle_tree(tree.get_json())
                print("RESP:", resp)
            else:
                (res, reason) = self.handle_tree(tree.get_json(), add_constraints=False, expand=self.expand)            
                print(res, reason)
        except Exception as exc:
            self.get_logger().error(f'EXCEPTION stand: {type(exc)} - {exc}')            
       

    def delivery_goal(self, new=True):
        try:
            if self.lat > 1.0:
                (lat, lon, alt) = (self.lat, self.lon, self.alt)
            else:
                (lat, lon, alt) = self.xyz_to_latlonalt(self.x, self.y, self.z)
                
            jobj = JsonExecNode(self.unit, name="delivery-goal", lat=lat, lon=lon, alt=alt)
            tree = JsonExecNode(self.root_ns, name="seq", node_uuid=str(uuid.uuid4()))
            tree.add_child(jobj)
            if new:
                resp = self.new_handle_tree(tree.get_json())
                print("RESP:", resp)
            else:
                (res, reason) = self.handle_tree(tree.get_json(), add_constraints=False, expand=self.expand)
                print(res, reason)
        except Exception as exc:
            self.get_logger().error(f'EXCEPTION delivery_goal: {type(exc)} - {exc}')            
       

    def start_data_recording(self, new=True):
        try:
            topics = [f"{self.unit}/dji_osdk_ros/main_camera_images"]
            topics = [f"/dji5/dji_osdk_ros/main_camera_images"]
            opts = ["compression=jpeg"]
            jobj = json_basic_start_data_recording(self.unit, topics, opts, self.dataset_uri)
            tree = json_basic_seq(self.root_ns)
            tree["children"].append(jobj)
            if new:
                resp = self.new_handle_tree(tree)
                print("RESP:", resp)
            else:
                (res, reason) = self.handle_tree(tree, add_constraints=False, expand=self.expand)            
                print(res, reason)
        except Exception as exc:
            self.get_logger().error(f'EXCEPTION start_data_recording: {type(exc)} - {exc}')            

    def stop_data_recording(self, new=True):
        try:
            topics = [f"{self.unit}/dji_osdk_ros/main_camera_images"]
            topics = [f"/dji5/dji_osdk_ros/main_camera_images"]            
            jobj = json_basic_stop_data_recording(self.unit, topics, self.dataset_uri)
            tree = json_basic_seq(self.root_ns)
            tree["children"].append(jobj)
            if new:
                resp = self.new_handle_tree(tree)
                print("RESP:", resp)
            else:
                (res, reason) = self.handle_tree(tree, add_constraints=False, expand=self.expand)            
                print(res, reason)
        except Exception as exc:
            self.get_logger().error(f'EXCEPTION stop_data_recording: {type(exc)} - {exc}')            
       

    def move(self, new=True):
        try:
            if self.lat > 1.0:
                (lat, lon, alt) = (self.lat, self.lon, self.alt)
            else:
                (lat, lon, alt) = self.xyz_to_latlonalt(self.x, self.y, self.z)
            self.get_logger().info(f'MOVE: {lat} {lon} {alt}')            
            moveto = JsonExecNode(self.unit, name="move-to", lat=lat, lon=lon, alt=alt, speed=self.speed)            
            self.tree_uuid = str(uuid.uuid4())
            tree = JsonExecNode(self.root_ns, name="seq", node_uuid=self.tree_uuid)
            tree.add_child(moveto)
            if new:
                print("TREE:", tree.get_json())
                resp = self.new_handle_tree(tree.get_json(), channel=self.channel)
                print("RESP:", resp)
            else:
                (res, reason) = self.handle_tree(tree, add_constraints=False, expand=self.expand)
                print(res, reason)
        except Exception as exc:
            self.get_logger().error(f'EXCEPTION move: {type(exc)} - {exc}')            
       
    def move3(self, new=True):
        try:
            if self.lat > 1.0:
                (lat, lon, alt) = (self.lat, self.lon, self.alt)
            else:
                (lat1, lon1, alt1) = self.xyz_to_latlonalt(self.x, self.y, self.z)
                (lat2, lon2, alt2) = self.xyz_to_latlonalt(self.x+100.0, self.y, self.z)
                (lat3, lon3, alt3) = self.xyz_to_latlonalt(self.x+200.0, self.y, self.z)
            self.get_logger().info(f'MOVE: {lat1} {lon1} {alt1}')            
            moveto1 = JsonExecNode(self.unit, name="move-to", lat=lat1, lon=lon1, alt=alt1, speed=self.speed)            
            moveto2 = JsonExecNode(self.unit, name="move-to", lat=lat2, lon=lon2, alt=alt2, speed=self.speed)            
            moveto3 = JsonExecNode(self.unit, name="move-to", lat=lat3, lon=lon3, alt=alt3, speed=self.speed)            
            self.tree_uuid = str(uuid.uuid4())
            tree = JsonExecNode(self.root_ns, name="seq", node_uuid=self.tree_uuid)
            tree.add_child(moveto1)
            tree.add_child(moveto2)
            tree.add_child(moveto3)
            if new:
                print("TREE:", tree.get_json())
                resp = self.new_handle_tree(tree.get_json(), channel=self.channel)
                print("RESP:", resp)
            else:
                (res, reason) = self.handle_tree(tree, add_constraints=False, expand=self.expand)
                print(res, reason)
        except Exception as exc:
            self.get_logger().error(f'EXCEPTION move: {type(exc)} - {exc}')            
       
    def walk_to(self, new=True):
        try:
            self.get_logger().info(f'WALK TO: {self.x} {self.y} {self.z}')            
            walk_to = json_basic_walkto(self.unit, self.x, self.y, self.z)
            self.tree_uuid = str(uuid.uuid4())
            tree = json_basic_seq(self.root_ns, node_uuid=self.tree_uuid)
            tree["children"].append(walk_to)
            if new:
                print("TREE:", tree)
                resp = self.new_handle_tree(tree)
                print("RESP:", resp)
            else:
                (res, reason) = self.handle_tree(tree, add_constraints=False, expand=self.expand)
                print(res, reason)
        except Exception as exc:
            self.get_logger().error(f'EXCEPTION move: {type(exc)} - {exc}')            
       
    def graph_walk_to(self, new=True):
        try:
            self.get_logger().info(f'GRAPH WALK TO: {self.x} {self.y} {self.z}')            
            walk_to = json_basic_graph_walkto(self.unit, self.x, self.y, self.z)
            self.tree_uuid = str(uuid.uuid4())
            tree = json_basic_seq(self.root_ns, node_uuid=self.tree_uuid)
            tree["children"].append(walk_to)
            if new:
                print("TREE:", tree)
                resp = self.new_handle_tree(tree)
                print("RESP:", resp)
            else:
                (res, reason) = self.handle_tree(tree, add_constraints=False, expand=self.expand)
                print(res, reason)
        except Exception as exc:
            self.get_logger().error(f'EXCEPTION move: {type(exc)} - {exc}')            
       
    def aero_1(self, new=True):
        try:
            (lat0, lon0, alt0) = self.xyz_to_latlonalt(5.0, 5.0, 2.0)
            (lat1, lon1, alt1) = self.xyz_to_latlonalt(5.0, -5.0, 2.0)
            (lat2, lon2, alt2) = self.xyz_to_latlonalt(2.0, 2.0, 2.0)
            moveto0 = json_basic_moveto("/drone0", lat0, lon0, alt0, speed=self.sspeed)
            moveto1 = json_basic_moveto("/drone1", lat1, lon1, alt1, speed=self.sspeed)
            moveto2 = json_basic_moveto("/drone2", lat2, lon2, alt2, speed=self.sspeed)
            conc = json_basic_conc("/drone0")
            conc["children"].append(moveto0)
            conc["children"].append(moveto1)
            conc["children"].append(moveto2)
            tree = json_basic_seq("/drone0", node_uuid=str(uuid.uuid4()))
            tree["children"].append(conc)
            if new:
                print("TREE:", tree)
                resp = self.new_handle_tree(tree)
                print("RESP:", resp)
            else:
                (res, reason) = self.handle_tree(tree, add_constraints=False, expand=self.expand)
                print(res, reason)
        except Exception as exc:
            self.get_logger().error(f'EXCEPTION aero_1: {type(exc)} - {exc}')            
       
    def spot_delivery(self, new=True):
        try:
            print("STRING:", self.spot_delivery_json)
            tree = json.loads(self.spot_delivery_json)
            tree["common_params"]["node-uuid"] = str(uuid.uuid4())
            print("TREEE:", tree)
            if new:
                print("TREE:", tree)
                resp = self.new_handle_tree(tree)
                print("RESP:", resp)
            else:
                (res, reason) = self.handle_tree(tree, add_constraints=False, expand=self.expand)
                print(res, reason)
        except Exception as exc:
            self.get_logger().error(f'EXCEPTION spot_delivery: {type(exc)} - {exc}')            
       
    def spot_delivery_bucket(self, new=True):
        try:
            print("STRING:", self.spot_delivery_bucket_json)
            tree = json.loads(self.spot_delivery_bucket_json)
            tree["common_params"]["node-uuid"] = str(uuid.uuid4())
            print("TREEE:", tree)
            if new:
                print("TREE:", tree)
                resp = self.new_handle_tree(tree)
                print("RESP:", resp)
            else:
                (res, reason) = self.handle_tree(tree, add_constraints=False, expand=self.expand)
                print(res, reason)
        except Exception as exc:
            self.get_logger().error(f'EXCEPTION spot_delivery: {type(exc)} - {exc}')            
       
    def explore1(self, new=True):
        try:
            print("STRING:", self.explore1_json)
            tree = json.loads(self.explore1_json)
            tree["common_params"]["node-uuid"] = str(uuid.uuid4())
            print("TREEE:", tree)
            if new:
                print("TREE:", tree)
                resp = self.new_handle_tree(tree)
                print("RESP:", resp)
            else:
                (res, reason) = self.handle_tree(tree, add_constraints=False, expand=self.expand)
                print(res, reason)
        except Exception as exc:
            self.get_logger().error(f'EXCEPTION explore1: {type(exc)} - {exc}')            
       
    def explore2(self, new=True):
        try:
            print("STRING:", self.explore2_json)
            tree = json.loads(self.explore2_json)
            tree["common_params"]["node-uuid"] = str(uuid.uuid4())
            #print("TREEE:", tree)
            if new:
                print("TREE:", tree)
                resp = self.new_handle_tree(tree)
                print("RESP:", resp)
            else:
                (res, reason) = self.handle_tree(tree, add_constraints=False, expand=self.expand)
                print(res, reason)
        except Exception as exc:
            self.get_logger().error(f'EXCEPTION explore2: {type(exc)} - {exc}')            
       
    def husky_2024(self, new=True):
        try:
            
            if self.lat > 1.0:
                (lat, lon, alt) = (self.lat, self.lon, self.alt)
            else:
                (lat, lon, alt) = self.xyz_to_latlonalt(self.x, self.y, self.z)
                (lat1, lon1, alt1) = self.xyz_to_latlonalt(self.x-3.0, self.y-3.0, self.z)
            self.get_logger().info(f'HUSKY 2024: {lat} {lon} {alt}')            

            string_pub1 = JsonExecNode(self.unit, name="string-pub", string="turnoff")            
            move = JsonExecNode(self.unit, name="move-to", lat=lat, lon=lon, alt=alt, speed="standard")
            string_pub2 = JsonExecNode(self.unit, name="string-pub", string="input")                        
            say = JsonExecNode(self.unit, name="say", string="Do you need help?")
            tree = JsonExecNode(self.root_ns, name="seq", node_uuid=str(uuid.uuid4()))
            input_handler = JsonExecNode(self.unit, name="input-handler", lat=lat, lon=lon, alt=alt)
            move_away = JsonExecNode(self.unit, name="move-to", lat=lat1, lon=lon1, alt=alt1, speed="standard")
            tree.add_child(string_pub1)            
            tree.add_child(move)
            tree.add_child(say)
            tree.add_child(string_pub2)                        
            tree.add_child(input_handler)
            tree.add_child(move_away)
            #print("TREEE:", tree)
            if new:
                print("TREE:", tree)
                resp = self.new_handle_tree(tree.get_json(), channel=self.channel)
                print("RESP:", resp)
            else:
                (res, reason) = self.handle_tree(tree, add_constraints=False, expand=self.expand)
                print(res, reason)
        except Exception as exc:
            self.get_logger().error(f'EXCEPTION say: {type(exc)} - {exc}')            
       
    def spot_2024(self, new=True):
        try:
            
            if self.lat > 1.0:
                (lat, lon, alt) = (self.lat, self.lon, self.alt)
            else:
                (lat, lon, alt) = self.xyz_to_latlonalt(self.x, self.y, self.z)
                (lat1, lon1, alt1) = self.xyz_to_latlonalt(self.x-3.0, self.y-3.0, self.z)
            self.get_logger().info(f'HUSKY 2024: {lat} {lon} {alt}')

            (lat, lon, alt) = self.xyz_to_latlonalt(self.x, self.y, self.z)
            (lat1, lon1, alt1) = self.xyz_to_latlonalt(1.2, 3.0, 2.0)
            (lat2, lon2, alt2) = self.xyz_to_latlonalt(3.7, 0.2, 2.0)
            object = "medication"
            clear_graph = JsonExecNode(self.unit, name="clear-graph")
            add_person = JsonExecNode(self.unit, name="add-person", lat=lat, lon=lon, object=object)
            add_object1 = JsonExecNode(self.unit, name="add-object", lat=lat1, lon=lon1, object="medication")
            add_object2 = JsonExecNode(self.unit, name="add-object", lat=lat2, lon=lon2, object="food")            

            mission_document = "retrieve: { what: [document, format], from: missions, matches: { name: deliver_supplies }}"
            mission_mode = "kdql"
            parameters_document = f"{{ source_graph: granso, supply_type: http://askco.re/thing#{object} }}"
            parameters_format = "yaml"
            parameters_mode = "raw"
            mission = JsonExecNode(self.unit, name="mission", mission_document=mission_document, mission_mode=mission_mode, parameters_document=parameters_document, parameters_format=parameters_format, parameters_mode=parameters_mode)

            (hlon, hlat, halt) = self.coordtrans.world_to_wgs84(-3.8, 2.7, 2.0)
            move_to = JsonExecNode(self.unit, name="move-to", lat=hlat, lon=hlon, alt=halt, speed="standard")
            
            tree = JsonExecNode(self.root_ns, name="seq", node_uuid=str(uuid.uuid4()))

            tree.add_child(clear_graph)
            tree.add_child(add_person)
            tree.add_child(add_object1)
            tree.add_child(add_object2)
            tree.add_child(mission)
            tree.add_child(move_to)

            
            #print("TREEE:", tree)
            if new:
                print("TREE:", tree)
                resp = self.new_handle_tree(tree.get_json(), channel=1)
                print("RESP:", resp)
            else:
                (res, reason) = self.handle_tree(tree, add_constraints=False, expand=self.expand)
                print(res, reason)
        except Exception as exc:
            self.get_logger().error(f'EXCEPTION say: {type(exc)} - {exc}')            
       
    def add_person(self, object="medication", new=True):
        try:
            
            if self.lat > 1.0:
                (lat, lon, alt) = (self.lat, self.lon, self.alt)
            else:
                (lat, lon, alt) = self.xyz_to_latlonalt(self.x, self.y, self.z)
            self.get_logger().info(f'add_person: {lat} {lon} {alt}')            

            query = f"""salient region:
                            action: insert
                            dataset: granso
                            geometry: POINT({lon} {lat})
                            classes: ["http://askco.re/thing#human"]
                            properties:
                                "http://askco.re/thing/property#needs": "http://askco.re/thing#medication"
                            """
            print("QUERY:", query)
            query_node = JsonExecNode(self.unit, name="query", language="kdb/krql", query=query)
            tree = JsonExecNode(self.root_ns, name="seq", node_uuid=str(uuid.uuid4()))
            tree.add_child(query_node)
            #print("TREEE:", tree)
            if new:
                print("TREE:", tree)
                resp = self.new_handle_tree(tree.get_json())
                print("RESP:", resp)
            else:
                (res, reason) = self.handle_tree(tree, add_constraints=False, expand=self.expand)
                print(res, reason)
        except Exception as exc:
            self.get_logger().error(f'EXCEPTION say: {type(exc)} - {exc}')            
       
    def clear_graph(self, graph_name="granso", new=True):
        try:

            query = f"""DROP SILENT GRAPH <{graph_name}>; CREATE GRAPH <{graph_name}>"""
            print("QUERY:", query)
            query_node = JsonExecNode(self.unit, name="query", language="kdb/sparql", query=query)
            tree = JsonExecNode(self.root_ns, name="seq", node_uuid=str(uuid.uuid4()))
            tree.add_child(query_node)
            #print("TREEE:", tree)
            if new:
                print("TREE:", tree)
                resp = self.new_handle_tree(tree.get_json())
                print("RESP:", resp)
            else:
                (res, reason) = self.handle_tree(tree, add_constraints=False, expand=self.expand)
                print(res, reason)
        except Exception as exc:
            self.get_logger().error(f'EXCEPTION say: {type(exc)} - {exc}')            

    def add_object(self, object="medication", new=True):
        try:
            
            if self.lat > 1.0:
                (lat, lon, alt) = (self.lat, self.lon, self.alt)
            else:
                (lat, lon, alt) = self.xyz_to_latlonalt(self.x, self.y, self.z)
            self.get_logger().info(f'add_person: {lat} {lon} {alt}')            

            query = f"""salient region:
                            action: insert
                            dataset: granso
                            geometry: POINT({lon} {lat})
                            classes: ["http://askco.re/thing#depot"]
                            properties:
                                "http://askco.re/thing#{object}": 1
                            """
            print("QUERY:", query)
            query_node = JsonExecNode(self.unit, name="query", language="kdb/krql", query=query)
            tree = JsonExecNode(self.root_ns, name="seq", node_uuid=str(uuid.uuid4()))
            tree.add_child(query_node)
            #print("TREEE:", tree)
            if new:
                print("TREE:", tree)
                resp = self.new_handle_tree(tree.get_json())
                print("RESP:", resp)
            else:
                (res, reason) = self.handle_tree(tree, add_constraints=False, expand=self.expand)
                print(res, reason)
        except Exception as exc:
            self.get_logger().error(f'EXCEPTION add_object: {type(exc)} - {exc}')            

    def mission(self, new=True):
        try:
            mission_document = "retrieve: { what: [document, format], from: missions, matches: { name: deliver_supplies }}"
            mission_mode = "kdql"
            parameters_document = "{ source_graph: granso, supply_type: http://askco.re/thing#medication }"
            parameters_format = "yaml"
            parameters_mode = "raw"
            mission = JsonExecNode(self.unit, name="mission", mission_document=mission_document, mission_mode=mission_mode, parameters_document=parameters_document, parameters_format=parameters_format, parameters_mode=parameters_mode)
            tree = JsonExecNode(self.root_ns, name="seq", node_uuid=str(uuid.uuid4()))
            tree.add_child(mission)
            #print("TREEE:", tree)
            if new:
                print("TREE:", tree)
                resp = self.new_handle_tree(tree.get_json())
                print("RESP:", resp)
            else:
                (res, reason) = self.handle_tree(tree, add_constraints=False, expand=self.expand)
                print(res, reason)
        except Exception as exc:
            self.get_logger().error(f'EXCEPTION say: {type(exc)} - {exc}')            
       
    def say(self, new=True):
        try:
            print("STRING:", self.string)

            jobj = JsonExecNode(self.unit, name="say", string=self.string, gain=f'{self.gain}')
            tree = JsonExecNode(self.root_ns, name="seq", node_uuid=str(uuid.uuid4()))
            tree.add_child(jobj)
            #print("TREEE:", tree)
            if new:
                print("TREE:", tree)
                resp = self.new_handle_tree(tree.get_json())
                print("RESP:", resp)
            else:
                (res, reason) = self.handle_tree(tree, add_constraints=False, expand=self.expand)
                print(res, reason)
        except Exception as exc:
            self.get_logger().error(f'EXCEPTION say: {type(exc)} - {exc}')            
       
    def monitor(self, new=True):
        try:
            # print("STRING:", self.string)

            jobj = JsonExecNode(self.unit, name="monitor")
            tree = JsonExecNode(self.root_ns, name="seq", node_uuid=str(uuid.uuid4()))
            tree.add_child(jobj)
            #print("TREEE:", tree)
            if new:
                print("TREE:", tree)
                resp = self.new_handle_tree(tree.get_json())
                print("RESP:", resp)
            else:
                (res, reason) = self.handle_tree(tree, add_constraints=False, expand=self.expand)
                print(res, reason)
        except Exception as exc:
            self.get_logger().error(f'EXCEPTION say: {type(exc)} - {exc}')            
       
    def string_pub(self, new=True):
        try:
            print("STRING:", self.string)

            jobj = JsonExecNode(self.unit, name="string-pub", string=self.string)
            tree = JsonExecNode(self.root_ns, name="seq", node_uuid=str(uuid.uuid4()))
            tree.add_child(jobj)
            #print("TREEE:", tree)
            if new:
                print("TREE:", tree)
                resp = self.new_handle_tree(tree.get_json())
                print("RESP:", resp)
            else:
                (res, reason) = self.handle_tree(tree, add_constraints=False, expand=self.expand)
                print(res, reason)
        except Exception as exc:
            self.get_logger().error(f'EXCEPTION say: {type(exc)} - {exc}')            
       
    def add_task(self, new=True):
        try:
            print("STRING:", self.string)

            task_jobj = JsonExecNode(self.unit, name="say", string=self.string)
            task_tree = JsonExecNode(self.root_ns, name="seq", node_uuid=str(uuid.uuid4()))
            task_string = json.dumps(task_tree.get_json(), sort_keys=True, indent=4, separators=(',', ': '))
            task_type = "json-tst"
            task_priority = 50
            team_name = "speak"
            task_name = "say"
            
            jobj = JsonExecNode(self.unit, name="add-task-to-team-queue", task_string=task_string, team_name=team_name, task_priority=task_priority, task_type=task_type, task_name=task_name)
            tree = JsonExecNode(self.root_ns, name="seq", node_uuid=str(uuid.uuid4()))
            tree.add_child(jobj)
            #print("TREEE:", tree)
            if new:
                print("TREE:", tree)
                resp = self.new_handle_tree(tree.get_json())
                print("RESP:", resp)
            else:
                (res, reason) = self.handle_tree(tree, add_constraints=False, expand=self.expand)
                print(res, reason)
        except Exception as exc:
            self.get_logger().error(f'EXCEPTION say: {type(exc)} - {exc}')            
       
    def add_task_monitor(self, new=True):
        try:
            print("STRING:", self.string)

            task_jobj = JsonExecNode("", name="monitor")
            task_tree = JsonExecNode("", name="seq", node_uuid=str(uuid.uuid4()))
            task_tree.add_child(task_jobj)
            task_string = json.dumps(task_tree.get_json(), sort_keys=True, indent=4, separators=(',', ': '))
            task_type = "json-tst"
            task_priority = 50
            team_name = "monitor"
            task_name = "monitor-areas"
            
            jobj = JsonExecNode(self.unit, name="add-task-to-team-queue", task_string=task_string, team_name=team_name, task_priority=task_priority, task_type=task_type, task_name=task_name)
            tree = JsonExecNode(self.root_ns, name="seq", node_uuid=str(uuid.uuid4()))
            tree.add_child(jobj)
            #print("TREEE:", tree)
            if new:
                print("TREE:", tree)
                resp = self.new_handle_tree(tree.get_json())
                print("RESP:", resp)
            else:
                (res, reason) = self.handle_tree(tree, add_constraints=False, expand=self.expand)
                print(res, reason)
        except Exception as exc:
            self.get_logger().error(f'EXCEPTION say: {type(exc)} - {exc}')            
       
    def input_handler(self, new=True):
        try:
            jobj = JsonExecNode(self.unit, name="input-handler")
            tree = JsonExecNode(self.root_ns, name="seq", node_uuid=str(uuid.uuid4()))
            tree.add_child(jobj)
            #print("TREEE:", tree)
            if new:
                print("TREE:", tree)
                resp = self.new_handle_tree(tree.get_json())
                print("RESP:", resp)
            else:
                (res, reason) = self.handle_tree(tree, add_constraints=False, expand=self.expand)
                print(res, reason)
        except Exception as exc:
            self.get_logger().error(f'EXCEPTION input_handler: {type(exc)} - {exc}')            
       
    def deliver_pkg(self, new=True):
        try:
            print("STRING:", self.string)

            jobj = JsonExecNode(self.unit, name="deliver-pkg", string=self.string)
            tree = JsonExecNode(self.root_ns, name="seq", node_uuid=str(uuid.uuid4()))
            tree.add_child(jobj)
            #print("TREEE:", tree)
            if new:
                print("TREE:", tree)
                resp = self.new_handle_tree(tree.get_json())
                print("RESP:", resp)
            else:
                (res, reason) = self.handle_tree(tree, add_constraints=False, expand=self.expand)
                print(res, reason)
        except Exception as exc:
            self.get_logger().error(f'EXCEPTION explore2: {type(exc)} - {exc}')            
       
    def collect_release(self, new=True):
        try:
            if self.lat > 1.0:
                (lat, lon, alt) = (self.lat, self.lon, self.alt)
            else:
                (lat0, lon0, alt0) = self.xyz_to_latlonalt(self.x, self.y, self.z)
                (lat1, lon1, alt1) = self.xyz_to_latlonalt(self.x+50, self.y, self.z)
                gp0 = GeoPoint()
                gp0.latitude = lat0
                gp0.longitude = lon0
                gp0.altitude = alt0
                
                gp1 = GeoPoint()
                gp1.latitude = lat1
                gp1.longitude = lon1
                gp1.altitude = alt1
                
                collect_release = JsonExecNode(self.unit,
                                               name="collect-release",
                                               geopoint0=gp0,
                                               geopoint1=gp1,
                                               speed="standard",
                                               cargo="ball",
                                               expandunit="/sup1")
                tree = JsonExecNode(self.unit, name="seq", node_uuid=str(uuid.uuid4()))                
                tree.add_child(collect_release)
                jobj = tree.get_json()
                if new:
                    print("TREE:", jobj)
                    resp = self.new_handle_tree(jobj)
                    print("RESP:", resp)
                else:
                    (res, reason) = self.handle_tree(tree, add_constraints=False, expand=self.expand)
                    print(res, reason)
        except Exception as exc:
            self.get_logger().error(f'EXCEPTION collect_release: {type(exc)} - {exc}')
       
    def move_extra(self, new=True):
        try:
            (lat, lon, alt) = self.xyz_to_latlonalt(self.x, self.y, self.z)
            self.get_logger().info(f'MOVE: {lat} {lon} {alt}')            
            moveto = json_basic_moveto_extra(self.unit, lat, lon, alt, speed=self.sspeed)
            tree = json_basic_seq(self.root_ns, node_uuid=str(uuid.uuid4()))
            tree["children"].append(moveto)
            if new:
                print("TREE:", tree)
                resp = self.new_handle_tree(tree)
                print("RESP:", resp)
            else:
                (res, reason) = self.handle_tree(tree, add_constraints=False, expand=self.expand)
                print(res, reason)
        except Exception as exc:
            self.get_logger().error(f'EXCEPTION move: {type(exc)} - {exc}')            
       

    def look_at_bearing(self, bearing, tilt, new=True):
        try:
            node_uuid = str(uuid.uuid4())

            # exec_unit = "/dji56"
            exec_unit = self.unit

            look = JsonExecNode(exec_unit, name="look-at-bearing", bearing=bearing, tilt=tilt, node_uuid=node_uuid)
            wait = JsonExecNode(self.unit, name="wait", duration=10)
            send_signal = JsonExecNode(self.unit, name="send-signal", signal_uuid=node_uuid, signal="$enough")
            
            seq = JsonExecNode(self.unit, name="seq")
            conc = JsonExecNode(self.unit, name="conc")
            tree = JsonExecNode(self.unit, name="seq")

            tree.add_child(conc)
            conc.add_child(look)
            conc.add_child(seq)
            seq.add_child(wait)
            seq.add_child(send_signal)

            if new:
                resp = self.new_handle_tree(tree.get_json())
                print("RESP:", resp)
            else:
                (res, reason) = self.handle_tree(tree.get_json(), add_constraints=False, expand=self.expand)
                print(res, reason)
        except Exception as exc:
            self.get_logger().error(f'EXCEPTION look_at_bearing: {type(exc)} - {exc}')            
       

    def test_expand(self, new=True):
        try:
            (lat, lon, alt) = self.xyz_to_latlonalt(self.x, self.y, self.z)
            jobj = json_moveto(self.ns, lat, lon, alt, speed=self.sspeed)
            resp = self.expand_tree(jobj)
            print("EXPANDED:", resp)
            tree = json.loads(resp.expanded_json)
            print("EXPANDED:", json.dumps(tree, sort_keys=True, indent=4, separators=(',', ': ')))
            return
            if new:
                resp = self.new_handle_tree(tree)
                print("RESP:", resp)
            else:
                (res, reason) = self.handle_tree(jobj)
                print(res, reason)
        except Exception as exc:
            self.get_logger().error(f'EXCEPTION test_expand: {type(exc)} - {exc}')            
      

    def move_path(self, new=True):
        try:
            path = [[140, 0, 40], [140, 80, 40], [0, 0, 40]]                        
            if self.vehicle == "tbot":
                path = [[10, 0, 0], [10, 10, 0], [0, 0, 0]]
            if self.vehicle == "turtle":
                path = [[40, 0, 20], [40, 80, 20], [0, 0, 20]]
            geopath = self.world_to_geo_path(path)
            print("GEOPATH:", geopath)
            # jobj = json_move_path(self.unit, geopath, speed=self.sspeed)
            fly = JsonExecNode(self.unit, name="move-path", waypoints=geopath, speed=self.sspeed)
            tree = JsonExecNode(self.unit, name="seq", node_uuid=str(uuid.uuid4()))
            tree.add_child(fly)
            if new:
                resp = self.new_handle_tree(tree.get_json())
                print("RESP:", resp)
            else:
                (res, reason) = self.handle_tree(tree.get_json(), expand=self.expand)
                print(res, reason)
        except Exception as exc:
            self.get_logger().error(f'EXCEPTION: {type(exc)} - {exc}')            

    def look_at_named_position(self, posname, new=True):
        print("look_at_named_position:", posname)
        try:
            node_uuid = str(uuid.uuid4())
            look = JsonExecNode(self.unit, name="look-at-named-position", posname=posname, node_uuid=node_uuid)
            tree = JsonExecNode(self.unit, name="seq")
            tree.add_child(look)
            
            if new:
                resp = self.new_handle_tree(tree.get_json())
                print("RESP:", resp)
            else:
                (res, reason) = self.handle_tree(tree.get_json(), expand=self.expand)
                print(res, reason)
        except Exception as exc:
            self.get_logger().error(f'EXCEPTION: {type(exc)} - {exc}')            

    def look_at_position(self, new=True):
        print("look_at_position:", self.x, self.y, self.z)
        try:
            (lat, lon, alt) = self.xyz_to_latlonalt(self.x, self.y, self.z)
            node_uuid = str(uuid.uuid4())
            look = JsonExecNode(self.unit, name="look-at-position", lat=lat, lon=lon, alt=alt, node_uuid=node_uuid)
            wait = JsonExecNode(self.unit, name="wait", duration=10)
            send_signal = JsonExecNode(self.unit, name="send-signal", signal_uuid=node_uuid, signal="$enough")
            
            seq = JsonExecNode(self.unit, name="seq")
            conc = JsonExecNode(self.unit, name="conc")
            tree = JsonExecNode(self.unit, name="seq")
            
            tree.add_child(conc)
            conc.add_child(look)
            conc.add_child(seq)
            seq.add_child(wait)
            seq.add_child(send_signal)

            if new:
                resp = self.new_handle_tree(tree.get_json())
                print("RESP:", resp)
            else:
                (res, reason) = self.handle_tree(tree.get_json(), expand=self.expand)
                print(res, reason)
        except Exception as exc:
            self.get_logger().error(f'EXCEPTION look_at_position: {type(exc)} - {exc}')
            
    def basic_look_at_position(self, new=True):
        print("look_at_position:", self.x, self.y, self.z)
        try:
            (lat, lon, alt) = self.xyz_to_latlonalt(self.x, self.y, self.z)
            node_uuid = str(uuid.uuid4())
            look = JsonExecNode(self.unit, name="look-at-position", lat=lat, lon=lon, alt=alt, node_uuid=node_uuid)
            tree = JsonExecNode(self.unit, name="seq")
            
            tree.add_child(look)

            if new:
                resp = self.new_handle_tree(tree.get_json())
                print("RESP:", resp)
            else:
                (res, reason) = self.handle_tree(tree.get_json(), expand=self.expand)
                print(res, reason)
        except Exception as exc:
            self.get_logger().error(f'EXCEPTION basic_look_at_position: {type(exc)} - {exc}')
            
    def observe_position(self, new=True):
        print("observe_position:", self.x, self.y, self.z)
        try:
            (lat, lon, alt) = self.xyz_to_latlonalt(self.x, self.y, self.z)
            node_uuid = str(uuid.uuid4())
            observe = JsonExecNode(self.unit, name="observe-position", lat=lat, lon=lon, alt=alt, distance=20.0, height=30.0, speed="standard", duration=10, node_uuid=node_uuid)
            tree = JsonExecNode(self.unit, name="seq")
            tree.add_child(observe)            

            if new:
                resp = self.new_handle_tree(tree.get_json())
                print("RESP:", resp)
            else:
                (res, reason) = self.handle_tree(tree.get_json(), expand=self.expand)
                print(res, reason)
        except Exception as exc:
            self.get_logger().error(f'EXCEPTION look_at_position: {type(exc)} - {exc}')
            
    def search_position(self, new=True):
        try:
            (lat, lon, alt) = self.xyz_to_latlonalt(self.x, self.y, self.z)
            jobj = json_search_position(self.ns, lat, lon, alt, self.radius, speed=self.sspeed, target_type=self.target_type, target_size=self.target_size)
            if new:
                resp = self.new_handle_tree(tree)
                print("RESP:", resp)
            else:
                (res, reason) = self.handle_tree(jobj, expand=self.expand)
                print(res, reason)
        except Exception as exc:
            self.get_logger().error(f'EXCEPTION search_position: {type(exc)} - {exc}')
        
    def search_area(self, new=True):
        try:
            path = [[40, 0, 20], [40, 80, 20], [0, 0, 20]]
            area = self.world_to_geo_path(path)
            print("GEOPATH:", area)
            #tree = json_search_area(self.ns, area, speed=self.sspeed, target_type=self.target_type, target_size=self.target_size, area_type=self.area_type)
            #print("SEARCH AREA TREE:", json.dumps(tree, sort_keys=True, indent=4, separators=(',', ': ')))
            sa = JsonExecNode(self.unit, name="basic-search-area", area=area, speed=self.sspeed, target_type=self.target_type, target_size=self.target_size, area_type=self.area_type)
            tree = JsonExecNode(self.unit, name="seq", node_uuid=str(uuid.uuid4()))
            tree.add_child(sa)
            print("SEARCH AREA TREE:", json.dumps(tree.get_json(), sort_keys=True, indent=4, separators=(',', ': ')))            
            if new:
                resp = self.new_handle_tree(tree.get_json())
                #resp = self.new_handle_tree(tree)
                print("RESP:", resp)
            else:
                (res, reason) = self.handle_tree(jobj, expand=self.expand)
                print(res, reason)
        except Exception as exc:
            self.get_logger().error(f'EXCEPTION search_area: {type(exc)} - {exc}')

    def get_seq(self, n_seq):
        res = JsonExecNode(self.unit, name="seq")
        lastseq = res
        for i in range(n_seq):
            tmp = JsonExecNode(self.unit, name="seq")
            lastseq.add_child(tmp)
            lastseq = tmp
        return (res, lastseq)
            

    def get_conc(self, n_seq):
        res = JsonExecNode(self.ns, name="conc")
        lastseq = res
        for i in range(n_seq):
            tmp = JsonExecNode(self.ns, name="conc")
            lastseq.add_child(tmp)
            lastseq = tmp
        return (res, lastseq)
            

    def test_perf_1(self, new=True):
        try:
            c1 = JsonExecNode(self.unit, name="wait", duration=5)
            c2 = JsonExecNode(self.unit, name="wait", duration=10)
            c3 = JsonExecNode(self.unit, name="wait", duration=3)

            tree = JsonExecNode(self.ns, name="seq")
            (s1, s1_last) = self.get_seq(15)
            (s2, s2_last) = self.get_seq(15)
            (s3, s3_last) = self.get_seq(15)
            
            s1_last.add_child(c1)
            s2_last.add_child(c2)
            s3_last.add_child(c3)
            
            tree.add_child(s1)
            tree.add_child(s2)
            tree.add_child(s3)
            if new:
                resp = self.new_handle_tree(tree.get_json())
                print("RESP:", resp)
            else:
                (res, reason) = self.handle_tree(tree)
                print(res, reason)
        except Exception as exc:
            print("test_seq", type(exc))
            print(exc)
        

    def test_perf_2(self, new=True):
        try:
            c1 = JsonExecNode(self.unit, name="wait", duration=5)
            c2 = JsonExecNode(self.unit, name="wait", duration=10)
            c3 = JsonExecNode(self.unit, name="wait", duration=3)

            tree = JsonExecNode(self.ns, name="conc")            

            conc = JsonExecNode(self.ns, name="conc")
            (s1, s1_last) = self.get_conc(15)
            (s2, s2_last) = self.get_conc(15)
            (s3, s3_last) = self.get_conc(15)
            
            s1_last.add_child(c1)
            s2_last.add_child(c2)
            s3_last.add_child(c3)
            
            tree.add_child(s1)
            tree.add_child(s2)
            tree.add_child(s3)
            
            if new:
                resp = self.new_handle_tree(tree.get_json())
                print("RESP:", resp)
            else:
                (res, reason) = self.handle_tree(tree)
                print(res, reason)
        except Exception as exc:
            print("test_perf_2", type(exc))
            print(exc)

    def test_perf_3(self, new=True):
        try:
            c1 = json_basic_wait(self.ns, 5)
            c2 = json_basic_wait(self.ns, 10)
            c3 = json_basic_wait(self.ns, 3)

            tree = json_basic_seq(self.ns)
            (s1, s1_last) = self.get_seq(2)
            (s2, s2_last) = self.get_seq(15)
            (s3, s3_last) = self.get_seq(15)
            
            s1_last["children"].append(c1)
            #s2_last["children"].append(c2)
            #s3_last["children"].append(c3)
            
            tree["children"].append(s1)
            #tree["children"].append(s2)
            #tree["children"].append(s3)
            if new:
                resp = self.new_handle_tree(tree)
                print("RESP:", resp)
            else:
                (res, reason) = self.handle_tree(tree)
                print(res, reason)
        except Exception as exc:
            print("test_seq", type(exc))
            print(exc)
        

    def test_time_1(self, new=True):
        try:
            c1 = JsonExecNode(self.ns, name="wait", duration=5)
            c2 = JsonExecNode(self.ns, name="wait", duration=10)
            c3 = JsonExecNode(self.ns, name="wait", duration=3)

            c1.set_stime(5, -1)
            c2.set_stime(8, -1)
            c3.set_stime(10, -1)

            tree = JsonExecNode(self.ns, name="seq")
            conc = JsonExecNode(self.ns, name="conc")

            conc.add_child(c1)
            conc.add_child(c2)
            conc.add_child(c3)

            tree.add_child(conc)

            now = self.get_clock().now().to_msg()
            tree.set_tree_start_time(now)

            if new:
                resp = self.new_handle_tree(tree.get_json())
                print("RESP:", resp)
            else:
                (res, reason) = self.handle_tree(tree.get_json())
                print(res, reason)
        except Exception as exc:
            print("test_seq", type(exc))
            print(exc)
            
    def test_time_2(self, new=True):
        try:
            c1 = JsonExecNode(self.ns, name="wait", duration=5)
            c2 = JsonExecNode(self.ns, name="wait", duration=10)
            c3 = JsonExecNode(self.ns, name="wait", duration=3)

            c1.set_stime(5, -1)
            c2.set_stime(8, -1)
            c3.set_stime(10, -1)

            c1.set_wait_for_stime(False)
            c2.set_wait_for_stime(True)
            c3.set_wait_for_stime(False)

            tree = JsonExecNode(self.ns, name="seq")
            conc = JsonExecNode(self.ns, name="conc")

            conc.add_child(c1)
            conc.add_child(c2)
            conc.add_child(c3)

            tree.add_child(conc)

            now = self.get_clock().now().to_msg()
            tree.set_tree_start_time(now)

            if new:
                resp = self.new_handle_tree(tree.get_json())
                print("RESP:", resp)
            else:
                (res, reason) = self.handle_tree(tree.get_json())
                print(res, reason)
        except Exception as exc:
            print("test_seq", type(exc))
            print(exc)
            

    def test_time_3(self, new=True):
        # test direct enough and abort when using times
        try:
            c1 = JsonExecNode(self.ns, name="wait", duration=5)
            c2 = JsonExecNode(self.ns, name="wait", duration=10)
            c3 = JsonExecNode(self.ns, name="wait", duration=3)

            c1.set_stime(5, -1)
            c2.set_stime(10, -1)
            c3.set_stime(15, -1)

            c1.set_wait_for_stime(True)
            c2.set_wait_for_stime(True)
            c3.set_wait_for_stime(True)

            c1.set_enough_when_starting(True)
            c2.set_abort_when_starting(True)
            c3.set_enough_when_starting(True)

            tree = JsonExecNode(self.ns, name="seq")
            conc = JsonExecNode(self.ns, name="conc")

            conc.add_child(c1)
            conc.add_child(c2)
            conc.add_child(c3)

            tree.add_child(conc)

            now = self.get_clock().now().to_msg()
            tree.set_tree_start_time(now)

            if new:
                resp = self.new_handle_tree(tree.get_json())
                print("RESP:", resp)
            else:
                (res, reason) = self.handle_tree(tree.get_json())
                print(res, reason)
        except Exception as exc:
            print("test_seq", type(exc))
            print(exc)

    def remote_test(self, new=True):
        try:
            tree = JsonExecNode(self.ns, name="seq", node_uuid=str(uuid.uuid4()))
            w1 = JsonExecNode("/op1", name="wait", duration=10)            
            w2 = JsonExecNode("/op2", name="wait", duration=10)            
            w3 = JsonExecNode("/op3", name="wait", duration=10)
            #tree.add_child(w1)
            tree.add_child(w2)
            #tree.add_child(w3)
            resp = self.new_handle_tree(tree.get_json())
            print("RESP:", resp)
        except Exception as exc:
            print("remote_test", type(exc))
            print(exc)

    def remote_test2(self, new=True):
        try:
            tree = JsonExecNode("/op1", name="conc", node_uuid=str(uuid.uuid4()))
            seq = JsonExecNode("/op2", name="seq", node_uuid=str(uuid.uuid4()))
            w1 = JsonExecNode("/op2", name="wait", duration=10)            
            w2 = JsonExecNode("/op2", name="wait", duration=10)            
            w3 = JsonExecNode("/op2", name="wait", duration=10)
            seq.add_child(w1)
            seq.add_child(w2)
            seq.add_child(w3)
            tree.add_child(seq)
            print("REMOTE TEST2")
            resp = self.new_handle_tree(tree.get_json())
            print("RESP:", resp)
        except Exception as exc:
            print("remote_test2", type(exc))
            print(exc)

    def test_supp(self, new=True):
        try:
            c1 = json_basic_wait(self.ns, 5)
            c2 = json_basic_wait(self.ns, 1000)
            tree = json_basic_seq(self.ns)
            
            supp = json_basic_supp(self.ns)
            supp["children"].append(c2)

            conc = json_basic_conc(self.ns)
            conc["children"].append(c1)            
            conc["children"].append(supp)

            tree["children"].append(conc)
            if new:
                resp = self.new_handle_tree(tree)
                print("RESP:", resp)
            else:
                (res, reason) = self.handle_tree(tree)
                print(res, reason)
        except Exception as exc:
            print("test_seq", type(exc))
            print(exc)
        

    def test_seq(self, fly_flag=False, new=True):
        try:
            (lat, lon, alt) = self.xyz_to_latlonalt(self.x, self.y, self.z)
            if fly_flag:
                pass
                #c1 = json_basic_flyto(self.ns, lat, lon, alt, self.speed)
                #c2 = json_basic_flyto(self.ns, lat, lon, alt, self.speed)
                #c3 = json_basic_flyto(self.ns, lat, lon, alt, self.speed)
            else:
                w1 = JsonExecNode(self.ns, name="wait", duration=10)            
                w2 = JsonExecNode(self.ns, name="wait", duration=10)            
                w3 = JsonExecNode(self.ns, name="wait", duration=10)
            tree = JsonExecNode(self.ns, name="seq", node_uuid=str(uuid.uuid4()))
            tree.add_child(w1)
            tree.add_child(w2)
            tree.add_child(w3)
            if new:
                resp = self.new_handle_tree(tree.get_json())
                print("RESP:", resp)
            else:
                (res, reason) = self.handle_tree(tree)
                print(res, reason)
        except Exception as exc:
            print("test_seq", type(exc))
            print(exc)
        

    def test_conc(self, fly_flag=False, diff_flag=False, new=True):
        try:
            (lat, lon, alt) = self.xyz_to_latlonalt(self.x, self.y, self.z)
            (lat1, lon1, alt1) = self.xyz_to_latlonalt(self.x+50.0, self.y, self.z)
            (lat2, lon2, alt2) = self.xyz_to_latlonalt(self.x, self.y+50.0, self.z)
            (lat3, lon3, alt3) = self.xyz_to_latlonalt(self.x-50.0, self.y-50.0, self.z)
            diff_ns = "/ex0"
            if diff_flag:
                diff_ns = "/ex1"                
            if fly_flag:
                if diff_flag:
                    c1 = json_basic_moveto("/dji61", lat1, lon1, alt1, self.speed)
                    c2 = json_basic_moveto("/dji62", lat2, lon2, alt2, self.speed)
                    c3 = json_basic_moveto("/dji63", lat3, lon3, alt3, self.speed)
                else:
                    c1 = json_basic_flyto(self.ns, lat, lon, alt, self.speed)
                    c2 = json_basic_flyto(self.ns, lat, lon, alt, self.speed)
                    c3 = json_basic_flyto(self.ns, lat, lon, alt, self.speed)
            else:
                c1 = json_basic_wait(self.ns, 10)
                c2 = json_basic_wait(self.ns, 10)
                c3 = json_basic_wait(diff_ns, 10)
            tree = json_basic_seq(self.ns, node_uuid=str(uuid.uuid4()))
            conc = json_basic_conc(self.ns)
            conc["children"].append(c1)
            conc["children"].append(c2)
            conc["children"].append(c3)
            tree["children"].append(conc)
            if new:
                resp = self.new_handle_tree(tree)
                print("test-conc-RESP:", resp)
            else:
                (res, reason) = self.handle_tree(tree)
                print(res, reason)
        except Exception as exc:
            print("test_conc", type(exc))
            print(exc)

    def get_move_tree(self, ns):
        try:
            (lat1, lon1, alt1) = self.xyz_to_latlonalt(6.0, 8.0, 0.0)
            (lat2, lon2, alt2) = self.xyz_to_latlonalt(1.0, 8.0, 0.0)
            (lat3, lon3, alt3) = self.xyz_to_latlonalt(1.0, 1.00, 0.0)
            move1 = json_basic_moveto(ns, lat1, lon1, alt1, speed=self.sspeed)
            move2 = json_basic_moveto(ns, lat2, lon2, alt2, speed=self.sspeed)
            move3 = json_basic_moveto(ns, lat3, lon3, alt3, speed=self.sspeed)
            tree = json_basic_seq(ns)
            tree["children"].append(move1)
            tree["children"].append(move2)
            tree["children"].append(move3)
            return tree
        except Exception as exc:
            print("get_move_tree", type(exc))
            print(exc)
            return None
    
    def test_turtle(self, new=True):
        try:
            tree = self.get_move_tree(self.ns)
            if new:
                resp = self.new_handle_tree(tree)
                print("RESP:", resp)
            else:
                (res, reason) = self.handle_tree(tree)
                print(res, reason)
        except Exception as exc:
            print("test_turtle", type(exc))
            print(exc)

    def debug_1(self, new=True):
        try:
            jobj1 = JsonExecNode(self.unit, name="yes")
            jobj2 = JsonExecNode(self.unit, name="no-op")
            jobj3 = JsonExecNode(self.unit, name="no-op")
            jobj4 = JsonExecNode(self.unit, name="no-op")
            jobj5 = JsonExecNode(self.unit, name="no-op")
            tree = JsonExecNode(self.root_ns, name="seq", node_uuid=str(uuid.uuid4()))
            seq1 = JsonExecNode(self.root_ns, name="seq")
            wait1 = JsonExecNode(self.root_ns, name="wait", duration=20)
            seq1.add_child(jobj2)
            test_if = JsonExecNode(self.unit, name="test-if")
            test_if.add_child(jobj1)
            test_if.add_child(seq1)
            #test_if.add_child(wait1)
            test_if.add_child(jobj3)
            test_if.add_child(jobj4)
            tree.add_child(test_if)
            tree.add_child(jobj5)
            if new:
                resp = self.new_handle_tree(tree.get_json())
                print("RESP:", resp)
            else:
                (res, reason) = self.handle_tree(tree.get_json(), add_constraints=False, expand=self.expand)            
                print(res, reason)
        except Exception as exc:
            print("test_two_turtle", type(exc))
            print(exc)

    def debug_2(self, new=True):
        try:
            jobj1 = JsonExecNode(self.unit, name="seq")
            jobj2 = JsonExecNode(self.unit, name="seq")
            jobj3 = JsonExecNode(self.unit, name="seq")
            jobj4 = JsonExecNode(self.unit, name="seq")
            jobj5 = JsonExecNode(self.unit, name="no-op")
            jobj4.add_child(jobj5)
            jobj3.add_child(jobj4)
            jobj2.add_child(jobj3)
            jobj1.add_child(jobj2)
            tree = JsonExecNode(self.root_ns, name="seq", node_uuid=str(uuid.uuid4()))
            tree.add_child(jobj1)
            if new:
                resp = self.new_handle_tree(tree.get_json())
                print("RESP:", resp)
            else:
                (res, reason) = self.handle_tree(tree.get_json(), add_constraints=False, expand=self.expand)            
                print(res, reason)
        except Exception as exc:
            print("test_two_turtle", type(exc))
            print(exc)
            
    def test_two_turtle(self, new=True):
        try:
            tree = json_basic_seq(self.ns)
            conc = json_basic_conc(self.ns)
            conc["children"].append(self.get_move_tree("/ex0"))
            conc["children"].append(self.get_move_tree("/ex1"))
            tree["children"].append(conc)
            if new:
                resp = self.new_handle_tree(tree)
                print("RESP:", resp)
            else:
                (res, reason) = self.handle_tree(tree)
                print(res, reason)
        except Exception as exc:
            print("test_two_turtle", type(exc))
            print(exc)
        
    def test_computation(self, infinite, new=True):
        #
        # This should create three topics: test/input/a test/input/b test/output/o
        #
        # ros2 topic echo /ns/test/output/o
        # ros2 topic pub /ns/test/input/a std_msgs/msg/Int64 "data: 4"
        #
        print(f"Make sure to run a computation server: ros2 run ros_march graph_flow_server --ros-args -r __ns:={self.ns}")
        print(f"It should create three topics {self.ns}/input/a {self.ns}/test/input/b {self.ns}/test/output/o")
        print(f"Run the following to test that the computation is executed:")
        print(f"ros2 topic echo {self.ns}/test/output/o ")
        print(f"ros2 topic pub {self.ns}/test/input/a std_msgs/msg/Int64 'data: 4'")
        print(f"ros2 topic pub {self.ns}/test/input/b std_msgs/msg/Int64 'data: 6'")

        if infinite:
            description = '{"nodes":[{"id":0,"library":"march_math","operation":"addition<std_msgs/integer64,std_msgs/integer64>","stage":0}],"connectors":[{"id":1,"direction":"output","type":"std_msgs/integer64","name":"a"},{"id":2,"direction":"output","type":"std_msgs/integer64","name":"b"},{"id":3,"direction":"input","type":"std_msgs/integer64","name":"o"}],"connections":[{"target_id":0,"target_input":0,"source_id":1},{"target_id":0,"target_input":1,"source_id":2},{"target_id":3,"source_id":0,"source_output":0}],"initial_messages":[]}'
        else:
            description = '{"nodes":[{"id":0,"library":"march_math","operation":"addition<std_msgs/integer64,std_msgs/integer64>","stage":0},{"id":1,"library":"march_math","operation":"greater_than<std_msgs/integer64,std_msgs/integer64>","stage":0}],"connectors":[{"id":2,"direction":"output","type":"std_msgs/integer64","name":"a"},{"id":3,"direction":"output","type":"std_msgs/integer64","name":"b"},{"id":4,"direction":"input","type":"std_msgs/integer64","name":"o"}],"connections":[{"target_id":0,"target_input":0,"source_id":2},{"target_id":0,"target_input":1,"source_id":3},{"target_id":1,"target_input":0,"source_id":0,"source_output":0},{"target_id":4,"source_id":0,"source_output":0}],"initial_messages":[{"target_id":1,"target_input":1,"message_type":"std_msgs/integer64","message_data":{"data":0}}],"finished_event":[{"source_id":1,"event_number":0}]}'
        try:
            c1 = {
                "children": [],
                "common_params": get_common_params (self.ns, False, ""),                
                "name": "computation",
                "params": {
                    "description": description,
                    "computation-ns": 'test'
                }
            }
            tree = json_basic_seq(self.ns)
            tree["children"].append(c1)
            if new:
                resp = self.new_handle_tree(tree)
                print("RESP:", resp)
            else:
                (res, reason) = self.handle_tree(tree)
                print(res, reason)
        except Exception as exc:
            print("test_computation", type(exc))
            print(exc)

    def do_signal(self):
        print("DO SIGNAL:", self.tree_uuid)
        self.timer2.cancel()
        if self.abort_tree:
            print("REALLY DO SIGNAL TREE ABORT:", self.tree_uuid)
            self.signal_tree(self.tree_uuid, "$abort")
        
    def do_command(self):
        try:
            print("do_command:", self.command)
            self.timer.cancel()
            if self.command == "look-at-bearing":
                self.look_at_bearing(self.yaw, self.pitch)
            if self.command == "deliver-pkg":
                self.deliver_pkg()
            if self.command == "dock":
                self.dock()
            if self.command == "undock":
                self.undock()
            if self.command == "pick-up":
                self.pick_up()
            if self.command == "pick-up-body-bucket":
                self.pick_up_body_bucket()
            if self.command == "arm-carry":
                self.arm_carry()
            if self.command == "arm-lower":
                self.arm_lower()
            if self.command == "arm-stow":
                self.arm_stow()
            if self.command == "gripper-close":
                self.gripper_close()
            if self.command == "gripper-open":
                self.gripper_open()
            if self.command == "claim":
                self.claim()
            if self.command == "upload-graph":
                self.upload_graph()
            if self.command == "localize-init":
                self.localize_init()
            if self.command == "power-motor":
                self.power_motor()
            if self.command == "release":
                self.release()
            if self.command == "find-person":
                self.find_person()
            if self.command == "add-to-bucket":
                self.add_to_bucket()
            if self.command == "fly":
                self.fly()
            if self.command == "climb":
                self.climb()
            if self.command == "fly-near":
                self.fly_near()
            if self.command == "return-home":
                self.return_home()
            if self.command == "go-home":
                self.go_home()
            if self.command == "fly-path":
                self.fly_path()
            if self.command == "fly-path-with-speed":
                self.fly_path_with_speed()
            if self.command == "fly-pattern":
                self.fly_pattern(name=self.name)
            if self.command == "check-pattern":
                self.check_pattern(name=self.name)
            if self.command == "take-off":
                self.take_off()
            if self.command == "basic-take-off":
                self.basic_take_off()
            if self.command == "in-air-goal":
                self.in_air_goal()
            if self.command == "land":
                self.land()
            if self.command == "basic-land":
                self.basic_land()
            if self.command == "drop":
                self.drop()
            if self.command == "sit":
                self.sit()
            if self.command == "stand":
                self.stand()
            if self.command == "delivery-goal":
                self.delivery_goal()
            if self.command == "start-data-recording":
                self.start_data_recording()
            if self.command == "stop-data-recording":
                self.stop_data_recording()
            if self.command == "collect-release":
                self.collect_release()
            if self.command == "move":
                self.move()
            if self.command == "move3":
                self.move3()
            if self.command == "move-to":
                self.move()
            if self.command == "walk-to":
                self.walk_to()
            if self.command == "graph-walk-to":
                self.graph_walk_to()
            if self.command == "move-to-extra":
                self.move_extra()
            if self.command == "move-path":
                self.move_path()
            if self.command == "search-position":
                self.search_position()
            if self.command == "search-area":
                self.search_area()
            if self.command == "test-seq-1":
                self.test_seq(fly_flag=False)
            if self.command == "test-seq-2":
                self.test_seq(fly_flag=True)
            if self.command == "test-seq-3":
                self.test_seq(fly_flag=False)
            if self.command == "test-conc-1":
                self.test_conc(fly_flag=False)
            if self.command == "test-conc-2":
                self.test_conc(fly_flag=True)
            if self.command == "test-conc-3":
                self.test_conc(fly_flag=False, diff_flag=True)
            if self.command == "test-conc-4":
                self.test_conc(fly_flag=True, diff_flag=True)
            if self.command == "test-perf-1":
                self.test_perf_1()
            if self.command == "test-perf-2":
                self.test_perf_2()
            if self.command == "test-perf-3":
                self.test_perf_3()
            if self.command == "test-supp":
                self.test_supp()
            if self.command == "remote-test":
                self.remote_test()
            if self.command == "remote-test2":
                self.remote_test2()
            if self.command == "test-turtle-1":
                self.test_turtle()
            if self.command == "test-turtle-2":
                self.test_two_turtle()
            if self.command == "look-at-named-position":
                self.look_at_named_position("Turtletest-P")
            if self.command == "look-at-position":
                self.look_at_position()
            if self.command == "basic-look-at-position":
                self.basic_look_at_position()
            if self.command == "observe-position":
                self.observe_position()
            if self.command == "expand":
                self.test_expand()
            if self.command == "test-time-1":
                self.test_time_1(new=True)
            if self.command == "test-time-2":
                self.test_time_2(new=True)
            if self.command == "test-time-3":
                self.test_time_3(new=True)
            if self.command == "test-fly-paths":
                self.test_fly_pattterns(new=True)
            if self.command == "wait1":
                self.wait1(new=True, duration=10)
            if self.command == "wait2":
                self.wait1(new=True, duration=0)
            if self.command == "test-computation-1":
                self.test_computation(False)
            if self.command == "test-computation-2":
                self.test_computation(True)
            if self.command == "debug-1":
                self.debug_1(True)
            if self.command == "debug-2":
                self.debug_2(True)
            if self.command == "aero-1":
                self.aero_1(True)
            if self.command == "spot-delivery":
                self.spot_delivery(True)
            if self.command == "spot-delivery-bucket":
                self.spot_delivery_bucket(True) 
            if self.command == "explore1":
                self.explore1(True) 
            if self.command == "explore2":
                self.explore2(True) 
            if self.command == "say":
                self.say(new=True) 
            if self.command == "monitor":
                self.monitor(new=True) 
            if self.command == "add-task":
                self.add_task(new=True) 
            if self.command == "add-task-monitor":
                self.add_task_monitor(new=True) 
            if self.command == "input-handler":
                self.input_handler(new=True) 
            if self.command == "deliver-pkg":
                self.deliver_pkg(new=True) 
            if self.command == "husky-2024":
                self.husky_2024(new=True) 
            if self.command == "spot-2024":
                self.spot_2024(new=True) 
            if self.command == "mission":
                self.mission(new=True) 
            if self.command == "add_person":
                self.add_person(new=True) 
            if self.command == "add_object":
                self.add_object(new=True) 
            if self.command == "clear_graph":
                self.clear_graph(new=True) 
            if self.command == "string_pub":
                self.string_pub(new=True) 
               
            print("COMMAND FINISHED: Abort with Ctrl-C")
        except Exception as exc:
            self.get_logger().error(f'EXCEPTION: {type(exc)} - {exc}')

def main(args=None):
    rclpy.init(args=args)

    node = TstCommand()

    executor = MultiThreadedExecutor(num_threads=8)
    # minimal_client.send_request()
    
    executor.add_node(node)

    #thread = threading.Thread(target=spin_thread, args=(node,))
    #thread.start()
    
    
    print("COMMAND:", node.command)
    try:
        executor.spin()
    except KeyboardInterrupt:
        pass
    
    # rclpy.spin(node)
    # node.do_command()

    node.destroy_node()
    executor.shutdown()    
    rclpy.shutdown()
    

#    while rclpy.ok():
#        rclpy.spin_once(minimal_client)
#        if minimal_client.future.done():
#            try:
#                response = minimal_client.future.result()
#            except Exception as e:
#                minimal_client.get_logger().info(
#                    'Service call failed %r' % (e,))
#            else:
#                minimal_client.get_logger().info(
#                    'Result of add_two_ints: for %d + %d = %d' %
#                    (minimal_client.req.a, minimal_client.req.b, response.sum))
#            break
#
#    minimal_client.destroy_node()
#    rclpy.shutdown()
#
