#!/usr/bin/env python3

from os import system, environ, getlogin, getenv
import time
import sys
import os

from lrs_util.coordtrans import CoordTrans
from tmux import *

from optparse import OptionParser
parser = OptionParser()
parser.add_option ("", "--manager", action="store_true", dest="manager", help="Manager")
parser.add_option ("-a", "--attach", action="store_true", dest="attach", help="Attach")
parser.add_option ("-s", "--simulator", action="store_true", dest="simulator", help="Simulator")
parser.add_option ("", "--ns", action="store", type="string", dest="ns", default="/dummy", help="Namespace to use (default /dummy)")
parser.add_option ("", "--m300", action="store_true", dest="m300", help="Start executors for M300")
parser.add_option ("", "--tstimages", action="store_true", dest="tstimages", help="Generate TST images")
parser.add_option ("", "--location", action="store", type="string", dest="location", default=f'{os.getenv("LRS_LOCATION")}', help=f'Location to use (default: {os.getenv("LRS_LOCATION")})')
parser.add_option ("", "--view-tst", action="store_true", dest="view_tst", help="If true the start tst viewer")
parser.add_option ("", "--domain-id", action="store", type="int", dest="domain_id", default=os.getenv("ROS_DOMAIN_ID"), help="ROS domain id to use")
(options, args) = parser.parse_args()

def ros2(name, command, run=True):
    new_window(name, f'ros2run', run=True)
    tmux_shell(f'export ROS_DOMAIN_ID={options.domain_id}', run=True)
    tmux_shell(command, run=run)

def main_unit(ns):
    command = f"new-session -n MISC -d -s {ns} 'exec bash'"
    tmux(command)

    if options.tstimages:
        ros2("tst_exec_streamer", f"ros2 launch lrs_exec tst_streamer.launch.py ns:={ns} display_type:=exec name:=tst_streamer_exex")
        
    ros2("exec", f"ros2 launch lrs_exec exec.launch.py location:={options.location} ns:={ns} external:=false basic_executors:=false tree_executor:=false")
    ros2("tree_executor", f'ros2 run lrs_exec tree_executor --ros-args -r __ns:={ns}')

    if options.m300:
        if options.manager:
            ros2("basic_executors", f"ros2 launch lrs_exec basic_executors.launch.py ns:={ns}")        
            ros2("m300_executors", f"ros2 launch lrs_dji m300_executors.launch.py ns:={ns} fly_when_no_plan_found:=true location:={options.location}")
    else:
        ros2("basic_executors", f"ros2 launch lrs_exec basic_executors.launch.py ns:={ns}")

    if options.view_tst:
        ros2(f'view_tst_{ns.lstrip("/")}', f'ros2 run rqt_image_view rqt_image_view --force-discover {ns}/tststream_exec/image_raw/compressed')

    if options.m300:
        ros2("tst_command", f'ros2 run lrs_exec tst_command --ros-args -r __ns:={ns} -p command:="move-to" -p name:="test" -p x:=100.0 -p y:=200.0 -p z:=40.0', run=False)
    else:
        ros2("tst_command", f'ros2 run lrs_exec tst_command --ros-args -r __ns:={ns} -p command:=test-perf-1', run=False)
        
    if options.attach:
        tmux(f'attach-session -t {ns}')
    
if __name__ == "__main__":

    if options.ns != "/dummy":
        ns = options.ns
    else:
        ns = f'/{os.environ["AGENT_NAME"].split("_")[-1]}'

    print("NS:", ns)
    
    main_unit(ns)   
 
        
