#!/usr/bin/env python3

from os import system, environ, getlogin, getenv
import time
import sys
import os
import uuid

from tmux import *

from optparse import OptionParser
parser = OptionParser()
parser.add_option ("", "--domain-id", action="store", type="int", dest="domain_id", default=os.getenv("ROS_DOMAIN_ID"), help="ROS domain id to use")
parser.add_option ("-a", "--attach", action="store_true", dest="attach", help="Attach")
parser.add_option ("", "--remote", action="store_true", dest="remote", help="Use xpra")
parser.add_option ("", "--ui", action="store_true", dest="ui", help="Start Waraps Test UI")
parser.add_option ("", "--team-ui", action="store_true", dest="team_ui", help="Start Waraps Team UI")
parser.add_option ("", "--ns", action="store", type="string", dest="ns", default="/op0", help="Namespace to use (default /op0)")
parser.add_option ("", "--wara", action="store_true", dest="wara", help="Connect to the specified waraps broker. Start the json api bridge.)")
parser.add_option ("", "--main-wara", action="store_true", dest="main_wara", help="Connect to main waraps broker. Start the json api bridge.)")
parser.add_option ("", "--location", action="store", type="string", dest="location", default=f'{os.getenv("LRS_LOCATION")}', help=f'Location to use (default: {os.getenv("LRS_LOCATION")})')
parser.add_option ("", "--external", action="store_true", dest="external", default=False, help="Push to aiics broker (default=False)")
parser.add_option ("", "--airspace", action="store_true", dest="airspace", default=False, help="Run airspace manager (default=False)")
parser.add_option ("", "--team", action="store_true", dest="team", default=False, help="Start team functionality (default=False)")
parser.add_option ("", "--spot", action="store_true", dest="spot", default=False, help="Select spot team functionality (default=False)")
parser.add_option ("", "--scale", action="store", type="int", dest="scale", default=1, help="Location to use")
parser.add_option ("", "--tststream", action="store_true", dest="tststream", help="Stream tst")
parser.add_option ("", "--omehost", action="store", type="string", dest="omehost", default="ome.waraps.org", help="Ome host (default ome.waraps.org)")
parser.add_option ("", "--omews", action="store", type="string", dest="omews", default="wss://ome.waraps.org::3334/", help="Ome ws (default wss://ome.waraps.org::3334/)")
parser.add_option ("", "--camera-base-url", action="store", type="string", dest="camera_base_url", default="wss://ome.waraps.org::3334/", help="Ome ws (default wss://ome.waraps.org::3334/)")
parser.add_option ("", "--view-tst", action="store_true", dest="view_tst", help="If true the start tst viewer")
parser.add_option ("", "--wara-broker-host", action="store", type="string", dest="wara_broker_host", default="broker.waraps.org", help="Wara broker (default broker.waraps.org)")
parser.add_option ("", "--wara-broker-port", action="store", type="string", dest="wara_broker_port", default="8883", help="Wara broker port (default 8883)")
parser.add_option ("", "--wara-broker-user", action="store", type="string", dest="wara_broker_user", default="", help="Wara broker user")
parser.add_option ("", "--wara-broker-password", action="store", type="string", dest="wara_broker_password", default="", help="Wara broker passwd")
parser.add_option ("", "--octree-filename", action="store", type="string", dest="octree_filename", default=f'{os.getenv("HOME")}/lrs2_ws/src/lrs_granso/octomaps/granso_l0_center_12.5cm.bt', help="")
parser.add_option ("", "--bitrate", action="store", type="string", dest="bitrate", default="5000k", help="Bitrate ffmpeg (default 5000k)")
parser.add_option ("", "--stream-deck", action="store_true", dest="stream_deck", help="Stream deck")
parser.add_option ("", "--husky", action="store", type="string", dest="husky", default="husky0", help="Husky to use for team queue (default husky0)")
parser.add_option ("", "--bt", action="store_true", dest="bt", help="If true start programs for behaviour trees.")
parser.add_option ("", "--zenoh", action="store_true", dest="zenoh", help="If true start zenoh bridge for testinh (do not use for production runs).")
(options, args) = parser.parse_args()

def ros2(name, command, run=True):
    new_window(name, f'ros2run', run=True)
    tmux_shell(f'export ROS_DOMAIN_ID={options.domain_id}', run=True)
    tmux_shell(command, run=run)

def get_list_param(name, items):
    res = f"{name}:='["
    first_item = True
    for item in items:
        if first_item:
            res += f'\\\"{item}\\\"'
            first_item = False
        else:
            res += f', \\\"{item}\\\"'
    res += "]'"
    return res

def ffmpeg_command(name, fifo, resolution="1024x600", omehost="ome.waraps.org"):
    command = f'while [ true ] ; do ffmpeg -f rawvideo -r 15 -vcodec rawvideo -pix_fmt bgr24 -s {resolution} -i {fifo} -r 15 -c:v libx264 -b:v {options.bitrate} -bufsize {options.bitrate} -pix_fmt yuv420p -tune zerolatency -preset ultrafast -f flv rtmp://{omehost}/app/{name} ; sleep 2; done'
    return command

def main_unit(ns):
    command = f"new-session -n MISC -d -s {ns} 'exec bash'"
    tmux(command)

    external = options.external

    transport = "mqtt"
    if os.getenv("TST_REMOTE_TRANSPORT"):
        transport = os.getenv("TST_REMOTE_TRANSPORT")


    if options.wara or options.main_wara:
        available_tasks = get_list_param("available_tasks",
                                         ["move-to", "move-path", "look-at-bearing",
                                          "look-at-position", "observe-position", "search-area", "say", "monitor"])
        supported_levels = get_list_param("supported_levels",
                                          ["sensor", "direct execution", "tst execution"])

        unit = ns.lstrip("/")
        #prefix = "ros2_gazebo_"
        prefix = ""
        
        available_sensors = get_list_param("available_sensors", ["position", "connect_status", "unit_status", "camera_tags", "videoserver_url"])
        camera_tags = get_list_param("camera_tags", [f"{unit}_tst"])
        camera_base_url = options.camera_base_url

        wara_host = ""
        mqtt_host = ""
        if options.wara_broker_host:
            wara_host = f'host:={options.wara_broker_host}'
            mqtt_host = f'mqtt_host:={options.wara_broker_host}'
        wara_user = ""
        mqtt_user = ""
        if options.wara_broker_user:
            wara_user = f'user:={options.wara_broker_user}'
            mqtt_user = f'mqtt_user:={options.wara_broker_user}'
        wara_password = ""
        if options.wara_broker_password:
            wara_password = f'password:={options.wara_broker_password}'

        if options.main_wara:
            wara_host = "host:=broker.waraps.org"
            mqtt_host = "mqtt_host:=broker.waraps.org"
            wara_user = "user:=mqtt"
            mqtt_user = "mqtt_user:=mqtt"
            wara_password = "password:=Check"

        recv_all_positions="false"  # if tf needed of positions set tp true
        aspart = "airspace:=false"
        if options.airspace:
            aspart = "airspace:=true"
        if transport == "mqtt":
            ros2("wara_json_api", f"ros2 launch lrs_json_api wara.launch.py ns:={ns} have_camera:=true {wara_host} port:={options.wara_broker_port} {wara_user} {wara_password} {supported_levels} {available_tasks} {available_sensors} {camera_tags} camera_base_url:={camera_base_url} prefix3:=ground prefix4:=real recv_all_positions:={recv_all_positions} ome_ws:={options.omews} recv_all_positions:=true {aspart}")

        if options.airspace:
            ros2("airspacemanager", f'ros2 run lrs_airspace airspacemanager --ros-args -r __ns:={ns}')

        if options.team:
            ros2("teams", f"ros2 launch lrs_team team.launch.py ns:={ns} {mqtt_host} mqtt_port:={options.wara_broker_port} {mqtt_user} {wara_password}")
            if options.spot:
                ros2("create_team", f'ros2 run lrs_team team_command --ros-args -r __ns:={ns} -p team_name:=deliver -p team_command:=set-team -p args:=\\\"spot0\\\"')
                ros2("create_team", f'ros2 run lrs_team team_command --ros-args -r __ns:={ns} -p team_name:=search -p team_command:=set-team -p args:=\\\"dji8\\\"')                
                ros2("create_team", f'ros2 run lrs_team team_command --ros-args -r __ns:={ns} -p team_name:=drop -p team_command:=set-team -p args:=\\\"dji17\\\"')                
            if False:
                ros2("create_team", f'ros2 run lrs_team team_command --ros-args -r __ns:={ns} -p team_name:=searchsurface -p team_command:=set-team -p args:=\\\"Piraya14 Manta\\\"')            
                ros2("create_team", f'ros2 run lrs_team team_command --ros-args -r __ns:={ns} -p team_name:=searchair -p team_command:=set-team -p args:=\\\"dji5\\\"')
                ros2("create_team", f'ros2 run lrs_team team_command --ros-args -r __ns:={ns} -p team_name:=examine -p team_command:=set-team -p args:=\\\"MiniUSV1 MiniUSV2 MiniUSV3 MiniUSV4\\\"')
                ros2("create_team", f'ros2 run lrs_team team_command --ros-args -r __ns:={ns} -p team_name:=sup -p team_command:=set-team -p args:=\\\"expand\\\"')        

            if True:
                ros2("create_team", f'ros2 run lrs_team team_command --ros-args -r __ns:={ns} -p team_name:=monitor -p team_command:=set-team -p args:=\\\"op0\\\"')
                ros2("create_team", f'ros2 run lrs_team team_command --ros-args -r __ns:={ns} -p team_name:=speak -p team_command:=set-team -p args:=\\\"dji5\\\"')
                ros2("create_team", f'ros2 run lrs_team team_command --ros-args -r __ns:={ns} -p team_name:=examine -p team_command:=set-team -p args:=\\\"{options.husky}\\\"')                                            
                ros2("create_team", f'ros2 run lrs_team team_command --ros-args -r __ns:={ns} -p team_name:=deliver -p team_command:=set-team -p args:=\\\"spot0\\\"')


    prefix = str(uuid.uuid4()).split("-")[1]
    streamname = f'tst_streamer_exec_{prefix}'
    ros2("tst_exec_streamer", f"ros2 launch lrs_exec tst_streamer.launch.py ns:={ns} display_type:=exec name:={streamname}")

    lat0 = 57.7605
    lon0 = 16.6827
    alt0 = 0.0
    expand_flag = "false"
    if options.ns == "/expand":
        lat0 = 57.7605 + 0.0004
        lon0 = 16.6827
        alt0 = 0.0
        expand_flag = "true"
        
    ros2("push_position", f"ros2 topic pub {options.ns}/sensor/position geographic_msgs/msg/GeoPoint '{{\\\"latitude\\\":{lat0}, \\\"longitude\\\":{lon0}, \\\"altitude\\\":{alt0} }}'")

    if external:
        ros2("mqtt_to_ros", f"ros2 launch lrs_mqtt mqtt_to_ros.launch.py ns:={ns}")
        ros2("ros_to_mqtt", f"ros2 launch lrs_mqtt ros_to_mqtt.launch.py ns:={ns}")
    ros2("exec", f"ros2 launch lrs_exec exec.launch.py location:={options.location} ns:={ns} external:={external} tree_executor:=false basic_executors:=true expand:={expand_flag}")
    ros2("tree_executor", f'ros2 run lrs_exec tree_executor --ros-args -r __ns:={ns}')

    if options.bt:
        ros2("btbridge", f'ros2 run lrs_py_trees bridge --ros-args -r __ns:={ns}', run=True)
        ros2("bt_tree_executor", f'roscd lrs_py_trees; cd lrs_py_trees; source venv/bin/activate', run=True)
        tmux_shell(f"python3 tree_executor.py --ros-args -r __ns:={ns}", run=True)

    if options.tststream:
        ros2("tst_to_fifo", f'ros2 run lrs_util to_fifo --ros-args -p image:={ns}/tststream_exec/image_raw -p compressed:=true -p fifo:=/tmp/fifo_tst_{ns.lstrip("/")}  -p scale:=false -p notext:=true')
        command = ffmpeg_command(f'{ns.lstrip("/")}_tst', f'/tmp/fifo_tst_{ns.lstrip("/")}', resolution="1280x720", omehost=options.omehost)
        # command = ffmpeg_command(f'{ns.lstrip("/")}_tst', f'/tmp/fifo_tst', omehost=options.omehost)
        ros2(f'ffmpeg_tst', f'sleep 15; {command}')
    
    ros2("octo", f"ros2 launch lrs_granso octomap_server.launch.py ns:={ns}")
    ros2("publish_auto_flag_true", f"ros2 topic pub {ns}/auto_flag  std_msgs/msg/Bool 'data: true' -r 1")
    #ros2("mqtt_to_ip", f"ros2 run lrs_mqtt mqtt_to_ip --ros-args -p mqtt_host:=10.8.0.77")
    ros2("mqtt_to_ip", "ros2 run lrs_mqtt mqtt_to_ip --ros-args -p mqtt_host:=broker.waraps.org -p mqtt_user:=mqtt -p mqtt_password:=Check -p mqtt_port:=8883")
    
    if options.stream_deck:
        ros2("streamros", f"ros2 run lrs_stream_deck streamros --ros-args -r __ns:={ns}")        

    if options.view_tst:
        ros2(f'view_tst_{ns.lstrip("/")}', f'ros2 run rqt_image_view rqt_image_view --force-discover {ns}/tststream_exec/image_raw/compressed')
        # ros2(f'view_tst_{ns.lstrip("/")}', f'sleep 5; ros2 run rqt_image_view rqt_image_view {ns}/tststream_exec/image_raw')


    if options.ui:
        if options.remote:
            command = f'xpra start --start=\\\"rqt -s lrs_rqt_waraps.warapsplugin.Waraps --ros-args -r __ns:={ns}\\\" --html=on --bind-tcp=0.0.0.0:10000'
            # print("COMMAND:", command)
            ros2("warapsui", command)
        else:
            ros2(f'warapsui', f'rqt -s lrs_rqt_waraps.warapsplugin.Waraps --ros-args -r __ns:={ns}')

    if options.team_ui:
        if options.remote:
            command = f'xpra start --start=\\\"rqt -s lrs_rqt_team.teamqueuesplugin.TeamQueues --ros-args -r __ns:={ns}\\\" --html=on --bind-tcp=0.0.0.0:10001'
            # print("COMMAND:", command)
            ros2("rqt_team", command)
        else:
            ros2("rqt_team", f'rqt -s lrs_rqt_team.teamqueuesplugin.TeamQueues --ros-args -r __ns:={ns}')
    
    

    ros2("demo_command", f'ros2 run waraps_ss2023 demo_move_path_many_agents', run=False)


    ros2("tst_command", f'ros2 run lrs_exec tst_command --ros-args -r __ns:={ns} -p command:="remote-test" -p name:="test" -p z:=20.0 -p x:=10.0 -p y:=10.0', run=False)

    ros2("team_command", f'ros2 run lrs_team team_command --ros-args -r __ns:=/op0 -p team_name:=speak -p team_command:=add-task  -p command:=say', run=False)

    ros2("scenario_2024", f'ros2 run lrs_team scenario_2024 --ros-args -r __ns:=/op0', run=False)

    ros2("unit_topic_msg", "sleep 10; ros2 topic echo /unit_topic_msg")

    if options.zenoh:
        ros2("bt_tree_executor", f'cd {os.getenv("HOME")}/zenoh', run=True)
        tmux_shell(f'./zenoh-bridge-ros2dds --config {os.getenv("HOME")}/lrs2_ws/src/lrs_zenoh/test.json', run=True)


    if options.spot:
        ros2("team_ui", "rqt -s lrs_rqt_team.teamqueuesplugin.TeamQueues --ros-args -r __ns:=/op0", run=False)
        ros2("spot_status", "ros2 topic echo /op0/global/spot_status")
        ros2("packages_server", "ros2 run lrs_wdb packages --ros-args -r __ns:=/op0")
        ros2("scenario_handler", "ros2 run lrs_exec scenario_handler --ros-args -r __ns:=/op0 -p fake:=true -p drop:=true -p search:=true", run=False)
        ros2("start_spot_scenario", "ros2 run lrs_mqtt test_salient --ros-args -p mqtt_host:=10.8.0.77 -p demo:=true -p person:=0", run=False)
    
        
    tmux(f'attach-session -t {ns}')
    
if __name__ == "__main__":
    ns = options.ns

    main_unit(ns)    
