#!/usr/bin/env python3

from os import system, environ, getlogin, getenv
import time
import sys
import os

from lrs_util.coordtrans import CoordTrans
from tmux import *

from optparse import OptionParser
parser = OptionParser()
parser.add_option ("-a", "--attach", action="store_true", dest="attach", help="Attach")
parser.add_option ("", "--ns", action="store", type="string", dest="ns", default="/dummy", help="Namespace to use (default /dummy)")
parser.add_option ("", "--domain-id", action="store", type="int", dest="domain_id", default=os.getenv("ROS_DOMAIN_ID"), help="ROS domain id to use")
parser.add_option ("", "--omehost", action="store", type="string", dest="omehost", default="ome.waraps.org", help="Ome host (default ome.waraps.org)")
parser.add_option ("", "--bitrate", action="store", type="string", dest="bitrate", default="5000k", help="Bitrate ffmpeg (default 5000k)")
(options, args) = parser.parse_args()

def ros2(name, command, run=True):
    new_window(name, f'ros2run', run=True)
    tmux_shell(f'export ROS_DOMAIN_ID={options.domain_id}', run=True)
    tmux_shell(command, run=run)

def ffmpeg_command(name, fifo, resolution="1024x600", omehost="ome.waraps.org"):
    command = f'while [ true ] ; do ffmpeg -f rawvideo -r 15 -vcodec rawvideo -pix_fmt bgr24 -s {resolution} -i {fifo} -r 15 -c:v libx264 -b:v {options.bitrate} -bufsize {options.bitrate} -pix_fmt yuv420p -tune zerolatency -preset ultrafast -f flv rtmp://{omehost}/app/{name} ; sleep 2; done'
    return command
    
def main_unit(ns):
    command = f"new-session -n MISC -d -s {ns} 'exec bash'"
    tmux(command)

    ros2("tst_to_fifo", f'ros2 run lrs_util to_fifo --ros-args -r __name:=to_fifo_tst -p image:={ns}/tststream_exec/image_raw -p compressed:=true -p fifo:=/tmp/fifo_tst  -p scale:=false -p notext:=true')
    command = ffmpeg_command(f'{ns.lstrip("/")}_tst', f'/tmp/fifo_tst', resolution="1280x720", omehost=options.omehost)
    # command = ffmpeg_command(f'{ns.lstrip("/")}_tst', f'/tmp/fifo_tst', omehost=options.omehost)
    ros2(f'ffmpeg_tst', f'sleep 15; {command}')
    

    if options.attach:
        tmux(f'attach-session -t {ns}')
    
if __name__ == "__main__":

    if options.ns != "/dummy":
        ns = options.ns
    else:
        ns = f'/{os.environ["AGENT_NAME"].split("_")[-1]}'

    print("NS:", ns)
    
    main_unit(ns)   
 
        
